% [G] = pathmatrix(nodes, adj, A)

function [G] = pathmatrix(nodes, adj, A)

% -- Build the path matrix

disp('Build path matrix');
ne = full(max(max(A)));
nv = length(nodes);
V = [];
node_use = zeros(length(A),1);
for i = 1:nv-1
  [d, p] = dijkstra(adj, nodes(i));
  V      = [V, pathv(nodes(i+1:nv), p, A, ne, node_use)];
end

% -- Eliminate unused edges

disp('Eliminate unused edges');
G          = V';
used_edges = find(sum(G) > 0);
G          = G(:,used_edges);

% -- Eliminate unused nodes and reduce A

disp('Eliminate unused nodes');
used_nodes = find(node_use);
A          = A(used_nodes, used_nodes);

% -- Eliminate unused edges from reduced A

disp('Eliminate unused edges from reduced A');
[i,j,Aij]   = find(A);
I           = find(ismember(Aij, used_edges));
A           = sparse(i(I), j(I), Aij(I));

% -- Find all internal nodes in the graph

disp('Find internal nodes');
[C,Iinternal] = setdiff(used_nodes, nodes);
fprintf('%f %% internal\n', 100 * length(Iinternal) / length(used_nodes));

% -- Find all internal (undirected) degree 2 nodes in the graph

disp('Find internal undirected degree 2 nodes');
node_degree = sum(A > 0);
I           = find(node_degree(Iinternal) <= 4);
Iinternal2  = Iinternal(I);
fprintf('%f %% degree 2\n', 100 * length(Iinternal2) / length(used_nodes));

% -- For all degree 2 internal nodes, kill the edge with smaller ID.
%    This leaves only one representative for each "line" in the reduced net

disp('Eliminate lines in the reduced network');
edge_use = ones(length(used_edges),1);
edge_use( max(A(:,Iinternal2)) ) = 0;
Iused      = find(edge_use);
used_edges = used_edges(Iused);
G          = G(:,Iused);

