% [G, R] = rowselect(G)

function [G, R, selected] = rowselect(G)

% --- Form transposed G matrix and reorder columns

V1 = G';
P  = colamd(V1);
V1 = V1(:,P);

% --- Loop through G in blocks of 100 and do block
%     Gram-Schmidt + column pivoting

V = [];
R = zeros(1000);

k       = 0;
nblocks = ceil(size(V1,2) / 100);
t0 = clock;
for j = 1:nblocks

  % -- Grab a block of 100
  fprintf('Processing block %d / %d (%f s): %d of %d used\n', ...
          j, nblocks, etime(clock, t0), k, (j-1)*100 );

  if j == nblocks
    Vi = V1;
  else
    Vi = V1(:,1:100);
    V1 = V1(:,101:end);
  end

  % -- Resize Cholesky store if needed
  if k + 100 >= length(R)
    fprintf('%d looks too small, resizing\n', length(R));
    Rold       = R;
    R          = zeros( k * 1.5 );
    R(1:k,1:k) = Rold(1:k,1:k);
    clear Rold;
  end

  % -- Update path record
  e = addpaths(V, R, Vi, 1e-5);
  k = k + length(e);
  V = [V, Vi(:,e)];

end

G = V';
R = R(1:k,1:k);
