/**
 *OBJECT:	This class is a "front end" for use in running batch jobs
 *
 * @version    $Id: BatchEnabler.java,v 1.1 10/11/2002
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: BatchEnabler.java,v 1.1$
 *     Revision 1.1  10/11/2002
 *     Initial revision
 *
 */


// // package LossyLink;

import java.util.*;

public class BatchEnabler {

	public static void main( String args[] ) {

		/** total number of nodes for non-Euclidian graph
		 *  n dimension of n x n grid for Euclidian graph */
  	int NODES = Integer.parseInt(args[0]);

  	/** the probabilty of an edge existing from one node to another
  		* edges restricted to adjacent nodes in Euclidian graph */
  	double PROB = Double.parseDouble(args[1]) / 100;

		/** minimum value for an edge weight */
		double WEIGHTLOW = Double.parseDouble(args[2]);

		/** maximum value for an edge weight */
  	double WEIGHTHIGH = Double.parseDouble(args[3]);

  	/** number used as a seed in random number generator */
  	long SEED = 1;

  	/** should the generated graph be a Euclidian graph */
  	boolean EUCLID = Boolean.valueOf(args[4]).booleanValue();

  	/** how many different algorithms or algorithm variations
  	 *	 will be tested - if all algorihtms are used there are 14 */
  	int NUMALGS = 7;

  	/** number of times each algorithm is run (for averaging) */
  	int NUM_TIMES = Integer.parseInt(args[5]);

  	/** if true, integer weights only */
		boolean INT_ONLY = Boolean.valueOf(args[6]).booleanValue();

		/** number of nodes to be solved for in the "many pairs" problem */
		int NUM_NODES_AS_SOURCE  = Integer.parseInt(args[7]);


		ShortestPath allAlgs [][] = new ShortestPath[NUMALGS][NODES + 1];
		String arg[][] = new String[NUMALGS][2];

		//Dijkstra with linear array priority queue
		arg[0][0] = "1";
		arg[0][1] = "l";
		//Dijkstra with binary heap priority queue
		arg[1][0] = "1";
		arg[1][1] = "b";
		//Dijkstra with fibinacci heap priority queue
		arg[2][0] = "1";
		arg[2][1] = "f";
		//Belman Ford
		arg[3][0] = "2";
		//Johnson with linear array priority queue
		arg[4][0] = "3";
		arg[4][1] = "l";
		//Johnson with binary heap priority queue
		arg[5][0] = "3";
		arg[5][1] = "b";
		//Johnson with fibinacci heap priority queue
		arg[6][0] = "3";
		arg[6][1] = "f";


		if( !EUCLID ) {

			Node testGraph [] = Graph.genRandGraph(
			  	NODES, PROB, WEIGHTLOW, WEIGHTHIGH );
			//Node testGraph [] = Graph.genRandGraphMaxOrMinOnly(
			//  NODES, PROB, WEIGHTLOW, WEIGHTHIGH );

			double[][] graphAM = Graph.convertToMatrix( testGraph );

		FloydWarshall fw = null;
		//run all algorithms on the same test graph
		for( int count = 0; count < NUM_TIMES; count++ ) {
			for( int i = 0; i < NUMALGS; i++ ) {
				if( i < 4 )
					for( int j = 1; j <= NODES; j++ )
						allAlgs[i][j] = Graph.runShortestPath( arg[i], testGraph, j, EUCLID );
				else
					allAlgs[i] = Graph.runJohnson( arg[i], testGraph, EUCLID );
			}
			fw = APSP.floydWarshall( graphAM );
		}
		//check to see that shortest path weights match for all algorithms
		for( int i = 1; i < NUMALGS; i++ ){
			System.out.println("--" + i + "--");
			for( int j = 1; j <= NODES; j++ )
				if( !allAlgs[0][j].equals( allAlgs[i][j] ) )
					System.out.println( "source: " + j + "\n" +
						allAlgs[0][j] + "\n\n" + allAlgs[i][j] );
		}
		//check Floyd-Warshall
		for( int i = 1; i <= NODES; i++ )
			for( int j = 1; j <= NODES; j++ )
				if( allAlgs[0][i].S[j] != fw.D[i][j] )
					System.out.println( "source: " + i + "\nnode: " +
						j + "\n" + allAlgs[0][i].S[j] + "\n\n" + fw.D[i][j] );
		}
		else {

			EuclidNode testGraph [] = Graph.genRandEuclid( NODES, PROB, NUM_NODES_AS_SOURCE );

	    double[][] graphAM = Graph.convertToMatrix( testGraph );

			FloydWarshall fw = null;
			//run all algorithms on the same test graph
			for( int count = 0; count < NUM_TIMES; count++ ) {
			for( int i = 0; i < NUMALGS; i++ ) {
				if( i < 4 )
					for( int j = 1; j <= NODES; j++ )
						allAlgs[i][j] = Graph.runShortestPath( arg[i], testGraph, j, EUCLID );
				else
					allAlgs[i] = Graph.runJohnson( arg[i], testGraph, EUCLID );
			}
			fw = APSP.floydWarshall( graphAM );

			APSP.linearAStar( testGraph, NODES );
			APSP.binaryAStar( testGraph, NODES );
			APSP.fibAStar( testGraph, NODES );
			}
			//check to see that shortest path weights match for all algorithms
			for( int i = 1; i < NUMALGS; i++ ){
				System.out.println("--" + i + "--");
				for( int j = 1; j <= NODES; j++ )
					if( !allAlgs[0][j].equals( allAlgs[i][j] ) )
						System.out.println( "source: " + j + "\n" +
							allAlgs[0][j] + "\n\n" + allAlgs[i][j] );
			}
			//check Floyd-Warshall
			for( int i = 1; i <= NODES; i++ )
				for( int j = 1; j <= NODES; j++ )
					if( allAlgs[0][i].S[j] != fw.D[i][j] )
						System.out.println( "source: " + i + "\nnode: " +
							j + "\n" + allAlgs[0][i].S[j] + "\n\n" + fw.D[i][j] );

		}
	}
}