/**
 *OBJECT:	This class computes the shortest path
 *				from a source node to all other nodes in
 *				a directed graph.  The algorithm used
 *				is the Bellman-Ford method.
 *
 * @version    $Id: BellmanFord.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: BellmanFord.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;


public class BellmanFord extends ShortestPath {

    int start = 1;  //number of smallest vertex id

    //array of upper bounds on the
    //weight of a shortest path from source s to index of d
    double [] d ;

    /**
     * Constructor serves to instantiate all needed arrays and run the
     * algorithm to completion, filling the S array with the final
     * shortest path weights and the pi array with the vertices' predecessors.
     * The start parameter is needed, because when the Johnson algorithm
     * uses Bellman-Ford there is a zero vertex.
     *
     * @param graph the graph in adjacency list form
     * @param source the vertex which all paths will eminate from
     * @param start the number of the smallest vertex id
     */
    public BellmanFord( Node [] graph, int source, int start ) {
    	super( graph );
    	this.start = start;
    	initializeSingleSource( graph, source );
    	for( int i = start; i < graph.length - 1; i++ )
    	    for( int j = start; j < graph.length; j++ )
    	        for( Node cursor = graph[j]; cursor.next != null;
    	             cursor = cursor.next )
    	            relax( graph[j].id, cursor.next.id,
    	                   cursor.edgeWeight );

    	for( int j = start; j < graph.length; j++ )
    	    for( Node cursor = graph[j]; cursor.next != null;
    	         cursor = cursor.next )
    	        if( d[cursor.next.id] > d[graph[j].id] +
    	            cursor.edgeWeight ) {
    	            System.out.println("Negative cycle found!");
    	            System.exit(0);
    	        }
    	S = d;
    }


    /**dummy method -- not used for Belman-Ford*/
     public int extractMin( ) {return -1;}


    /**
     * Initialize the shortest-path estimates and predecessor function.
     * The predecessor function is left at the default of all zeros.
     * Because there is no zero node in the graph, zero denotes a null
     * value for a predecessor.
     *
     * @param graph the input graph in adjacency-list form
     * @param source the number identifier of the source node
     */
    public void initializeSingleSource( Node [] graph, int source ) {
        //System.out.println( "initializeSingleSource" );
        d = new double[graph.length];
        for( int i = 1; i < graph.length; i++ )
            d[i] = Double.MAX_VALUE;
        d[source] = 0;
    }

    /**
     * Test if shortest path to v can be improved by going through u, and
     * if so, updating d[v] and pi[v]. w is the weight from u to v.
     *
     * @param u vertex relaxing from
     * @param v vertex relxing to
     * @param w the weight from u to v
     */
    public void relax( int u, int v, double w ) {
    	//System.out.println( "relaxing" + u + v + w );
    	if( d[v] > d[u] + w ) {
    	    d[v] = d[u] + w;
    	    pi[v] = u;
    	    //System.out.println( "updating d" + d[v] );
    	    //try {
    	    //     SimpleInput.readInt();} catch(Exception e){}
    	}
    }

}

