/**
 *OBJECT:	This class is used
 *				for the Dijkstra algorithm with a Fibonacci heap
 *				priority queue.  It defines the atributes of
 *				a node in a Fibonacci heap.  Also provided is a
 *				method to print out a Fibonacci heap.
 *
 * @version    $Id: FibNode.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: FibNode.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;


public class FibNode extends HeapNode {

  FibNode left, right, parent, child;
  int degree;
  boolean mark;

	/**
	 * The constructor initializes a node in the
	 * Fibonacci heap.
	 *
	 * @param id the identification number of the new node
   * @parma value the current shortest path estimate
   */
    public FibNode( int id, double value ) {
        super( id, value );
        left = right = this;
    }

    /**
    * This method is used to walk through the root list to print out the entire
    * tree of each root using recursive calls.  An individual call to printHeap
    * causes the node that called it to be printed out, printHeap is then called
    * on its child, and finally, one at a time, each sibling is printed out and
    * printHeap is called on the sibling's children.
    */
    public void printHeap( ) {
        if( parent != null )
            System.out.print( "child of " + parent + "-> " );
        System.out.print( toString() + " " );
        if( child != null )
            child.printHeap();
        System.out.println();
        for( FibNode cursor = left; cursor != this; cursor = cursor.left ) {
            if( cursor.parent != null )
                System.out.print( "child of " + cursor.parent + "-> " );
            System.out.print( cursor + " ");
            if( cursor.child != null )
                cursor.child.printHeap();
        }
        System.out.println();
    }

}
