/**
 *OBJECT:	This class computes the shortest path
 *				from a all nodes to all other nodes in
 *				a directed graph.  The algorithm is used
 *				is the Floyd-Warshall algorithm.
 *
 * @version    $Id: FloydWarshall.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: FloydWarshall.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;


public class FloydWarshall {

	double[][] D;	//graph in adjacency matrix form

	int[][] pi; //predecessors in matrix form

	/**
    * Constructor serves to instantiate all needed arrays and run the
    * algorithm to completion, filling the matrix D with the final
    * shortest path weights and the matrix pi with the vertices' predecessors.
    *
    * @param graph the graph in adjacency list form
    * @param type l for linear array priority queue
    *					b for binary heap priority queue
    *					f for fibonacci heap priority queue
    */
	public FloydWarshall( double[][] graphAM ) {
		//initialize D matrix
		D = graphAM;

		//for(double i = 0;i< 100;i++)
		//;

		//initialize pi matrix
		pi = new int[D.length][D.length];
		for( int i = 1; i < D.length; i++ )
			for( int j = 1; j < D.length; j++ )
				if( i != j && D[i][j] < Double.MAX_VALUE )
					pi[i][j] = i;
				else
					pi[i][j] = -1; //-1 indicates a null value

		//run algorithm
		for( int k = 1; k < D.length; k++ )
			for( int i = 1; i < D.length; i++ )
				for( int j = 1; j < D.length; j++ )
					if( D[i][j] > D[i][k] + D[k][j] ) {
						D[i][j] = D[i][k] + D[k][j];
						pi[i][j] = pi[k][j];
					}
	}
}