/**
 *OBJECT:	This class computes the shortest path
 *				from a all nodes to all other nodes in
 *				a directed graph.  The algorithm is used
 *				is the Johnson algorithm with either a
 *				linear array, Binary heap, or Fibonacci heap
 *				priority queue.
 *
 * @version    $Id: Johnson.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: Johnson.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;


public class Johnson{

    //array of upper bounds on the
    //weight of a shortest path from source s to index of d
    double [] d ;

    //shortest path results after running Johnson
    ShortestPath [] allPairs;

    /**
    * Constructor serves to instantiate all needed arrays and run the
    * algorithm to completion, filling the ShortestPath objects with the
    * final shortest path weights and the vertices' predecessors.
    *
    * @param graph the graph in adjacency list form
    * @param type l for linear array priority queue
    *					b for binary heap priority queue
    *					f for fibonacci heap priority queue
    */
    public Johnson( Node [] graph, char type ) {
      this( graph, type, null, null );
    }


    public Johnson( Node [] graph, char type, LossyLinkPathInfo pathInfo, LossyLinkData data ) {
        //run Bellman-Ford with added zero vertex as the source
        BellmanFord bf = new BellmanFord( graph, 0, 0 );
        d = bf.d;
        //adjust weights to be all positive
        for( int j = 0; j < graph.length; j++ )
            for( Node cursor = graph[j]; cursor.next != null;
                 cursor = cursor.next )
                cursor.edgeWeight +=
                    d[graph[j].id] - d[cursor.next.id];
        //Graph.printGraph( graph );
    //run Dijkstra |V| times
        allPairs = new ShortestPath[graph.length];

        switch( type ) {
        case 'l':
            for( int i = 1; i < graph.length; i++ )
		{
		    allPairs[i] = new LinearDijkstra( graph, i, pathInfo, data );
		}
            break;
        case 'b':
            for( int i = 1; i < graph.length; i++ )
                allPairs[i] = new BinaryDijkstra( graph, i );
            break;
        case 'f':
            for( int i = 1; i < graph.length; i++ )
                allPairs[i] = new FibDijkstra( graph, i );
            break;
        default:
            System.out.println( "error," + type +
                                " not a known priority queue type");
            System.exit(0);
        }

        //adjust shortest path weights to compensate for
        //changing negative weights to positive ones before
        //Dijkstra or A* was run
        for( int i = 1; i < graph.length; i++ )
            for( int j = 1; j < graph.length; j++ )
                allPairs[i].S[j] += d[j] - d[i];

        //print final results
//        for( int i = 1; i < graph.length; i++ )
//            System.out.println( allPairs[i] );

    }
}



