/**
 *OBJECT:	This class computes the shortest path
 *				from a source node to a goal node in
 *				a Euclidian graph.  The algorithm is used
 *				is the A* method with a linear array heap
 *				priority queue.
 *
 * @version    $Id: LinearAStar.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: LinearAStar.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;


public class LinearAStar extends LinearDijkstra {

	double [] goalEstimates;

	/**
    * Constructor serves to instantiate all needed arrays and run the
    * algorithm to completion, filling the S array with the final
    * shortest path weights and the pi array with the vertices' predecessors.
    * The constructor runs the A* algorithm until either the
	 * goal node is found or all nodes connected to the source
	 * have been searched.
    *
    * @param graph the graph in adjacency list form
    * @param source the vertex which the shortest path will eminate from
    * @param goal the id number of the goal node
    */
	public LinearAStar( EuclidNode [] graph, int source, int goal ) {
		super( graph );
		//System.out.println("source = " + source + " goal = " + goal);
		initializeSingleSource( graph, source );
      goalEstimates = new double[graph.length];

      int min = extractMin( );
        while( min != goal ) {
        		if( min == -1 ) {
        			System.out.println( "Goal not reachable from source." );
        			break;
        		}
            S[min] = minWeight;
            for( EuclidNode curNode = graph[min]; curNode.next != null;
                 curNode = (EuclidNode)curNode.next ) {
                //System.out.print(curNode.id + " ");

                if( goalEstimates[curNode.next.id] == 0 )
                	goalEstimates[curNode.next.id] =
                		( (EuclidNode)curNode.next ).goalEst( graph[goal] );

                relax( graph[min].id, curNode.next.id, curNode.edgeWeight );
            }
            //System.out.println();
            d[min] = -1;  //"remove" min from d
            min = extractMin( );
        }
        S[min] = minWeight;
        d = null;
	}

	/**
	 * This method removes the minimum valued node form the shortest path
	 * estimate linear array priority queue.
	 *
	 * @return the node number with the minimum shortest path estimate
	 */
    public int extractMin( ) {
        minWeight = Double.MAX_VALUE;
        int minIndex = -1;
        for( int i = 1; i < d.length; i++ ) {
            if( d[i] != -1 &&
            	( d[i] + goalEstimates[i] ) < minWeight ) {
                minWeight = d[i];
                minIndex = i;
            }
            //System.out.println( d[i] );
        }
        if( minIndex != -1 )
            minWeight = d[minIndex];
        //System.out.println( "new min is:" + minIndex );
        return minIndex;
    }


}