/**
 *OBJECT:	This class computes the shortest path
 *				from a source node to all other nodes in
 *				a directed graph.  The algorithm is used
 *				is the Dijkstra method with a linear array
 *				priority queue.
 *
 * @version    $Id: LinearDijkstra.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: LinearDijkstra.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;



import java.util.*;

public class LinearDijkstra extends ShortestPath{

    //array of upper bounds on the
    //weight of a shortest path from source s to index of d
    double [] d ;
    int iLinkList[];
    int [] iLinkCount ;
    int iPathIndex;
    int iLinkIndex;
    int iSource;
    int iNode[];

    LossyLinkPathInfo m_pathInfo;
    LossyLinkData m_data;

    public LinearDijkstra( Node [] graph )
    {
        super( graph );
    }

    public LinearDijkstra( Node [] graph, int source )
    {
	this( graph, source, null, null );
    }

    public LinearDijkstra( Node [] graph, int source, LossyLinkPathInfo pathInfo, LossyLinkData data )
    {
	this( graph );

        iNode = new int[2];
        iNode[0] = -1;
        iNode[1] = -1;
        initializeSingleSource( graph, source );
        iLinkIndex = 0;
        iSource = source;
        iLinkList = new int[graph.length];

        m_pathInfo = pathInfo;
        m_data = data;

        for( int a = 0; a < d.length -1; ++a )
        {
	    if (a == iSource - 1)
	    {
		continue;
	    }

	    //HHSONG
	    iPathIndex = m_pathInfo.newLinkInPath( 0 );
            iNode[0] = -1;
            iNode[1] = -1;
            //System.out.println( "PathInfo : Src : " + (source - 1) + " Dst : " + a + " iPathIndex : " + iPathIndex );	    }
        }

        int i = 0;
        int min = extractMin( );
        while( min != -1 )
	{
            S[min] = minWeight;
	    L[min] = iLinkCount[min];

            if( m_pathInfo !=  null )
            {

		//HHSONG
		iPathIndex = ( iSource - 1 ) * ( d.length - 1 ) - ( iSource - 1 ) + min-1;
		if (iSource <= min) {
		    iPathIndex--;
		}
                iNode[0] = -1;
                iNode[1] = -1;
		if (iSource != min)
		{
/*			System.out.println("my path index : pathIndex: " + iPathIndex +
				       " iSrc: " + (iSource - 1) +
				       " d.len : " + (d.length - 1) +
				       " min : " + (min - 1));
*/

                        setRankElement(min);

/*			System.out.println("my link count : pathIndex: " + iPathIndex +
				       " iLinkCount[min]: " +
				       iLinkCount[min] + " min: " + (min-1));
*/
                }
            }

            for( Node cursor = graph[min]; cursor.next != null; cursor = cursor.next )
		{
                    iLinkIndex = cursor.iLinkIndex;
		    relax( graph[min].id, cursor.next.id, cursor.edgeWeight );
		}
            d[min] = -1;  //"remove" min from d
	    iLinkCount[min] = 0;

            min = extractMin( );
        }

        d = null;
	iLinkCount = null;
    }


    private int setRankElement( int iNodeIndex )
    {
      if( iNodeIndex == iSource )
      {
        return 1;
      }



      LossyLinkPathData pathData = (LossyLinkPathData)m_pathInfo.pathList[iPathIndex];
      if( pathData.linkIndexList.length == 0 )
      {
//          iNode[0] = pathData.getDestHost();
          iNode[1] = iLinkList[iNodeIndex] - 1;
      }
      else
      {
          if( iNode[0] == -1 || iNode[1] == -1 )
          {
              System.out.println("ERROR 00055" );
          }
          iNode[0] = iNode[1];
          iNode[1] = iLinkList[iNodeIndex] - 1;
      }

      int iLinkIndex = -1;
      if( ( iLinkIndex = m_data.link.linkIndex.nodeIndex2linkIndex( iNode ) ) == -1 )
      {
          int iReverseNode[] = new int[2];
          iReverseNode[1] = iNode[0];
          iReverseNode[0] = iNode[1];
          if( ( iLinkIndex = m_data.link.linkIndex.nodeIndex2linkIndex( iReverseNode ) ) == -1 )
          {
              System.out.println("ERROR 00056" );
          }
      }
      m_pathInfo.addLinkInPath( iPathIndex, iLinkIndex );
//      m_pathInfo.setRankData( iPathIndex, iLinkList[iNodeIndex] - 1 );

/*	    System.out.println("pathInfo.setRankData iPathIndex : " +
                           iPathIndex + " iLinkIndex : " +
                           (iLinkList[iNodeIndex] - 1) + " iNodeIndex : " + (iNodeIndex - 1)); // + " u: " + u + " d[u] : " + d[u] + " v: " + v + " d[v] : " + d[v] );
*/
      setRankElement( iLinkList[iNodeIndex] );
      return 1;
    }

    /**
	 * This method removes the minimum valued node form the shortest path
	 * estimate linear array priority queue.
	 *
	 * @return the node number with the minimum shortest path estimate
	 */
    public int extractMin() {
        minWeight = Double.MAX_VALUE;
        int minIndex = -1;
        for( int i = 1; i < d.length; i++ ) {
            if( d[i] != -1 && d[i] < minWeight ) {
                minWeight = d[i];
                minIndex = i;
            }
        }
        if( minIndex != -1 )
	    {
		minWeight = d[minIndex];
	    }
        return minIndex;
    }


    /**
     * Initialize the shortest-path estimates and predecessor function.
     * The predecessor function is left at the default of all zeros.
     * Because there is no zero node in the graph, zero denotes a null
     * value for a predecessor.
     *
     * @param graph the input graph in adjacency-list form
     * @param source the number identifier of the source node
     */
    public void initializeSingleSource( Node [] graph, int source ) {
        //System.out.println( "initializeSingleSource" );
        d = new double[graph.length];
        for( int i = 1; i < graph.length; i++ )
            d[i] = Double.MAX_VALUE;
        d[source] = 0;

	iLinkCount = new int[graph.length];
    }

    /**
     * Test if shortest path to v can be improved by going through u, and
     * if so, updating d[v] and pi[v]. w is the weight from u to v.
     *
     * @param u vertex relaxing from
     * @param v vertex relxing to
     * @param w the weight from u to v
     */
    public void relax( int u, int v, double w ) {
        if( d[v] > d[u] + w )
	{
            d[v] = d[u] + w;
	    if( w != 0 )
		{
		    //HHSONG
//		    System.out.println("iLinkList : idx : " + v + " iLinkList[idx] : " + iLinkList[v] );
		    iLinkList[v] = u;

		    iLinkCount[v] = iLinkCount[u] + 1;
		}
	    pi[v] = u;
            //try {
            //     SimpleInput.readInt();} catch(Exception e){}
        }
    }
}
