/**
 * <p>Title: LossyLink</p>
 * <p>Description: The <CODE>LossyLink</CODE> class is used to compute the lossy links,
 * given the input of e2e loss and path.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;


import java.io.*;
import java.util.*;
import cluster.*;

public class LossyLink implements Serializable, WNMMSConstants
{

    static String srcDate;
    static PrintWriter errorStream;

    static String baseFilename;
    static HostMap hostMap;
    static int hosts;
    static String root;

    static LossyLinkData data;
    static boolean bReduceIP = false;

    static LossyLinkPathInfo pathInfo = new LossyLinkPathInfo();

    public static void main(String[] args)
    {

        data = new LossyLinkData();

        int[] iData = new int[2];

	if(args.length < 4 || args.length > 5)
	    {
		showUsage();
		return;
	    }

	if( args.length == 5 )
	    {
		if( args[4].compareToIgnoreCase( "-r" ) == 0 )
		    {
			bReduceIP = true;
		    }
		else
		    {
			showUsage();
			return;
		    }
	    }

	if( initialize( args[0], Integer.parseInt( args[1] ), args[2], args[3], bReduceIP ) == 0 )
	    {
		System.err.println( "ERROR : main() !Initialize()" );
		return;
	    }

	readTopology( errorStream, data );

	data.printCount();

    }

    private static void showUsage()
    {
	System.err.println("\nUsage: \njava LossyLink " +
			   "<date> <number of hosts> <sitesfile> <root server name> [OPTION]\n"+
			   "\ni.e. java LossyLink 102.11.1 111 newsites.lst amp-alaska -r" +
			   "\n\nOption: \n\t-r\tReduce IP address length to 29 bit instead of 32." );
    }

    public static int initialize( String strDateInput, int iHostsInput, String sitesFileInput, String rootInput, boolean bReduceIPInput )
    {
	srcDate = strDateInput;
	hosts = iHostsInput;
	root = rootInput;	// root server name, e.g., amp-alaska
	baseFilename = ".." + SLASH + "data" + SLASH + "raw-data" + SLASH + srcDate + SLASH;
   	hostMap = new HostMap(sitesFileInput, hosts);
	bReduceIP = bReduceIPInput;

	return 1;
    }







    /* Checks input string whether it is IP address string, floating point number, or just a star(*)
     *
     * Return Values :
     * 2 for a floating point number
     * 1 for a IP address string
     * 0 for a *
     * -1 for others
     */
    private static int CheckDataType( String strInput )
    {
	//	LossyLinkDebug.print( "CheckDataType : " + strInput );

	if( strInput.compareTo("*") == 0 )
	    {
		//a star(*)
		//		LossyLinkDebug.println("       a star" );
		return 0;
	    }

	int iTmp = 0;
	int i = 0;

	for( i = 0; i < 3; i++ )
	    {
		if( ( iTmp = strInput.indexOf( '.', iTmp + 1 ) ) == -1 )
		    {
			break;
		    }
	    }

	if( i == 0 )
	    {
		//a string which doesn't contain a dot(.)
		//		LossyLinkDebug.println("     no category");
		return -1;
	    }
	else if( i == 1 )
	    {
		//a string with only one dot(.)
		//		LossyLinkDebug.println("     fp");
		return 1;
	    }
	else if( i == 3 )
	    {
		//the rest is a string which has multiple dots(~~~.~~~.~~~.~~~);
		//		LossyLinkDebug.println("     IP");
		return 2;
	    }

	System.err.println("     a string with 2 dots???");
	return -1;
    }



    static String reduceIP( String strIP )
    {
	int iLastDotIndex = 0;

	if( ( iLastDotIndex = strIP.lastIndexOf( "." ) ) == -1 )
	    {
		System.out.println( "ERROR LL 00200 : ReduceIP(), No dot found in a IP addr" );
		return null;
	    }
	int iLastByte = Integer.parseInt( strIP.substring(iLastDotIndex+1) );
	iLastByte = iLastByte >> 3;

        return new String( strIP.substring(0, iLastDotIndex+1) + Integer.toString( iLastByte ));
    }

    /* Checks if strInput indicates a new set of data
     *
     * Return values
     * 0 : A string that doesn't begin with a mixture of alphabet, number, and/or other character  has been found.
     * 1 : A string that begins with a number has been found.
     * 2 : A string that begins with alphabet has been found.
     */
    static int CheckCharacterType( String strInput )
    {
	boolean bLetter = false;
	boolean bDigit = false;

       	//debug codes
	//      	LossyLinkDebug.println( "CheckCharacterType() : strInput : " + strInput );

	for( int i = 0; i < strInput.length(); i++ )
	    {
		if( Character.isDigit( strInput.charAt(i) ) )
		    {
			if( bDigit == false )
			    bDigit = true;
		    }
		else if( Character.isLetter( strInput.charAt(i) ) )
		    {
			if( bLetter == false )
			    bLetter = true;
		    }
	    }

	if( bDigit == true  && bLetter == true )
	    {
		return 0;
	    }
	else if( bDigit == true && bLetter == false )
	    {
		return 1;
	    }
	else if( bDigit == false && bLetter == true )
	    {
		return 2;
	    }
	else
	    {
		return 0;
	    }
    }






	/**
	 * 	Read in the topology from a single root.
	 **/


    public static int readTopology( PrintWriter errorStream, LossyLinkData data )
    {
	return readTopology( errorStream, data, false );
    }

    public static int readTopology( PrintWriter errorStream, LossyLinkData data, boolean bTest )
    {

	String traceFile;
	String traceLine;
	int iNode[] = new int[2];                //NodeID( actually an IP address) of one node of an link,
                                                 //and nodeID of the other node of the link.
	String tmpStr = new String();
	int iDataSetCount = 0;                   //Counts number of data set in a host
	int iTmp = 0;
	int iPathPerHost = 0;

	//loop for every hosts.
	if( bTest == true )
	    {
		hosts = 5;
	    }

       	for(int j=0; j < hosts; j++)
	    {	// vertically set up the values

		//initialization
      		Host h_j = hostMap.getHost(j);
       		String h_jName = h_j.getName();
	        iDataSetCount = 0;

       		traceFile = baseFilename + root + MIDDLE + h_j.getName() + SLASH + srcDate+ ".trace";

		//Skip test for 'root to root' trace data.
       		if( h_jName.compareTo( root ) == 0 )
       		    {
       			LossyLinkDebug.println( h_jName );
       			continue;
       		    }

/*for debugging purpose

		if( (h_jName.compareToIgnoreCase( "amp-test" ) != 0) && h_jName.compareToIgnoreCase( "amp-test2" ) != 0 )
		    {
			continue;
		    }
*/


		//debug info
		LossyLinkDebug.println( "DestHost : " + h_jName + " [" + j + "]" );

       		try {
       		    BufferedReader traceReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(traceFile))));

		    //loop every row in host's data set
		    while ((traceLine = traceReader.readLine()) != null )
			{
			    StringTokenizer st = new StringTokenizer( traceLine, "\t\n\r\fms " );
			    if( st.hasMoreTokens() == true )
				{
				    tmpStr = st.nextToken();

				    //Check if this line is the beginning of a new set of data.
				    if( CheckCharacterType( tmpStr ) == 2 )
					{
					    //A new set of data begins
					    //skip this line('date and time') from tokenizing
					    iDataSetCount++;
					    iNode[0] = -1;
					    iNode[1] = -1;

					    //					    LossyLinkDebug.println( "Data set " + iDataSetCount + "....." );

					    continue;
					}

  	       			    //For now, test only the first data set per host.
				    if( iDataSetCount > 1 )
			       	        break;

				}

			    //loop for every token
	       		    for( int k = 0; st.hasMoreTokens(); k++ )
	       			{
				    //For now, fetch the IP address(Name of the link) only.
				    tmpStr = st.nextToken();
				    iTmp = CheckDataType( tmpStr );

				    //IP address
				    if( iTmp == 2  )
					{
					    if( iNode[0] == -1 && iNode[1] == -1 )//2
						{
       						    iNode[0] = data.node.set( tmpStr, -1, -1, 0, 0, bReduceIP );
						}
					    else if( iNode[0] != -1 && iNode[1] == -1 )//1
						{
						    iNode[1] = data.node.set( tmpStr, -1, -1, 0, 0, bReduceIP );
						    data.link.set( iNode, 1.0, -1.0, false );
						    //Initialize floating pt slot whenever we store link data.
						}
					    else if( iNode[0] != -1 && iNode[1] != -1 )//1,2
						{
						    iNode[0] = iNode[1];
						    iNode[1] = data.node.set( tmpStr, -1, -1, 0, 0, bReduceIP );
						    data.link.set( iNode, 1.0, -1.0, false );
						    //Reinitialize floating pt slot whenever we store link data.
						}
					    else if( iNode[0] == -1 && iNode[1] != -1 )//1.2
						{
						    System.err.println( "ERROR : ReadTopology() DataCorruption");
						}
					}
				    else if( iTmp == 1 )//Floating pt number
					{
					    //not implemented yet
					}
				    else
					{
					    //just skip this token.
					}

				}//end for
			    //			    LossyLinkDebug.println("");
			} //end while

  	       		traceReader.close();
			//count link per path
			iPathPerHost++;

		} catch(IOException e) {
		    //errorStream.println("File format exception.\n" + e.getMessage());
		    System.err.println("File format exception.\n" + e.toString());
			//			System.exit(0);
       		}
       	}//end for( j; hosts; )

       	return iPathPerHost;
    }










    public static void compareTopology( PrintWriter errorStream, LossyLinkData data, LossyLinkPathInfo pathInfo, int iSrcHostIndex )
    {
	compareTopology( errorStream, data, pathInfo, iSrcHostIndex, false );
    }

    public static void compareTopology( PrintWriter errorStream, LossyLinkData data, LossyLinkPathInfo pathInfo, int iSrcHostIndex, boolean bTest )
    {

	String traceFile;
	String traceLine;

	int iNode[] = new int[2];                //NodeID( actually an IP address) of one node of a link.
	String strToken = new String();
	int iDataSetCount = 0;                   //Counts number of data set in a host
	int iTokenType = 0;
      	int k = 0;
        LinkedList nodeSetList = new LinkedList();
        int iNodeSetListLen = 0;
	int iPathListIndex = 0;
	int iLinkIndex = -1;

	//loop for every hosts.
	if( bTest == true )
	    {
		hosts = 5;
	    }

       	for(int j=0; j < hosts; j++)
	    {	// vertically set up the values

		//initialization
      		Host h_j = hostMap.getHost(j);
       		String h_jName = h_j.getName();
	        iDataSetCount = 0;

       		traceFile = baseFilename + root + MIDDLE + h_j.getName() + SLASH + srcDate+ ".trace";

		//Skip test for 'root to root' trace data.
       		if( h_jName.compareTo( root ) == 0 )
       		    {
       			LossyLinkDebug.println( h_jName );
       			continue;
       		    }

/*for debugging purpose

		if( (h_jName.compareToIgnoreCase( "amp-test" ) != 0) && h_jName.compareToIgnoreCase( "amp-test2" ) != 0 )
		    {
			continue;
		    }
*/
		//debug info
		LossyLinkDebug.println( "DestHost : " + h_jName + " [" + j + "]" );

       		try {
       		    BufferedReader traceReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(traceFile))));


		    //loop every row in host's data set
		    while ( (traceLine = traceReader.readLine()) != null )
			{
			    StringTokenizer st = new StringTokenizer( traceLine, "\t\n\r\fms " );
			    if( st.hasMoreTokens() == true )
				{
				    strToken = st.nextToken();

				    //Check if this line is the beginning of a new set of data.
				    if( CheckCharacterType( strToken ) == 2 )
					{
					    //A new set of data begins
					    //skip this line('date and time') from tokenizing
					    iDataSetCount++;
					    continue;
					}

  	       			    //For now, test only the first data set per host.
				    if( iDataSetCount > 1 )
			       	        break;
				}

			    //loop for every token
	       		    for( k = 0; st.hasMoreTokens(); k++ )
	       			{
				    //For now, fetch the IP address(Name of the link) only.
				    strToken = st.nextToken();
				    iTokenType = CheckDataType( strToken );

				    //IP address
				    if( iTokenType == 2  )
					{
					    Integer intTmp = new Integer(data.node.nodeIndex.ip2index( strToken, bReduceIP ));
	     				    nodeSetList.add( intTmp );
					}
				}//end for
			} //end while

  	       		traceReader.close();
			//count link per path


			iPathListIndex = pathInfo.newLinkInPath( 0 );
			iNodeSetListLen = nodeSetList.size();
			for( int b = 0; b < iNodeSetListLen; b++ )
			    {
				iNode[0] = ((Integer)nodeSetList.get( b )).intValue();
				for( int a = b+1; a < iNodeSetListLen; a++ )
				    {
    					iNode[1] = ((Integer)nodeSetList.get( a )).intValue();
					if( ( iLinkIndex = data.link.linkIndex.nodeIndex2linkIndex( iNode ) ) != -1 )
					    {
						pathInfo.addLinkInPath( iPathListIndex, iLinkIndex );
						break;
					    }
				    }
			    }
			iNodeSetListLen = 0;
			nodeSetList.clear();


		} catch(IOException e) {
		    //errorStream.println("File format exception.\n" + e.getMessage());
		    System.err.println("File format exception.\n" + e.toString());
		    //System.exit(0);
       		}
       	}//end for( j; hosts; )

       	return;
    }




    /**
    * Returns a string representing the relative error matrix.
    * @return
    *   a string
    **/
    public String toString()
    {
        StringBuffer result = new StringBuffer();

        for(int i=0;i<hosts;i++)
        {
            for(int j=0;j<hosts;j++)
            {
                result.append("");
            }

            result.append("\n");
        }

        return result.toString();
    }
}






