/**
 * <p>Title: LossyLinkBrite</p>
 * <p>Description: The <CODE>LossyLinkBrite</CODE> class is used to compute the lossy links,
 * given the input of e2e loss and path.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;



import java.io.*;
import java.util.*;
import cluster.*;
//import APSP.*;


public class LossyLinkBrite implements Serializable
{
    /* Checks if strInput indicates a new set of data
     *
     * Return values
     * 0 : A string that doesn't begin with a mixture of alphabet, number, and/or other character  has been found.
     * 1 : A string that begins with a number has been found.
     * 2 : A string that begins with alphabet has been found.
     */
    public int CheckCharacterType( String strInput )
    {
	boolean bLetter = false;
	boolean bDigit = false;

       	//debug codes
	//      	LossyLinkDebug.println( "CheckCharacterType() : strInput : " + strInput );

	for( int i = 0; i < strInput.length(); i++ )
	    {
		if( Character.isDigit( strInput.charAt(i) ) )
		    {
			if( bDigit == false )
			    bDigit = true;
		    }
		else if( Character.isLetter( strInput.charAt(i) ) )
		    {
			if( bLetter == false )
			    bLetter = true;
		    }
	    }

	if( bDigit == true  && bLetter == true )
	    {
		return 0;
	    }
	else if( bDigit == true && bLetter == false )
	    {
		return 1;
	    }
	else if( bDigit == false && bLetter == true )
	    {
		return 2;
	    }
	else
	    {
		return 0;
	    }
    }

    public int read( String strFileName, LossyLinkData data )
    {
	String strFileLine;
	String strTmp;

	int iNodeIndex = -1;
	int iIndegree = -1;
	int iOutdegree = -1;

	int iLinkVertex[] = new int[2];
	double dWeight = -1.0f;

	int iDataSet = 0;

	try
	    {
		BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strFileName))));
		while( (strFileLine = bufferedReader.readLine()) != null )
		    {
			StringTokenizer st = new StringTokenizer( strFileLine, "\t :" );
			if( st.hasMoreTokens() == true )
			    {
				strTmp = st.nextToken();

				//Check if this line is the beginning of a new set of data.
				if( CheckCharacterType( strTmp ) != 1 )
				    {
					iDataSet = 0;
					if( strTmp.compareTo( "Nodes" ) == 0 )
					    {
						iDataSet = 1;
					    }
					else if( strTmp.compareTo( "Edges" ) == 0 )
					    {
						iDataSet = 2;
					    }
					//if( iDataSet == 0 )
					    {
						continue;
					    }
				    }
			    }
			else
			    {
				continue;
			    }
			if( iDataSet == 1 )
			    {
				for( int i = 0; st.hasMoreTokens() && i < 5; ++i )
				    {
					switch( i )
					    {
					    case 0 :
						iNodeIndex = Integer.parseInt(strTmp);
						break;
					    case 3 :
						iIndegree = Integer.parseInt(st.nextToken());
						break;
					    case 4 :
						iOutdegree = Integer.parseInt(st.nextToken());
						break;
					    default :
						st.nextToken();
						break;
					    }
				    }

				data.node.set( iNodeIndex, 0, 0, 0, 0 );//iIndegree, iOutdegree, iEndHost );
                                //SHH added
                                LossyLinkNodeData nodeData = (LossyLinkNodeData)data.node.nodeTable.get( new Integer( iNodeIndex ) );
                                nodeData.iOriginalIndex = iNodeIndex;
                                data.node.nodeTable.put( new Integer( iNodeIndex ), nodeData );
                                //end
			    }
			else if( iDataSet == 2 )
			    {
				for( int i = 0; st.hasMoreTokens() && i < 10; ++i )
				    {
					switch( i )
					    {
					    case 0:
						break;
					    case 1 :
						iLinkVertex[0] = Integer.parseInt(st.nextToken());
						break;
					    case 2 :
						iLinkVertex[1] = Integer.parseInt(st.nextToken());
						break;
					    case 4 :
						dWeight = Double.parseDouble(st.nextToken());
						break;
					    case 9 :

                                                char charTmp = st.nextToken().charAt(0);
						if( charTmp == 'U' )
						{
						    data.link.setLinkType( data.link.UNDIRECTED );
						}
						else if( charTmp == 'D' )
						{
						    data.link.setLinkType( data.link.DIRECTED );
						    LossyLinkDebug.println( "you don;t need this check anymore" );
						}
						else
						{
						    System.out.println("ERROR : 00043 " );
						}

						break;
					    default :
						st.nextToken();
						break;
					    }
				    }
				data.link.set( iLinkVertex, dWeight, -1.0, false );
				//data.link.set( iToNode, iFromNode );
				data.node.incIndegree( iLinkVertex[1] );
				data.node.incOutdegree( iLinkVertex[0] );
/*                                if( data.link.getLinkType()  == data.link.UNDIRECTED )
                                {
                                    data.node.incIndegree( iLinkVertex[0] );
                                    data.node.incOutdegree( iLinkVertex[1] );
                                }
*/			    }
			else
			    {
					System.out.println( "ERROR 00001 " );
      					System.exit( 1 );
			    }
		    }
                    bufferedReader.close();
	    }
	catch( IOException e )
	    {
		System.err.println("LossyLinkBrite : File format error\n" + e.toString() );
		System.exit(1);
	    }
	return 1;
    }


    public void createAPSP( LossyLinkData data , LossyLinkPathInfo pathInfo )
    {

	int iNodeCount = data.node.count();
	Node rootNode[] = new Node[iNodeCount+1];
	for( int i = 0; i < iNodeCount; ++i )
	    {
		rootNode[i+1] = new Node( i+1 );
		buildGraph( data, i, rootNode[i+1] );
	    }

	Graph graph = new Graph();
	if( LossyLinkDebug.isDebug() == true )
	{
	    graph.printGraph( rootNode );
	}

	APSP.linearJohnson( rootNode, false, pathInfo, data );
    }


    private Node buildGraph( LossyLinkData data, int iNodeIndexInput, Node rootNode )
    {
	int iNodeIndex[] = new int[2];
	int iNodeCount = data.node.count();
	double dLinkWeight = -1.0f;
	int iNodeData[] = new int[3];
	Node node = rootNode;

	iNodeIndex[0] = iNodeIndexInput;

	for( int i = 0; i < iNodeCount; ++i )
	    {
		if( i == iNodeIndex[0] )
		    continue;

		iNodeIndex[1] = i;
		if( ( dLinkWeight = data.link.getWeight( iNodeIndex ) ) != -1 )
                {
		    LossyLinkDebug.println("buildGraph() node0:" + iNodeIndex[0] +
					   " node1:" + iNodeIndex[1] + " weight:" +
					   dLinkWeight);
		    node.setNext(new Node(iNodeIndex[1] + 1), dLinkWeight,
				 data.link.linkIndex.nodeIndex2linkIndex(iNodeIndex));
		    node = node.next;
                }
//-----------------------------------------------------------
                if( data.link.getLinkType() == data.link.UNDIRECTED )
                {
                    int iReverseNodeIndex[] = new int[2];
		    iReverseNodeIndex[0] = iNodeIndex[1];
		    iReverseNodeIndex[1] = iNodeIndex[0];
		    if ( (dLinkWeight = data.link.getWeight(iReverseNodeIndex)) != -1 )
                    {
                        LossyLinkDebug.println("buildGraph() undirected node0:" + iNodeIndex[0] +
                                               " node1:" + iNodeIndex[1] + " weight:" +
                                               dLinkWeight);
			node.setNext(new Node(iNodeIndex[1] + 1), dLinkWeight,
				     data.link.linkIndex.nodeIndex2linkIndex(iNodeIndex));
			node = node.next;
                    }
                }
	    }

	return node;
    }

/*
    public void bfs( int v, LossyLinkData data, LossyLinkPathInfo pathInfo, LossyLinkRank rank )
    {
	LinkedList linkList = new LinkedList();

	int vw = 0;
	int vwVertex[] = new int[2];
	int vData[] = new int[4];

	int iLinkId = -1;
	int iLinkIndex = -1;
	int iPathListIndex = -1;
	int iParentPathListIndex = -1;


	data.node.get( v, vData );
	data.node.set( v, vData[0], vData[1], 1 );
	search( v, data, linkList );

	while( linkList.size() > 0 )
	    {
		vw = ((Integer)linkList.removeFirst()).intValue();
	        data.link.linkIndex.linkIndex2nodeIndex( vw, vwVertex );

      	        LossyLinkDebug.println( "bfs() destin. v : " + vwVertex[0] + " w : " + vwVertex[1] + " llSize : " + linkList.size() );

       		iPathListIndex = pathInfo.newLinkData( v, vwVertex[1] );
       		//I think I don't need this if
       		if( (iLinkId = data.link.linkIndex.nodeIndex2linkIndex( vwVertex )) != -1 )
       		    {
       			iLinkIndex = data.link.getElement( iLinkId );
       			//I think I don't need this if
       			if( ( iParentPathListIndex = pathInfo.getIndex( v, vwVertex[0] ) ) !=  -1 )
       			    {
       				for( int j = 0; j < rank.getColCount(); ++j )
       				    {
       					if( rank.getElement( iParentPathListIndex, j ) != 0 )
       					    {
       						rank.setElement( iPathListIndex, j );
						pathInfo.incLinkData( iPathListIndex );
						LossyLinkDebug.println( "rank.set() : [" + v + "," + vwVertex[0] + "/" + iParentPathListIndex + "] [" + j + "]" );
       					    }
       				    }
       			    }
       			//remove this else afterword
       			else
       			    {
       				System.out.println("DO NOT REMOVE IF2 " );
       			    }
			pathInfo.incLinkData( iPathListIndex );
       			rank.setElement( iPathListIndex, iLinkIndex );
      			LossyLinkDebug.println( "rank.se2() : [" + v + "," + vwVertex[1] + "/" + iPathListIndex + "] [" + iLinkIndex + "/" + iLinkId + "]\n" );
       		    }
       		else
       		    {
       			System.out.println( "DO NOT REMOVE IF" );
       		    }
       		search( vwVertex[1], data, linkList );
	    }
	data.node.clearFlag();
    }

    public void search( int v, LossyLinkData data, LinkedList linkList )
    {
	int vwVertex[] = new int[2];
	int vw = 0;

	int wData[] = new int[3];
	int vData[] = new int[3];

	int iSearchCount = 0;

	data.node.get( v, vData );

	for( Enumeration enumLink = data.link.keys(); enumLink.hasMoreElements(); )
	    {
		if( vData[1] <= iSearchCount )
		    {
			break;
		    }
		vw = ((Integer)enumLink.nextElement()).intValue();
		data.link.linkIndex.linkIndex2nodeIndex( vw, vwVertex );
		if( vwVertex[0] == v )
		    {
      			data.node.get( vwVertex[1], wData );
			if( wData[2] == 0 )
			    {
				data.node.set( vwVertex[1], wData[0], wData[1], 1 );
  				//BfsData bfsData = new BfsData();
				//bfsData.iNode = vwVertex[1];
				//bfsData.
				linkList.add( new Integer(vw) );
			    }
		    }
	    }
	//	linkList.add( new Integer( -1 ) );
    }
*/


}

