/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

import java.io.*;
import java.util.*;

public class LossyLinkCDF
{
  static double dInputData[];
  static double dCDF[][];
  static double dMaxPercentage;
  static boolean bIgnoreFirstRow;
  static boolean bReverse;

  public LossyLinkCDF()
  {
    init();
  }

  static private void init()
  {
    dInputData        = null;
    dCDF              = null;
    dMaxPercentage    = 100.0;    //Maximum percentage CDF graph to plot
    bIgnoreFirstRow   = false;
    bReverse          = false;
  }

  public static void main(String[] args)
  {
    if( args.length < 1 || args.length > 4 )
    {
      System.out.println("\nUsage \t: java LossyLinkCDF <Input_File_Name> [OPTIONS]\n" +
                         "Options\t:\n"+
                         "\t-I : Ignore first row. Do not consider first row of input file as data.\n" +
                         "\t-R : Reverse X axis. Sort X axis data in decreasing order.\n" +
                         "Note \t: Output file name will be \"<Input_File_Name>.cdf\"\n" +
                         "Example\t: java LossyLinkCDF input.out 100\n" );
      System.exit(0);
    }
    LossyLinkCDF lossyLinkCDF1 = new LossyLinkCDF();

    if( args.length > 1 )
    {
      for( int i = 0; i < args.length -1; i++ )
      {
        if( args[1+i].compareToIgnoreCase("-I") == 0 )//ignore first row of input file
        {
          bIgnoreFirstRow = true;
        }
        else if( args[1+i].compareToIgnoreCase("-R") == 0 )//reverse x axis
        {
          bReverse = true;
        }
      }
    }

    open( args[0] );
    calculateCDF();
    String strOutputFile = args[0].substring( 0, args[0].lastIndexOf(".") ).concat(".cdf");
    System.out.println("Output file : " + strOutputFile );
    saveCDF( strOutputFile );
  }

  public static int open( String strFileName )
  {
    int iRowCount = 0;

    if( strFileName == null )
    {
        System.out.println( "ERROR LLCDF 00001 : File not found [File :"+ strFileName + "]" );
        return -1;
    }

    try
    {
        BufferedReader fileReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strFileName))));

        String strTmp;
	if( bIgnoreFirstRow == true )
	    {
		if( fileReader.readLine() == null )
		    {
			System.err.println("ERROR LLCDF 00400 : Can not read input file ["+strFileName+"]");
			System.exit(0);
		    }

	    }

        Vector inputData = new Vector();
        for( int i = 0; ( ( strTmp = fileReader.readLine() ) != null ); ++i )
        {
            if( strTmp.trim().length() == 0 )
            {
              continue;
            }
            if( strTmp.indexOf("%") > 0 )
            {
              inputData.add( strTmp.substring( 0, strTmp.indexOf("%") ) );
            }
            else
            {
              inputData.add( strTmp);
//              inputData.add( strTmp );
            }
        }
        fileReader.close();

        dInputData = new double[inputData.size()];



        if( bReverse )
        {
          for( int i = 0; i < dInputData.length; ++i )
          {
            dInputData[i] = 100 - Double.parseDouble( (String)inputData.get(i));
          }
        }
        else
        {
          for( int i = 0; i < dInputData.length; ++i )
          {
//            System.out.println("dINputData = "+(String)inputData.get(i));
              dInputData[i] = Double.parseDouble( (String)inputData.get(i));
          }
        }
    }
    catch (IOException e)
    {
        System.err.println( "ERROR LossyLinkCDF : File not opened properely\n" + e.toString());
        System.exit(1);
    }
    return 1;
  }

  static public void saveCDF( String strFileName )
  {
      if( strFileName == null )
      {
          Calendar calendar = Calendar.getInstance();
          strFileName = "LossyLink_CDF" + "_"
                                    + calendar.get( Calendar.DAY_OF_MONTH ) + "-"
                                    + calendar.get( Calendar.MONTH ) + "-"
                                    + calendar.get( Calendar.YEAR ) + "_"
                                    + calendar.get( Calendar.HOUR ) + "-"
                                    + calendar.get( Calendar.MINUTE )
                                    + ".out";
      }

      int iRowCount = dCDF.length;
      try
      {
          FileWriter output = new FileWriter( strFileName );
          for( int i = 0; i < iRowCount; ++i )
          {
            output.write( Double.toString( dCDF[i][0] ) );
            output.write( "\t" );
            output.write( Double.toString( dCDF[i][1] ) );
            output.write( '\n' );
            //for debug
            //System.out.println("dCDF["+i+"] x:"+dCDF[i][0]+"\ty:"+dCDF[i][1]);
          }
          output.close();
      }
      catch (IOException e)
      {
          System.err.println( "ERROR LLCDF : File not written properely\n" + e.toString());
          System.exit(1);
      }
  }


  public static void calculateCDF()
  {
    Arrays.sort( dInputData );

    for( int i = 0; i < dInputData.length; i++ )
    {
//      System.out.println("dInputDataAfterSort : " + dInputData[i] );
    }


    if( bReverse )
    {
      for( int i = 0; i < dInputData.length; i++ )
      {
        dInputData[i] = 100 - dInputData[i];
      }
    }


    //Compute average.
    double dAverage = 0.0;
    for( int i = 0; i < dInputData.length; i++ )
    {
      dAverage = dAverage + dInputData[i];
    }
    dAverage = dAverage / dInputData.length;
    System.out.println("Data count = " + dInputData.length );
    System.out.println("Average = " + rounding( dAverage, 3 ) );

    dCDF = new double[dInputData.length][2];

    for( int i = 0; i < dInputData.length; i++ )
    {
      if( i > 0 )
      {
        dCDF[i][1] = dCDF[i-1][1];
      }
      dCDF[i][0] = dInputData[i];
      dCDF[i][1] = dCDF[i][1] + ( 1 * 100 / dInputData.length );
    }

/*    for( int i = 0; i < iNumPoint; i++ )
    {
        //calculate Y axis values
        if( bReverse )
        {
          dXvalue = dInputData[0] + dDX * (i+1);
          dYvalue = dCumLossProb[dCumLossProb.length-1] + dCumLossProb[0] - ( dCumLossProb[i] / dCumLossProb[dCumLossProb.length-1] * 100.0 );
        }
        else
        {
          dXvalue = dInputData[0] + dDX * i;
          dYvalue = dCumLossProb[i] / dCumLossProb[dCumLossProb.length-1] * 100.0 ;
        }
        if( dYvalue > dMaxPercentage )
        {
          break;//just plot this much.
        }

        //write X, Y values to a data str.
        dCDF[i][0] = dXvalue;       // X axis : error (loss probability)
        dCDF[i][1] = dYvalue;       // Y axis : cumulative number of errors
    }
*/
    return;
  }

  public static String rounding( double dNumber, int point )
  {
      String strNumber = Double.toString( dNumber );
      if( ( strNumber.indexOf( "E" ) != -1 )
         || ( strNumber.indexOf( "NaN" ) != -1 )
         || ( strNumber.indexOf( "Inf" ) != -1 ) )
      {
          return strNumber;
      }
      int iDelimeter = strNumber.lastIndexOf(".");
      int iOffset = iDelimeter + point + 1;
      if( iOffset > strNumber.length() )
      {
          iOffset = strNumber.length();
      }
      return strNumber.substring( 0, iOffset );
  }
}









