/**
 * <p>Title: LossyLinkData</p>
 * <p>Description: The <CODE>LossyLinkData</CODE> class is used by LossyLink*.java
 * to store and load node data and link data using a hash tbl.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;


import java.io.*;
import java.util.*;
import cluster.*;



public class LossyLinkData// implements Serializable, WNMMSConstants
{
    public LossyLinkLink link;
    public LossyLinkNode node;
    public int iLinkReindexTable[];
    public int iLinkReindexTableCount;
    public int iNodeReindexTable[];
    public int iNodeReindexTableCount;

    public LossyLinkData()
    {
	node = new LossyLinkNode();
	link = new LossyLinkLink();
    }


    public void print( int iStat[] )
    {
	iStat[0] = node.print();
	iStat[1] = link.print();
    }

    public void printCount()
    {
    	System.out.println( "\nTotal number of links in topology : " + link.count() );
	System.out.println( "Total number of nodes(including the root and the leaf nodes) in topology : " + node.count() + "\n" );
    }


    public void preEdgePruning()
    {
      Object objData;
      boolean bMoreDegree1NodeExist = true;
      int iLink = -1;
      int iOppositeLink = -1;
      int iLinkVertex[] = new int[2];
      int iLinkVertexSwitched[] = new int[2];
      int iIndegree = -1;
      int iOutdegree = -1;
      int iNodeData[] = new int[4];
      //LossyLinkDebug.setDebug( true );
      //for debug
      int iEndHostCount = 0;
      for( int i = 0; i < node.count(); i++ )
      {
        node.get( i, iNodeData );
        if( iNodeData[0] + iNodeData[1] <= 1 )
        {
          iEndHostCount++;
        }
      }
      System.out.println("iEndHostCount : " + iEndHostCount );


      while( bMoreDegree1NodeExist )
      {
        iNodeReindexTableCount = node.count() + 1;
        iNodeReindexTable = new int[iNodeReindexTableCount];
        iLinkReindexTableCount = link.linkIndex.maxIndex() + 1;
        iLinkReindexTable = new int[iLinkReindexTableCount];

        bMoreDegree1NodeExist = false;

        for (int i = 0; i < node.count(); i++)
        {
          //if a node has only one in/out degree
          objData = node.nodeTable.get(new Integer(i));
          iIndegree = ( (LossyLinkNodeData) objData).getIndegree();
          iOutdegree = ( (LossyLinkNodeData) objData).getOutdegree();
          if ( ( (LossyLinkNodeData) objData).getEndHost() == 0 &&
               ( iIndegree + iOutdegree <= 1 ) )
          {
            bMoreDegree1NodeExist = true;

            //remove the node.
            node.remove(i);

            //remove connecting link also.
            for( Enumeration enumLink = link.keys(); enumLink.hasMoreElements(); )
            {
              iLink = ((Integer)enumLink.nextElement()).intValue();
              link.linkIndex.linkIndex2nodeIndex( iLink, iLinkVertex );
              if( iLinkVertex[0] == i )
              {
                if( link.remove(iLink) == 1 )
                {
                  node.decIndegree( iLinkVertex[1] );
                }
                break;
              }
              if( iLinkVertex[1] == i )
              {
                if( link.remove(iLink) == 1 )
                {
                  node.decOutdegree( iLinkVertex[0] );
                }
                break;
              }
            }
          }
          //if a node has two in/out degree but the links are opposite,
          //consider them as one.
          else if ( ( (LossyLinkNodeData) objData).getEndHost() == 0 &&
                    ( iIndegree + iOutdegree == 2 ) )
          {
            //the links are opposite?
            for( Enumeration enumLink = link.keys(); enumLink.hasMoreElements(); )
            {
              iLink = ((Integer)enumLink.nextElement()).intValue();
              link.linkIndex.linkIndex2nodeIndex( iLink, iLinkVertex );
              iLinkVertexSwitched[0] = iLinkVertex[1];
              iLinkVertexSwitched[1] = iLinkVertex[0];
              if( ( iOppositeLink = link.linkIndex.nodeIndex2linkIndex( iLinkVertexSwitched ) ) != -1 )
              {
                //remove the two links
                if( iLinkVertex[0] == i )
                {
                  //remove the node
                  node.remove(i);
                  if( link.remove( iLink ) == 1 )
                  {
                    node.decIndegree( iLinkVertex[1] );
                  }
                  if( link.remove( iOppositeLink ) == 1 )
                  {
                    node.decOutdegree( iLinkVertex[1] );
                  }
                  break;
                }
                if( iLinkVertex[1] == i )
                {
                  //remove the node
                  node.remove(i);
                  if( link.remove( iLink ) == 1 )
                  {
                    node.decOutdegree( iLinkVertex[0] );
                  }
                  if( link.remove( iOppositeLink ) == 1 )
                  {
                    node.decIndegree( iLinkVertex[0] );
                  }
                  break;
                }
              }
            }
          }
          //for debug
	  /*
          objData = node.nodeTable.get(new Integer(i));
          if( objData != null )
          {
          iIndegree = ( (LossyLinkNodeData) objData).getIndegree();
          iOutdegree = ( (LossyLinkNodeData) objData).getOutdegree();
          System.out.println("Node:" + i + " indeg:" + iIndegree + " outdeg:" + iOutdegree + " endhost:" + ( (LossyLinkNodeData) objData).getEndHost() );

          }
	  */

        }
        node.reindex( iNodeReindexTable, iNodeReindexTableCount );
        link.reindex( iLinkReindexTable, iLinkReindexTableCount,
                      iNodeReindexTable, iNodeReindexTableCount );

      }
      return;
    }



    public int edgePruning( LossyLinkPathInfo pathInfo )
    {
        Object objKey;
        int iLinkCount = link.count();
        int iNodeIndex[] = new int[2];
        int j = 0;

//If array boundary error comes up, suspect here first.
//        iLinkReindexTableCount = j;
//        iLinkReindexTable = new int[j];
        iLinkReindexTableCount = link.linkIndex.maxIndex() + 1;
        iLinkReindexTable = new int[iLinkReindexTableCount];
        iNodeReindexTableCount = node.count() + 1;
        iNodeReindexTable = new int[iNodeReindexTableCount];

        for( int i = 0; i < iLinkCount; ++i )
        {
            if( link.getCoverage( i ) == false )
            {
                if( link.linkIndex.linkIndex2nodeIndex( i, iNodeIndex ) != 0 )
                {
                    link.remove(i);
                    node.decOutdegree( iNodeIndex[0] );
                    node.decIndegree( iNodeIndex[1] );
                    LossyLinkDebug.println("Link pruned : " + i );
                }
                else
                {
                    System.out.println("ERROR : Link not pruned but it should have been pruned: " + i );
                }
            }
            else
            {
                iLinkReindexTable[j] = i;
                j++;
            }
        }

        //remove isolated nodes from graph.
        int iNode = -1;
        int iNodeData[] = new int[4];
        for( Enumeration enumNode = node.keys(); enumNode.hasMoreElements(); )
        {
            iNode = ((Integer)enumNode.nextElement()).intValue();
            node.get( iNode, iNodeData );
            if( iNodeData[0] == 0 && iNodeData[1] == 0 )
            {
                LossyLinkDebug.println( "EP Node removed. NodeId: "+iNode + "Deg: "+iNodeData[0] + ", " + iNodeData[1] + ", iEndHost: " + iNodeData[3] );//                System.out.println( "EP Node removed NodeId: "+iNode + "Deg: "+iNodeData[0] + ", " + iNodeData[1] + ", iEndHost: " + iNodeData[3] );
                node.remove( iNode );
            }
        }



        node.reindex( iNodeReindexTable, iNodeReindexTableCount );
        link.reindex( iLinkReindexTable, iLinkReindexTableCount,
                      iNodeReindexTable, iNodeReindexTableCount );
//for debug
        if( LossyLinkDebug.isDebug() == true )
        {
	    for (int i = 0; i < iLinkReindexTableCount; ++i)
            {
		LossyLinkDebug.println("EP iLinkReindexTable : [" + i + "]\t[" + iLinkReindexTable[i] + "]");
	    }
	    for (int i = 0; i < iNodeReindexTableCount; ++i)
            {
		LossyLinkDebug.println("EP iNodeReindexTable : [" + i + "]\t[" + iNodeReindexTable[i] + "]");
	    }
        }
//end

        pathInfo.reindex( this );

        return 1;
    }


    public int degreeDenotion()
    {
	int iNode = -1;
	int iLinkVertex[] = new int[2];
	int iIndegree = 0;
	int iOutdegree = 0;

	for( Enumeration enumNode = node.keys(); enumNode.hasMoreElements(); )
	    {
		iNode = ((Integer)enumNode.nextElement()).intValue();
       		iIndegree = 0;
       		iOutdegree = 0;

		for( Enumeration enumLink = link.keys(); enumLink.hasMoreElements(); )
		    {
                        int iKey = ((Integer)enumLink.nextElement()).intValue();

			link.linkIndex.linkIndex2nodeIndex( iKey, iLinkVertex );
			if( iLinkVertex[0] == iNode )
			    {
                                iOutdegree ++;
                                LossyLinkDebug.println( "Degree denotion : link_key : " + iKey + " iOutdegree : " + iOutdegree );
			    }
			if( iLinkVertex[1] == iNode )
			    {
				iIndegree ++;
                                LossyLinkDebug.println( "Degree denotion : link_key : " + iKey + " iIndegree : " + iIndegree );
			    }
		    }
		LossyLinkDebug.println( iNode + " " + "\t\t" + iIndegree + " & " + iOutdegree );
		node.setDegree( iNode, iIndegree, iOutdegree );
	    }
	return 1;
    }



/*
    public int edgeReduction( LossyLinkPathInfo pathInfo )
    {
	//	LossyLinkNodeData nodeData = new LossyLinkNodeData();
	int iLink = -1;
	int iNode = -1;
	int iNodeData[] = new int[4];
	int iLinkVertex[] = new int[2];
	int iReducibleLink[] = new int[2];
	int iReduceFlag[] = new int[2];
	int iReduceCount = 0;
        boolean bReducible = false;

        //initialize reindex settings
        iLinkReindexTableCount = link.linkIndex.maxIndex() + 1;
        iLinkReindexTable = new int[iLinkReindexTableCount];
        iNodeReindexTableCount = node.count() + 1;
        iNodeReindexTable = new int[iNodeReindexTableCount];

	for( Enumeration enumNode = node.keys(); enumNode.hasMoreElements(); )
	    {
		iNode = ((Integer)enumNode.nextElement()).intValue();
		node.get( iNode, iNodeData );
                LossyLinkDebug.println( "ER candidate NodeId: "+iNode + " Deg: "+iNodeData[0] + ", " + iNodeData[1] + ", iEndHost: " + iNodeData[3] );
                if( iNodeData[3] == 1 )//if the node is an End Host, skip reducing the node.
                {
                    continue;
                }

                bReducible = false;
                if( link.getLinkType() == link.DIRECTED )
                {
                    if( iNodeData[0] == 1 && iNodeData[1] == 1 )
                    {
                        bReducible = true;
                    }
                }
                else //UNDIRECTED
                {
                    if( iNodeData[0] + iNodeData[1] == 2 )
                    {
                        bReducible = true;
                    }
                }

		if( bReducible == true )
		    {
	      		//reducible node
			iReducibleLink[0] = 0;
			iReducibleLink[1] = 0;
	       		for( Enumeration enumLink = link.keys(); enumLink.hasMoreElements(); )
	       		    {
	       			iLink = ((Integer)enumLink.nextElement()).intValue();
	       			link.linkIndex.linkIndex2nodeIndex( iLink, iLinkVertex );
				if( iLinkVertex[1] == iNode )
				    {
					if( iReducibleLink[0] == 0)
                                        {
                                            iReducibleLink[0] = iLink;
                                        }
                                        else
                                        {
                                            iReducibleLink[1] = iLink;
                                        }
					iReduceFlag[0]++;
				    }
				else if( iLinkVertex[0] == iNode )
				    {
                                        if( iReducibleLink[0] == 0)
                                        {
                                            iReducibleLink[0] = iLink;
                                        }
                                        else
                                        {
                                            iReducibleLink[1] = iLink;
                                        }
					iReduceFlag[1]++;
				    }

                                bReducible = false;
                                if( link.getLinkType() == link.DIRECTED )
                                {
                                    if( iReduceFlag[0] == 1 && iReduceFlag[1] == 1 )
                                    {
                                        bReducible = true;
                                    }
                                }
                                else //UNDIRECTED
                                {
                                    if( iReduceFlag[0] + iReduceFlag[1] == 2 )
                                    {
                                        bReducible = true;
                                    }
                                }
				if( bReducible == true )
				    {
                                        LossyLinkDebug.println("ER : Reduce Link1 : " + iReducibleLink[0] + " Reduce Link2 : " + iReducibleLink[1] );
                                        //iLinkReindexTableCount = link.linkIndex.maxIndex() + 1;
                                        //iReducibleNode = link.reduce( iReducibleLink, iLinkReindexTable, iLinkReindexTableCount );
                                        int iOutput[] = new int[2];
                                        link.reduce( iReducibleLink, iOutput );
                                        if( pathInfo != null )
                                        {
                                          pathInfo.reduceLink( iReducibleLink, iOutput[0] );
                                        }
                                        LossyLinkDebug.println("ER : Reduced node: "+iNode );
					node.remove( iNode );
                                        //remove self loop connected to iNode.
                                        {
                                          int iLinkIndex4SelfLoop = -1;
                                          int iNodeIndex4SelfLoop[] = new int[2];
                                          iNodeIndex4SelfLoop[0] = iNode;
                                          iNodeIndex4SelfLoop[1] = iNode;
                                          if( ( iLinkIndex4SelfLoop = link.linkIndex.nodeIndex2linkIndex( iNodeIndex4SelfLoop ) ) != -1 )
                                          {
                                            LossyLinkDebug.println("Removing self looping link " + iLinkIndex4SelfLoop );
                                            link.remove( iLinkIndex4SelfLoop );
                                          }
                                        }
					iReduceCount++;
					break;
				    }
	       		    }
	  		iReduceFlag[0] = 0;
			iReduceFlag[1] = 0;
		    }
	    }

        //reindex
        iLinkReindexTableCount = link.linkIndex.maxIndex() + 1;
        iLinkReindexTable = new int[iLinkReindexTableCount];
        node.reindex( iNodeReindexTable, iNodeReindexTableCount );
        link.reindex( iLinkReindexTable, iLinkReindexTableCount,
                      iNodeReindexTable, iNodeReindexTableCount );

//for debug
        if( LossyLinkDebug.isDebug() == true )
        {
            for( int i = 0;  i < iLinkReindexTableCount; ++i )
            {
                LossyLinkDebug.println( "ER2 iLinkReindexTable : [" + i + "]\t[" + iLinkReindexTable[i] + "]" );
            }
            for( int i = 0;  i < iNodeReindexTableCount; ++i )
            {
                LossyLinkDebug.println( "ER2 iNodeReindexTable : [" + i + "]\t[" + iNodeReindexTable[i] + "]" );
            }
        }
//end
        if( pathInfo !=  null )
        {
            pathInfo.reindex( this );
        }
	System.out.println( "Edge Reduction() : Number of reduced edges : " + iReduceCount );
	return iReduceCount;
    }
*/

    public boolean isNewTriangle( int iLinkIndex, int iLinkIndex2, int iLinkIndex3,
                                  int iLinkVertex[], int iLinkVertex2[], int iLinkVertex3[] )
    {
        int iOutput[] = new int[4];

        for( int i = 0; i < iTriangleListCount; ++i )
        {
            if( iTriangleList[i][0] == iLinkIndex &&
                iTriangleList[i][1] == iLinkIndex2 &&
                iTriangleList[i][2] == iLinkIndex3 )
            {
                return false;
            }
            else if( iTriangleList[i][0] == iLinkIndex &&
                iTriangleList[i][2] == iLinkIndex2 &&
                iTriangleList[i][1] == iLinkIndex3 )
            {
                return false;
            }
            else if( iTriangleList[i][1] == iLinkIndex &&
                iTriangleList[i][0] == iLinkIndex2 &&
                iTriangleList[i][2] == iLinkIndex3 )
            {
                return false;
            }
            else if( iTriangleList[i][1] == iLinkIndex &&
                iTriangleList[i][2] == iLinkIndex2 &&
                iTriangleList[i][0] == iLinkIndex3 )
            {
                return false;
            }
            else if( iTriangleList[i][2] == iLinkIndex &&
                iTriangleList[i][0] == iLinkIndex2 &&
                iTriangleList[i][1] == iLinkIndex3 )
            {
                return false;
            }
            else if( iTriangleList[i][2] == iLinkIndex &&
                iTriangleList[i][1] == iLinkIndex2 &&
                iTriangleList[i][0] == iLinkIndex3 )
            {
                return false;
            }
        }


        if( node.isEndHost( iLinkVertex[0] ) == 0 &&
            node.isEndHost( iLinkVertex[1] ) == 0 &&
            node.isEndHost( iLinkVertex2[0] ) == 0 &&
            node.isEndHost( iLinkVertex2[1] ) == 0 &&
            node.isEndHost( iLinkVertex3[0] ) == 0 &&
            node.isEndHost( iLinkVertex3[1] ) == 0 )
        {
            node.get( iLinkVertex[0], iOutput );
            if( iOutput[0] + iOutput[1] != 3 )
                return false;
            node.get( iLinkVertex[1], iOutput );
            if( iOutput[0] + iOutput[1] != 3 )
                return false;
            node.get( iLinkVertex2[0], iOutput );
            if( iOutput[0] + iOutput[1] != 3 )
                return false;
            node.get( iLinkVertex2[1], iOutput );
            if( iOutput[0] + iOutput[1] != 3 )
                return false;
            node.get( iLinkVertex3[0], iOutput );
            if( iOutput[0] + iOutput[1] != 3 )
                return false;
            node.get( iLinkVertex3[1], iOutput );
            if( iOutput[0] + iOutput[1] != 3 )
                return false;



            iTriangleList[iTriangleListCount][0] = iLinkIndex;
	    iTriangleList[iTriangleListCount][1] = iLinkIndex2;
	    iTriangleList[iTriangleListCount][2] = iLinkIndex3;
	    iTriangleListCount++;

	    return true;
        }

        return false;
    }


    int iTriangleListCount = 0;
    int iTriangleList[][] = new int[100][3];

    public int findTriangularSubgraph()
    {
        int iSubgraphCount = 0;
        int iLinkIndex = 0;
        int iLinkIndex2 = 0;
        int iLinkIndex3 = 0;
        int iLinkVertex3[] = new int[2];
        int iLinkVertex2[] = new int[2];
        int iLinkVertex[] = new int[2];

        System.out.println( "\nFind Triangular Subgraph ------------------------------------");

        for( Enumeration enumLink = link.keys(); enumLink.hasMoreElements(); )
        {
            iLinkIndex = ((Integer)enumLink.nextElement()).intValue();
            link.linkIndex.linkIndex2nodeIndex( iLinkIndex, iLinkVertex );

            for( Enumeration enumLink2 = link.keys(); enumLink2.hasMoreElements(); )
            {
                iLinkIndex2 = ((Integer)enumLink2.nextElement()).intValue();
                if( iLinkIndex2 == iLinkIndex )
                    continue;
                link.linkIndex.linkIndex2nodeIndex( iLinkIndex2, iLinkVertex2 );

                if( iLinkVertex[0] == iLinkVertex2[0] )
                {
                    for( Enumeration enumLink3 = link.keys(); enumLink3.hasMoreElements(); )
                    {
                        iLinkIndex3 = ((Integer)enumLink3.nextElement()).intValue();
                        if( iLinkIndex3 == iLinkIndex2 || iLinkIndex3 == iLinkIndex )
                            continue;
                        link.linkIndex.linkIndex2nodeIndex( iLinkIndex3, iLinkVertex3 );

                        if( iLinkVertex3[0] == iLinkVertex[1] && iLinkVertex3[1] == iLinkVertex2[1] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {

                                //triangle found
                                System.out.println("A Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                        if( iLinkVertex3[1] == iLinkVertex[1] && iLinkVertex3[0] == iLinkVertex2[1] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("B Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                    }
                }
                if( iLinkVertex[0] == iLinkVertex2[1] )
                {
                    for( Enumeration enumLink3 = link.keys(); enumLink3.hasMoreElements(); )
                    {
                        iLinkIndex3 = ((Integer)enumLink3.nextElement()).intValue();
                        if( iLinkIndex3 == iLinkIndex2 || iLinkIndex3 == iLinkIndex )
                            continue;
                        link.linkIndex.linkIndex2nodeIndex( iLinkIndex3, iLinkVertex3 );

                        if( iLinkVertex3[0] == iLinkVertex[1] && iLinkVertex3[1] == iLinkVertex2[0] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("C Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                        if( iLinkVertex3[1] == iLinkVertex[1] && iLinkVertex3[0] == iLinkVertex2[0] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("D Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                    }
                }


                if( iLinkVertex[1] == iLinkVertex2[0] )
                {
                    for( Enumeration enumLink3 = link.keys(); enumLink3.hasMoreElements(); )
                    {
                        iLinkIndex3 = ((Integer)enumLink3.nextElement()).intValue();
                        if( iLinkIndex3 == iLinkIndex2 || iLinkIndex3 == iLinkIndex )
                            continue;
                        link.linkIndex.linkIndex2nodeIndex( iLinkIndex3, iLinkVertex3 );

                        if( iLinkVertex3[0] == iLinkVertex[0] && iLinkVertex3[1] == iLinkVertex2[1] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("E Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                        if( iLinkVertex3[1] == iLinkVertex[0] && iLinkVertex3[0] == iLinkVertex2[1] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("F Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                    }
                }
                if( iLinkVertex[1] == iLinkVertex2[1] )
                {
                    for( Enumeration enumLink3 = link.keys(); enumLink3.hasMoreElements(); )
                    {
                        iLinkIndex3 = ((Integer)enumLink3.nextElement()).intValue();
                        if( iLinkIndex3 == iLinkIndex2 || iLinkIndex3 == iLinkIndex )
                            continue;
                        link.linkIndex.linkIndex2nodeIndex( iLinkIndex3, iLinkVertex3 );

                        if( iLinkVertex3[0] == iLinkVertex[0] && iLinkVertex3[1] == iLinkVertex2[0] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("G Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                        if( iLinkVertex3[1] == iLinkVertex[0] && iLinkVertex3[0] == iLinkVertex2[0] )
                        {
                            if( isNewTriangle( iLinkIndex, iLinkIndex2, iLinkIndex3,
                                               iLinkVertex, iLinkVertex2, iLinkVertex3 ) == true )
                            {
                                //triangle found
                                System.out.println("H Link1: " + iLinkIndex + " Link2: " + iLinkIndex2 + " Link3: " + iLinkIndex3 );
                                System.out.println("  LV1[0]: " + iLinkVertex[0] + " LV1[1]: " + iLinkVertex[1] + "\n" +
                                                   "  LV2[0]: " + iLinkVertex2[0] + " LV2[1]: " + iLinkVertex2[1] + "\n" +
                                                   "  LV3[0]: " + iLinkVertex3[0] + " LV3[1]: " + iLinkVertex3[1] );
                                System.out.println("\n" );
                                iSubgraphCount++;
                            }
                        }
                    }
                }
            }
        }
        System.out.println( "Triangle List Count : " + iTriangleListCount + "\n" );
        return iSubgraphCount;
    }


    public int degreeDistributionByFile( String strEndhostFile )
    {
      int iEndHostCount = 0;
      iEndHostCount = node.endHostDenotionByFile( strEndhostFile );
      node.saveDegree( null );

      return iEndHostCount;
    }

    public int degreeDistribution( int iEndHostMaxDegree, int iEndHostCountInput )
    {
        int iEndHostCount = 0;
        if( iEndHostMaxDegree == -1 && iEndHostCountInput != -1 )
        {
            iEndHostCount = node.endHostDenotionByCount( iEndHostCountInput );
        }
        else
        {
            iEndHostCount = node.endHostDenotionByDegree( iEndHostMaxDegree );
        }
        node.saveDegree( null );

        return iEndHostCount;
    }
}
