/**
 * <p>Title: LossyLinkMeshReduced</p>
 * <p>Description: The <CODE>LossyLinkMeshReduced</CODE> class is used to compute the lossy links,
 * given the input of e2e loss and path</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;



import java.io.*;
import java.util.*;
import cluster.*;

public class LossyLinkGetG implements Serializable, WNMMSConstants
{
    static LossyLink lossyLink;
    static String root;
    static String baseFilename;
    static HostMap rootMap;
    static PrintWriter errorStream;
    static LossyLinkData data;
    static boolean bReduceIP = false;
    static boolean bTest = false;
    static boolean bDebug = false;
    static boolean bReduceGraph = false;
    static LossyLinkPathInfo pathInfo;
    static LossyLinkBrite lossyLinkBrite;
    static int iInputFileType = 0;
    static String strBriteFile;
    static String strDateInput;
    static int iNumberOfHosts = 0;
    static String strSitesFile;
    static int iEndHostMaxDegree = -1;
    static int iEndHostCount = -1;
//    static LossyLinkVirtualize virtualize;
    static String strOutputFileName;
    static String strGmatrixFile;
    static String strBvectorFile;
    static String strTvectorFile;
    static String strCvectorFile;
    static String strVirGmatrixFile;
    static String strVirBvectorFile;
    static String strVirTvectorFile;

    public static void main(String[] args)
    {
        int i = 0;
	int iTotalPathCount = 0;
        lossyLink = new LossyLink();
	data = new LossyLinkData();
	pathInfo = new LossyLinkPathInfo();
//        virtualize = new LossyLinkVirtualize( pathInfo, data );

        int iStat[][] = new int[5][2];// iStat ----------------------------------------
                                     //                        Node_count  Link_count
                                     //  Initially             0,0         0,1
                                     //  After_pruning         1,0         1,1
                                     //  After_graph_reduction 2,0         2,1
                                     //  Total_path_count      3,0
                                     //  End_host_count        3,1
                                     //  After_virtualization  4,0         4,1

        parseInput( args );

	System.out.println( "\nData Read ---------------------------------------" );
	if( iInputFileType == 0 )
	    {
		initialize( strDateInput, strSitesFile );
		if( bTest == true )
		    {
			iNumberOfHosts = 1;
		    }

		for (i = 0; i < iNumberOfHosts; i++)
   		    {
		        Host r_j = rootMap.getHost(i);
			String r_jName = r_j.getName();

			LossyLinkDebug.println("SrcHost : " + r_jName + " [" + i + "]");

			/* for Debugging purpose
			      if( r_jName.compareToIgnoreCase( "amp-alaska" ) != 0 )
			       {
			    continue;
			       }
			 */

			if ( (lossyLink.initialize( strDateInput, iNumberOfHosts, strSitesFile, r_jName, bReduceIP)) == 0)
                        {
			    System.err.println("ERROR : main() !Initialize()");
			    return;
			}

			iTotalPathCount += lossyLink.readTopology(errorStream, data, bTest);
		    }
	    }
	else
	    {
		lossyLinkBrite = new LossyLinkBrite();
		lossyLinkBrite.read( strBriteFile, data );
		iTotalPathCount = data.node.count() * ( data.node.count() - 1 );
	    }

        //System.out.println("Initial TotalPathCount:" + iTotalPathCount + " TotalLinkCount:" + data.link.count() );
      	data.print( iStat[0] );

        System.out.println("\nCreate Shortest Path -------------------------" );
        if( iInputFileType == 0 )
	    {
		for (i = 0; i < iNumberOfHosts; i++)
		    {
			Host r_j = rootMap.getHost(i);
			String r_jName = r_j.getName();

			LossyLinkDebug.println("SrcHost : " + r_jName + " [" + i + "]");

			/* for Debugging purpose
			   if( r_jName.compareToIgnoreCase( "amp-alaska" ) != 0 )
			    {
			 continue;
			    }
			 */

			if ( (lossyLink.initialize( strDateInput, iNumberOfHosts, strSitesFile, r_jName, bReduceIP)) == 0)
			{
			    System.err.println("ERROR : main() !Initialize()");
			    return;
			}

			lossyLink.compareTopology(errorStream, data, pathInfo, i, bTest);
		    }
	    }
	else
	    {
		iStat[3][1] = data.degreeDistribution( iEndHostMaxDegree, iEndHostCount );
		lossyLinkBrite.createAPSP( data, pathInfo );
	    }

//        pathInfo.removeOppositePath( data );
//	pathInfo.removeInternalNode( data );

        System.out.println("\nsaveGmatrix -------------------------" );
	pathInfo.saveGmatrix( strGmatrixFile, data.link );
        System.out.println("\nLLMR DONE" );
    }



    private static void showUsage()
    {
	System.err.println("\nUsage: \njava LossyLinkMeshReduced " +
			   "<date> <number_of_hosts> <sites_file> [OPTION]\n"+
			   "or java LossyLinkMeshReduced <BRITE_file> [OPTIION]\n"+
                           "or java LossyLinkMeshReduced <Input_script_file>\n"+
			   "\ni.e. java LossyLinkMeshReduced 102.11.1 111 newsites.lst -ar" +
			   "\ni.e. java LossyLinkMeshReduced ../BRITE/topology/ASBarabasi100U.brite -ar -m2" +
                           "\ni.e. java LossyLinkMeshReduced ASBarabasi100U.input" +
			   "\n\nOption: "+
			   "\n\t-a\tAggregate IP address length to 29 bit instead of 32."+
			   "\n\t  \tThis option is only applied to *.trace input files." +
			   "\n\t-r\tReduce graph."+
			   "\n\t-m###\tMaximum degree of end hosts." +
			   "\n\t  \tEvery node is considered as end hosts if not stated." +
			   "\n\t  \tThis option is only applied to BRITE input files." +
                           "\n\t-c###\tEnd host count." +
                           "\n\t  \tEvery node is considered as end hosts if not stated." +
                           "\n\t  \tThis option is only applied to BRITE input files." +
                           "\n\t  \t-m and -c options can not be specified at the same time. The latter will be ignored." +
                           "\n\nNotes: "+
			   "\n\tTemporary output files are <BRITE_file>_A.out, <BRITE_file>_X.out, <BRITE_file>_T.out, and LossyLinkDegree.out" );
    }


    private static int initialize( String strDateInput, String strSitesFile )
    {
	baseFilename = ".." + SLASH + "data" + SLASH + "raw-data" + SLASH + strDateInput + SLASH;
	rootMap = new HostMap( strSitesFile, iNumberOfHosts );

	return 1;
    }



    private static void parseInput( String args[] )
    {
        if( args.length != 1 )
        {
            System.err.println("ERROR : Too many arguments");
            showUsage();
            System.exit( 0 );
        }

        //input script file
        iInputFileType = 2;
        parseInputScriptFile( args[0] );
	LossyLinkDebug.setDebug(bDebug);

        LossyLinkDebug.println( "\nInternal flags info -------------------\n" +
                                "\tbReduceIP["+bReduceIP+"]\n"+
                                "\tbReduceGraph["+bReduceGraph+"]\n"+
                                "\tbTest["+bTest+"]\n"+
                                "\tbDebug["+bDebug+"]\n");
    }



    private static void parseInputScriptFile( String strFileName )
    {
        String strLine;
        System.out.println("InputFileName : "+strFileName );
        try
        {
            BufferedReader inputScriptReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strFileName))));

            for( int i = 0; ( strLine = inputScriptReader.readLine() ) != null; ++i )
            {
                StringTokenizer st = new StringTokenizer( strLine, "\t\n\r= " );

                //loop for every token
                String strDefinition = null;
                String strValue = null;
                String strToken = null;
                while( st.hasMoreTokens() )
                {
                    strToken = st.nextToken();

                    //Ignore comments
                    if( strToken.startsWith( "//" ) == true )
                    {
                        break;
                    }

                    //Store script file fields into strDefinition and strValue
                    if( strDefinition == null )
                    {
                        strDefinition = strToken;
                        continue;
                    }
                    strValue = strToken;
                    if( strValue.length() == 0 )
                    {
                        System.err.println("ERROR LLMR 00121 : No value specified for " + strDefinition + " field" );
                        showUsage();
                        System.exit( 0 );
                    }
                    LossyLinkDebug.println("strDefinition : " + strDefinition );
                    LossyLinkDebug.println("strValue : " + strValue );

                    //Common section for both BRITE and Trace files.
                    if( strDefinition.compareToIgnoreCase( "InputFileType" ) == 0 )
                    {
                        if( strValue.compareToIgnoreCase( "TRACE" ) == 0 )
                        {
                            iInputFileType = 0;
                        }
                        else if( strValue.compareToIgnoreCase( "BRITE" ) == 0 )
                        {
                            iInputFileType = 1;
                        }
                        else
                        {
                            System.err.println("ERROR LLMR 00120 : Unknown Input script value " + strDefinition + " = " + strValue );
                            showUsage();
                            System.exit( 0 );
                        }
                        continue;
                    }
                    else if( strDefinition.compareToIgnoreCase( "ReduceGraph" ) == 0 )
                    {
                        if( strValue.compareTo( "1" ) == 0 )
                        {
                            bReduceGraph = true;
                        }
                        break;
                    }
                    else if( strDefinition.compareToIgnoreCase( "Test" ) == 0 )
                    {
                        if( strValue.compareTo( "1" ) == 0 )
                        {
                            bTest = true;
                        }
                        break;
                    }
                    else if( strDefinition.compareToIgnoreCase( "Debug" ) == 0 )
                    {
                        if( strValue.compareTo( "1" ) == 0 )
                        {
                            bDebug = true;
                        }
                        break;
                    }
                    else if( strDefinition.compareToIgnoreCase( "OutputFileName" ) == 0 )
                    {
                        strOutputFileName = strValue;
                        strGmatrixFile = strValue + "_G.out";
                        strBvectorFile = strValue + "_B.out";
                        strTvectorFile = strValue + "_T.out";
                        strCvectorFile = strValue + "_C.out";
                        strVirGmatrixFile = strValue + "_virG.out";
                        strVirBvectorFile = strValue + "_virB.out";
                        strVirTvectorFile = strValue + "_virT.out";
                        break;
                    }


                    if( iInputFileType == 0 )//TRACE file specific section
                    {
                        if( strDefinition.compareToIgnoreCase( "Date" ) == 0 )
			{
			    strDateInput = strValue;
                            break;
			}
                        else if( strDefinition.compareToIgnoreCase( "NumberOfHosts" ) == 0 )
                        {
                            iNumberOfHosts = Integer.parseInt( strValue );
                            break;
                        }
                        else if( strDefinition.compareToIgnoreCase( "SitesFile" ) == 0 )
                        {
                            strSitesFile = strValue;
                            break;
                        }
                        else if( strDefinition.compareToIgnoreCase( "AggregateIP" ) == 0 )
                        {
                            if( strValue.compareTo( "1" ) == 0 )
                            {
                                bReduceIP = true;
                            }
                            break;
                        }
                    }
                    else if( iInputFileType == 1 )//BRITE file specific section
                    {
                        if( strDefinition.compareToIgnoreCase( "BriteFile" ) == 0 )
                        {
                            strBriteFile = strValue;
                            break;
                        }
                        else if( strDefinition.compareToIgnoreCase( "MaximumDegreeOfEndHosts" ) == 0 )
                        {
                            iEndHostMaxDegree = Integer.parseInt( strValue );
                            break;
                        }
                        else if( strDefinition.compareToIgnoreCase( "EndHostCount" ) == 0 )
                        {
                            if( iEndHostMaxDegree != -1 )
                            {
                                System.err.println("WARNING LLMR 00123 : MaximumDegreeOfEndHosts and EndHostCount can not be used at a same time. Ignoring the latter" );
                            }
                            iEndHostCount = Integer.parseInt( strValue );
                            break;
                        }
                    }

                }//end while
            }//end for
            inputScriptReader.close();
        }
        catch (IOException e)
        {
            System.err.println( "LossyLinkMeshReduced : File not opened properely\n" + e.toString());
            showUsage();
            System.exit(1);
        }
    }
}
