/**
 * <p>Title: LossyLinkLink</p>
 * <p>Description: The <CODE>LossyLinkLink</CODE> class is used by LossyLink*.java
 * to store and load link data using a hash tbl.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;


import java.io.*;
import java.util.*;
import cluster.*;


public class LossyLinkLink// implements Serializable, WNMMSConstants
{
    Hashtable linkTable;
    LinkIndex linkIndex;
    int iCoveredLinkCount;
    static boolean bLinkType;
    static public boolean DIRECTED = false;
    static public boolean UNDIRECTED = true;

    public LossyLinkLink()
    {
	linkTable = new Hashtable();
	linkIndex = new LinkIndex();
	bLinkType = UNDIRECTED;

        iCoveredLinkCount = 0;
    }

    public int count()
    {
      return linkTable.size();
    }

    public Enumeration keys()
    {
	return linkTable.keys();
    }

//Methods concerning LinkType
    static public void setLinkType( boolean bLinkTypeInput )
    {
	bLinkType = bLinkTypeInput;
    }
    static public boolean getLinkType()
    {
	return bLinkType;
    }


    /* Stores an link data to hash table.
     */
    public int set( int iNode[], double dWeight, double dLossRate, boolean bCoverage )
    {
	return set( linkIndex.set( iNode ), linkIndex.set( iNode ), dWeight, dLossRate, bCoverage );
    }

    public int set( int iLinkIndex, int iLinkData, double dWeight, double dLossRate, boolean bCoverage )
    {
	//Path information is set as Link data(element).
	//Check if there exists a link going from node1 to node2

      	if( (linkTable.put( new Integer(iLinkIndex), new LinkData( iLinkData, dWeight, dLossRate, bCoverage ) )) == null )
	    {
	      	//If there is no link going either from v1 to v2 or from v2 to v1,
	       	//store the link in the hash table.
		//LossyLinkDebug.println( "StoreLinkData1 : " + iLink );
	       	return iLinkIndex;
	    }
	else
	    {
		//If there is an link going from v1 to v2,
		//do not store an link going from v2 to v1. Just leave as it is.
		//LossyLinkDebug.println( "StoreLinkData2 : " + iLink );
		return iLinkIndex;
	    }
    }

    public int setLossRate( int iLinkIndex, double dLossRate )
    {
	Object objData = new Object();
	Integer intLinkIndex = new Integer( iLinkIndex );
	if( ( objData = linkTable.get( intLinkIndex )) == null )
	{
	    //If there is no link going either from v1 to v2 or from v2 to v1,
	    //store the link in the hash table.
	    System.out.println("ERROR 00040");
	    return 0;
	}
	else
        {
	    ((LinkData)objData).setLossRate( dLossRate );
	    linkTable.put( intLinkIndex, objData );
	    return 1;
	}
    }

    public int setCoverage( int iLinkIndex, boolean bCoverage )
    {
        Object objData = new Object();
        Integer intLinkIndex = new Integer( iLinkIndex );
        if( ( objData = linkTable.get( intLinkIndex )) == null )
        {
            //If there is no link going either from v1 to v2 or from v2 to v1,
            //store the link in the hash table.
            System.out.println("ERROR 00041");
            return 0;
        }
        else
        {
            ((LinkData)objData).setCoverage( bCoverage );
            linkTable.put( intLinkIndex, objData );

            return 1;
        }
    }

    public int getLinkData( int iLinkIndex )
    {
        Object objTmp;
        if((objTmp = linkTable.get( new Integer( iLinkIndex ) ) ) == null )
        {
          return -99999;
        }
        return ((LinkData)objTmp).getLinkData();
    }

    public double getWeight( int iNode[] )
    {
	return getWeight( linkIndex.nodeIndex2linkIndex( iNode ) );
    }

    public double getWeight( int iLinkIndex )
    {
	Object objTmp;
	if((objTmp = linkTable.get( new Integer( iLinkIndex ) ) ) == null )
	    {
		return -99999;
	    }
	return ((LinkData)objTmp).getWeight();
    }

    public double getLossRate( int iLinkIndex )
    {
	Object objTmp;
	if((objTmp = linkTable.get( new Integer( iLinkIndex ) ) ) == null )
	    {
		return -99999;
	    }
	return ((LinkData)objTmp).getLossRate();
    }

    public boolean getCoverage( int iLinkIndex )
    {
        Object objTmp;
        if((objTmp = linkTable.get( new Integer( iLinkIndex ) ) ) == null )
            {
                System.out.println("ERROR : 00060");
                System.exit(0);
            }
        return ((LinkData)objTmp).getCoverage();
    }

    public int remove( int iRemoveLinkIndex )
    {
        if( linkTable.remove( new Integer( iRemoveLinkIndex ) ) == null )
        {
            System.err.println( "remove() Can't remove link : " + iRemoveLinkIndex );
            return 0;
        }
        linkIndex.remove( iRemoveLinkIndex );
        return 1;
    }

/*
    public int reduce( int iReduceableLink[], int iOutput[] )
    {
	int iReduceableNode[][] = new int[2][2];
	int iReducedNode[] = new int[2];
	int iReducedLink = -1;
        //new parameter values
        int iNewLinkIndex = -1;
        double dNewLinkWeight = 0;
        double dNewLinkLossRate = -1.0;
        boolean bNewLinkCoverage = false;

       	LossyLinkDebug.println( "reduceLink : " + iReduceableLink[0] + " " + iReduceableLink[1] );

//set new parameters for newely created link.
        dNewLinkWeight = getWeight( iReduceableLink[0] ) + getWeight( iReduceableLink[1] );
        if( getCoverage( iReduceableLink[0] ) == true &&
            getCoverage( iReduceableLink[1] ) == true )
        {
            bNewLinkCoverage = true;
        }
        double dTemp[] = new double[2];
        if( ( dTemp[0] = getLossRate( iReduceableLink[0] ) ) != -1.0 ||
            ( dTemp[1] = getLossRate( iReduceableLink[1] ) ) != -1.0 )
        {
          dNewLinkLossRate = 1 - ( ( 1 - dTemp[0] ) * ( 1 - dTemp[1] ) );
        }
//end parameter settings

	if( linkTable.remove( new Integer( iReduceableLink[0] ) ) == null )
	    {
		System.err.println( "reduceNode() Can't remove strReduceableLink[0]" );
		return 0;
	    }
	if( linkTable.remove( new Integer( iReduceableLink[1] ) ) == null )
	    {
		System.err.println( "reduceNode() Can't remove strReduceableLink[1]" );
		return 0;
	    }

	linkIndex.linkIndex2nodeIndex( iReduceableLink[0], iReduceableNode[0] );
	linkIndex.linkIndex2nodeIndex( iReduceableLink[1], iReduceableNode[1] );

        if( iReduceableNode[0][0] == iReduceableNode[1][0] )
        {
            iReducedNode[0] = iReduceableNode[0][1];
            iReducedNode[1] = iReduceableNode[1][1];
        }
        else if( iReduceableNode[0][0] == iReduceableNode[1][1] )
        {
            iReducedNode[0] = iReduceableNode[0][1];
            iReducedNode[1] = iReduceableNode[1][0];
        }
        else if( iReduceableNode[0][1] == iReduceableNode[1][0] )
        {
            iReducedNode[0] = iReduceableNode[0][0];
            iReducedNode[1] = iReduceableNode[1][1];
        }
        else if( iReduceableNode[0][1] == iReduceableNode[1][1] )
        {
            iReducedNode[0] = iReduceableNode[0][0];
            iReducedNode[1] = iReduceableNode[1][0];
        }
        else
        {
            System.out.println("ERROR 00100" );
        }

	if( iReducedNode[0] == iReducedNode[1] )
        {
            LossyLinkDebug.println( "ERROR : Self loop found while reducing nodes : "
                                    +iReduceableNode[0][0] + "," + iReduceableNode[1][1] );
            // For now, just go ahead...
        }


        //bCoverge set to be true because before reduction, the two links were covered by some paths.
        //This part( below 4 lines from here) is a bit tricky.
        //First remove linkIndexes of reduceable link.
        //Second set both new linkIndex and new linkTable.
        //Notice that if we don't remove linkIndex before setting a new one, it will just ignore the request to set a new link index
	linkIndex.remove( iReduceableLink[0] );
	linkIndex.remove( iReduceableLink[1] );
        iNewLinkIndex = this.set( iReducedNode, dNewLinkWeight, dNewLinkLossRate, bNewLinkCoverage );

        iOutput[0] = iNewLinkIndex;
	return 1;
    }
*/
    //update every links in topology from old link vertex to new link vertex.
    public int updateLinkVertex( int iOldLinkVertex, int iNewLinkVertex )
    {
      //I think using Enumeration class is more safe and accurate.
      //But for the sake of simplicity I use int i.
      int iLinkCount = this.count();
      int iUpdateCount = 0;
      for( int i = 0; i < iLinkCount; ++i )
      {
        if( linkIndex.updateEachLinkVertex( i, iOldLinkVertex, iNewLinkVertex ) == 1 )
        {
          iUpdateCount ++;
        }
      }

      return iUpdateCount;
    }


    public void reindex( int iIndexConversionChart[], int iIndexConversionChartCount,
                         int iNodeReindexTable[], int iNodeReindexTableCount )
    {
      int iLinkTableSize = linkTable.size();
      Hashtable newLinkTable = new Hashtable( iLinkTableSize );
      int i = 0;
      int iKey[] = new int[iLinkTableSize];
      Object objData = new Object();

      if( iIndexConversionChart != null )
      {
          for ( int a = 0; a < iIndexConversionChartCount; ++a)
          {
              iIndexConversionChart[a] = -1;
          }
      }

      for( Enumeration enum = linkTable.keys(); enum.hasMoreElements(); ++i )
      {
          iKey[i] = ((Integer)enum.nextElement()).intValue();
          newLinkTable.put( new Integer(i), objData );
      }

      LossyLinkSort.sortArray( iKey, 0 , iLinkTableSize-1 );
      for( int j = 0; j < iLinkTableSize; ++j )
      {
          objData = linkTable.get( new Integer(iKey[j]) );
//            Double d = new Double( ((LinkData)objData).dWeight );
//            System.out.println("objData : " + d );
          newLinkTable.put( new Integer( j ), objData );
          if( iIndexConversionChart != null )
          {
              iIndexConversionChart[iKey[j]] = j;
          }
      }
      linkTable = newLinkTable;

      if( iIndexConversionChart != null )
      {
          linkIndex.reindex( iNodeReindexTable, iNodeReindexTableCount );
      }
    }


    public int print()
    {
	Object objIndex = new Object();
        Object objData = new Object();
	int iNodeIndex[] = new int[2];
	int iLinkIndex = -1;
        int iLinkCount = linkTable.size();

	if( LossyLinkDebug.isDebug() == true )
	    {
		LossyLinkDebug.println("print Link Hashtable ---------------------------\n");
		for( Enumeration enum = linkTable.keys(); enum.hasMoreElements(); )
		    {
			objIndex = enum.nextElement();
			iLinkIndex = ((Integer)objIndex).intValue();
			LossyLinkDebug.println( "Link Index : " + iLinkIndex );
                        objData = linkTable.get( objIndex );
                        LossyLinkDebug.println( "Link Data : weight : " + ((LinkData)objData).getWeight() +
						" lossRate : " + ((LinkData)objData).getLossRate() );
			linkIndex.linkIndex2nodeIndex( iLinkIndex, iNodeIndex );
			LossyLinkDebug.println( "Link vertex index : " + iNodeIndex[0] + ", " + iNodeIndex[1] );
		    }
		LossyLinkDebug.println("------------------------------------------\n");
	    }

        System.out.println( "Total number of links in topology : " + iLinkCount + '\n' );

        return iLinkCount;
    }


    //T vector : 1 - LossRate of each link.
    public void saveTvector(String strFileName)
    {
	if (strFileName == null)
        {
	    Calendar calendar = Calendar.getInstance();
	    strFileName = "LossyLink_T" + "_"
		    + calendar.get(Calendar.DAY_OF_MONTH) + "-"
		    + calendar.get(Calendar.MONTH) + "-"
		    + calendar.get(Calendar.YEAR) + "_"
		    + calendar.get(Calendar.HOUR) + "-"
		    + calendar.get(Calendar.MINUTE)
		    + ".out";

	    //strFileName = "LossyLink.out";
	}

	int iColCount = this.count();
	try
        {
	    FileWriter output = new FileWriter(strFileName);
	    output.write(Integer.toString(iColCount));
	    output.write('\n');

            int j = 0;//for dehbug
	    for (int i = 0; i < iColCount; ++i)
            {
                if( getCoverage( i ) == false )
                {
                    continue;
                }
                j++;
		double dLossRate = -1.0;
		output.write( Double.toString( getLossRate(i) ) );
		output.write('\n');
	    }
	    output.close();
	}
	catch (IOException e)
        {
	    System.err.println("ERROR LLL 00300 : saveTvector() File not written properely\n" +
			       e.toString());
	    System.exit(1);
	}
    }
}




class LinkData
{
       int iLinkData;
       double dWeight;
       double dLossRate;
       boolean bCoverage;        //1 if used in shortest path. 0 if not used.

       public LinkData()
       {
           iLinkData = -1;
	   dWeight = 0.0f;
	   dLossRate = -1.0f;
           bCoverage = false;
       }
       public LinkData( int iLinkData, double dWeight, double dLossRate, boolean bCoverage )
       {
           this.iLinkData = iLinkData;
	   this.dWeight = dWeight;
	   this.dLossRate = dLossRate;
           this.bCoverage = bCoverage;
       }

       void setLinkData( int iLinkData )
       {
           this.iLinkData = iLinkData;
       }
       void setWeight( double dWeight )
       {
	   this.dWeight = dWeight;
       }
       void setLossRate( double dLossRate )
       {
	   this.dLossRate = dLossRate;
       }
       void setCoverage( boolean bCoverage )
       {
           this.bCoverage = bCoverage;
       }

       int getLinkData()
       {
           return iLinkData;
       }
       double getWeight()
       {
	   return dWeight;
       }
       double getLossRate()
       {
	   return dLossRate;
       }
       boolean getCoverage()
       {
           return bCoverage;
       }
}


class TwowayHashtable
{
    Hashtable h1;
    Hashtable h2;

    public TwowayHashtable()
    {
	h1 = new Hashtable();
	h2 = new Hashtable();
    }

    Integer merge( int iDataInput[] )
    {
	if( iDataInput[0] < 0 || iDataInput[0] > 1000000
	    || iDataInput[1] < 0 || iDataInput[1] > 1000000 )
	    {
		LossyLinkDebug.println("merge() iDataInput[0] : " + iDataInput[0] + " iDataInput[1] : " + iDataInput[1] );
		return null;
	    }

	Integer iData = new Integer( iDataInput[0] + ( iDataInput[1]*1000000 ) );
	return iData;
    }

    void divide( int iDataInput, int iDataOutput[] )
    {
	iDataOutput[0] = iDataInput % 1000000;
	iDataOutput[1] = iDataInput / 1000000;
    }

    //h1 put
    public int put( Object objKey, int iData[] )
    {
	Integer intData;
	if( ( intData = merge( iData ) ) == null )
	    {
		return 0;
	    }

	//LossyLinkDebug.println( "Put() key : " + ((Integer)objKey).intValue() + " data[0] : " + iData[0] + " data[1] : " + iData[1] );
	if( h1.put( objKey, intData ) != null )
	    {
		System.out.println( "ERROR 00008" );
		return 0;
	    }
	if( h2.put( intData, objKey ) != null )
	    {
		System.out.println( "ERROR 00007" );
		return 0;
	    }
	return 1;
    }

    public int changeData( Object objKey, int iData[] )
    {
	Integer intData;
	if( ( intData = merge( iData ) ) == null )
	    {
		return 0;
	    }

	Object objPrevData = new Object();
	if( ( objPrevData = h1.put( objKey, intData ) ) == null )
	    {
		System.out.println( "ERROR 00009" );
		return 0;
	    }
	//IF not nessary
	if( h2.remove( objPrevData ) == null )
	    {
		System.out.println( "ERROR 00005" );
		return 0;
	    }
	if( h2.put( intData, objKey ) != null )
	    {
		System.out.println( "ERROR 00006" );
		return 0;
	    }
	return 1;
    }

    public int getData( Object objKey, int iData[] )
    {
	Object objData = new Object();

	if( ( objData = h1.get( objKey ) ) == null )
	    {
		System.out.println( "ERROR 00010" );
		return 0;
	    }

	divide( ((Integer)objData).intValue(), iData );
	return 1;
    }

    public int getKey( int iData[] )
    {
	Object objKey = new Object();
	Integer intData;
	if( ( intData = merge( iData ) ) == null )
	    {
		return -1;
	    }

	if( ( objKey = h2.get( intData ) ) == null )
	    {
		return -1;
	    }
	return ((Integer)objKey).intValue();
    }

    public int remove( Object objKey )
    {
	Object objData = new Object();

	if( ( objData = h1.get( objKey ) ) == null )
	    {
		System.out.println( "ERROR 00012" );
		return 0;
	    }
	if( h1.remove( objKey ) == null )
	    {
		System.out.println( "ERROR 00013" );
		return 0;
	    }
	if( h2.remove( objData ) == null )
	    {
		System.out.println( "ERROR 00014" );
		return 0;
	    }
	return 1;
    }

    public void reindex( int iNodeReindexTable[], int iNodeReindexTableCount )
    {
      int iH1Size = h1.size();
      int iH2Size = h2.size();
      Hashtable newH1 = new Hashtable( iH1Size );
      Hashtable newH2 = new Hashtable( h2.size() );
      int i = 0;
      int iKey[] = new int[iH1Size];
      Object objKey = new Object();
      int iData[][] = new int[iH2Size][2];

//Hashtable H1 Key : linkIndex, Data : nodeIndex[]
      int iNodeIndex[] = new int[2];
      Object objData = new Object();
      for( Enumeration enum = h1.keys(); enum.hasMoreElements(); ++i )
      {
        iKey[i] = ((Integer)enum.nextElement()).intValue();
      }
      LossyLinkSort.sortArray( iKey, 0 , iH1Size-1 );
      for( int j = 0;j < iH1Size; ++j )
      {
        objData = h1.get( new Integer(iKey[j]) );

        this.divide( ((Integer)objData).intValue(), iNodeIndex );
//        System.out.println("LinkIndex reindex() : iNodeIndex[0]before : " + iNodeIndex[0] + " iNodeIndex[1]before : " + iNodeIndex[1] );
        int iNewNodeIndex[] = new int[2];
        iNewNodeIndex[0] = iNodeReindexTable[ iNodeIndex[0] ];
        iNewNodeIndex[1] = iNodeReindexTable[ iNodeIndex[1] ];
//        System.out.println("LinkIndex reindex() : iNodeIndex[0]after : " + iNodeIndex[0] + " iNodeIndex[1]after : " + iNodeIndex[1] );
        // for debug
        if( iNewNodeIndex[0] == -1 || iNewNodeIndex[1] == -1 )
        {
            System.out.println("ERROR 00080" );
            System.exit( 0 );
        }
        // ---------
        newH1.put( new Integer( j ), this.merge( iNewNodeIndex ) );
//        newH1.put( new Integer( j ), objData );
      }
      h1 = newH1;



//Hashtable H2 Key : nodeIndex[], Data : linkIndex
      i = 0;
      for( Enumeration enum = h2.keys(); enum.hasMoreElements(); ++i )
      {
	  objKey = enum.nextElement();
	  iData[i][0] = ( (Integer) h2.get(objKey)).intValue();
	  iData[i][1] = ( (Integer) objKey).intValue();

          this.divide( iData[i][1], iNodeIndex );
//          System.out.println("LinkIndex reindex() : iNodeIndex[0]before : " + iNodeIndex[0] + " iNodeIndex[1]before : " + iNodeIndex[1] );
	  iNodeIndex[0] = iNodeReindexTable[iNodeIndex[0]];
          iNodeIndex[1] = iNodeReindexTable[iNodeIndex[1]];
//          System.out.println("LinkIndex reindex() : iNodeIndex[0]after : " + iNodeIndex[0] + " iNodeIndex[1]after : " + iNodeIndex[1] );
          // for debug
          if( iNodeIndex[0] == -1 || iNodeIndex[1] == -1 )
          {
              System.out.println( "ERROR 00081" );
          }
          // ---------
          iData[i][1] = this.merge( iNodeIndex ).intValue();
      }
      LossyLinkSort.sort2dArray( iData, 0, iH2Size-1 );
      for( int j = 0; j < iH2Size; ++j )
      {
          newH2.put( new Integer( iData[j][1] ), new Integer( j ) );
      }
      h2 = newH2;
    }
}


class LinkIndex
{
    TwowayHashtable linkIndexTable;
    int iLinkIndex;

    public LinkIndex()
    {
	linkIndexTable = new TwowayHashtable();
	iLinkIndex = -1;
    }

    public int set( int iNodeIndex[] )
    {
	int iRet = -1;
	if( ( iRet = nodeIndex2linkIndex( iNodeIndex ) ) != -1 )
	    {
		return iRet;
	    }
	iLinkIndex++;
        linkIndexTable.put( new Integer( iLinkIndex ), iNodeIndex );
	return iLinkIndex;
    }

    //This function updates one of two vertices of a link.
    public int updateEachLinkVertex( int iLinkIndex, int iOldNodeIndex, int iNewNodeIndex )
    {
      int iNodeIndex[] = new int[2];
      boolean bUpdate = false;

      if( linkIndex2nodeIndex( iLinkIndex, iNodeIndex ) == -1 )
      {
        System.out.println("ERROR LLL 00100 : UpdateEachLinkVertex() No corresponding link found in topology" );
        System.exit( 0 );
      }

      int iNodeIndex2[] = new int[2];
      linkIndex2nodeIndex( 4, iNodeIndex2 );

      if( iNodeIndex[0] == iOldNodeIndex )
      {
        iNodeIndex[0] = iNewNodeIndex;
        bUpdate = true;
      }
      else if( iNodeIndex[1] == iOldNodeIndex )
      {
        iNodeIndex[1] = iNewNodeIndex;
        bUpdate = true;
      }

      if( bUpdate )
      {
        linkIndexTable.changeData( new Integer( iLinkIndex ), iNodeIndex );
        return 1;
      }
      return 0;
    }

    public int get( int iLinkIndex, int iNodeIndex[] )
    {
	return linkIndexTable.getData( new Integer( iLinkIndex ), iNodeIndex );
    }

    public int curIndex()
    {
       return iLinkIndex;
    }

    public int maxIndex()
    {
       	return curIndex();
    }

    public int nodeIndex2linkIndex( int iNodeIndexInput[] )
    {
        return linkIndexTable.getKey( iNodeIndexInput );
    }

    public int linkIndex2nodeIndex( int iLinkIndexInput, int iNodeIndex[] )
    {
	return linkIndexTable.getData( new Integer( iLinkIndexInput ), iNodeIndex );
    }

    public int remove( int iLinkIndexInput )
    {
	return linkIndexTable.remove( new Integer( iLinkIndexInput ) );
    }

    public void reindex( int iNodeReindexTable[], int iNodeReindexTableCount )
    {
      linkIndexTable.reindex( iNodeReindexTable, iNodeReindexTableCount );
    }
}
