/**
 * <p>Title: LossyLinkLossRate</p>
 * <p>Description: The <CODE>LossyLinkLossRate</CODE> class is used in reporting
 * average number of links per path and median of links.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;


import java.io.*;
import java.util.*;



public class LossyLinkLossRate// implements Serializable, WNMMSConstants
{
    static LossRateData trueLink;
    static LossRateData inferredLink;
    static double dToleranceRate;

    static int iTrueLinkCount;
    static int iInferredLinkCount;
    static int iUncomputableLinkCount;

    static double UNCOMPUTABLE;

    static String strTvectorFile;
    static String strXvectorFile;
    static String strNewXvectorFile;
    static String strVirTvectorFile;
    static String strVirXvectorFile;
    static String strVirNewXvectorFile;

    static LossyLinkModel lossyLinkModel;

    public LossyLinkLossRate()
    {
        init();
    }

    static private void init()
    {
	//      dToleranceRate = 0.1;//10% of margin

      iTrueLinkCount = 0;
      iInferredLinkCount = 0;
      iUncomputableLinkCount = 0;

      UNCOMPUTABLE = -9.9999;
    }


    public static void main(String[] args)
    {
      LossyLinkDebug.setDebug(true);
      lossyLinkModel = new LossyLinkModel(1);

        parseInput( args );

        init();

        System.out.println("\n  Before virtualization ----------------------");

        //1. Evaluate X vector file
        System.out.println("    Specification for unreduced G matrix" );
        LossyLinkDebug.println("\n1. Evaluate ["+strTvectorFile+"] ["+strXvectorFile+"]" );
        {
	    open(strTvectorFile, strXvectorFile);
	    compareXtoT(dToleranceRate);
	    identifyLossyLink( false );
        }
/*
        init();

        //2. Evaluate new X vector file
        System.out.println("\n    Specification for reduced G matrix" );
        LossyLinkDebug.println("\n\n2. Evaluate ["+strTvectorFile+"] ["+strNewXvectorFile+"]" );
        {
            open(strTvectorFile, strNewXvectorFile);
            compareXtoT(dToleranceRate);
            identifyLossyLink( true );
        }

        init();

        System.out.println("\n  After virtualization ----------------------");

        //3. Evaluate virtualized X vector file
        System.out.println("    Specification for unreduced G matrix" );
        LossyLinkDebug.println("\n3. Evaluate ["+strVirTvectorFile+"] ["+strVirXvectorFile+"]" );
        {
            open(strVirTvectorFile, strVirXvectorFile);
            compareXtoT(dToleranceRate);
            identifyLossyLink( false );
        }

        init();

        //4. Evaluate virtualized new X vector file
        System.out.println("\n    Specification for reduced G matrix" );
        LossyLinkDebug.println("\n\n4. Evaluate ["+strVirTvectorFile+"] ["+strVirNewXvectorFile+"]" );
        {
            open(strVirTvectorFile, strVirNewXvectorFile);
            compareXtoT(dToleranceRate);
            identifyLossyLink( true );
        }
*/
    }


    public static int open( String strTrueFileName, String strInferredFileName )
    {
        int iArrSize = 0;//Total link count in the topology.
        int iComputableLinkCount = 0;//Computable link count, iComputableLinkCount <= iArrSize

        if( strTrueFileName == null && strInferredFileName == null )
        {
            System.out.println( "ERROR 00001" );
            return -1;
        }

        try
        {
            BufferedReader trueLossRateReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strTrueFileName))));

            String strTmp;
            if( ( strTmp = trueLossRateReader.readLine() ) != null )
            {
                iArrSize = Integer.parseInt( strTmp );
                trueLink = new LossRateData( iArrSize );
            }
            else
            {
                System.err.println("ERROR LLLR 00400 : Can not read input file ["+strTrueFileName+"]");
                System.exit(0);
            }

            for( int i = 0; i < trueLink.iCount && ( ( strTmp = trueLossRateReader.readLine() ) != null ); ++i )
            {
                trueLink.dLossRate[i] = Double.parseDouble( strTmp );
            }
            trueLossRateReader.close();
            iTrueLinkCount = trueLink.iCount;
        }
        catch (IOException e)
        {
            System.err.println( "LossyLinkLossRate : File not opened properely\n" + e.toString());
            System.exit(1);
        }

        // ----------------------------------------------

        try
        {
            BufferedReader inferredLossRateReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strInferredFileName))));

            String strTmp;
            //first line in X file indicates initial link count
            if( ( strTmp = inferredLossRateReader.readLine() ) != null )
            {
                iArrSize = Integer.parseInt( strTmp );
                inferredLink = new LossRateData( iArrSize );
            }
//            //second line in X file indicates computable link count
//            if( ( strTmp = inferredLossRateReader.readLine() ) != null )
//            {
//                iComputableLinkCount = Integer.parseInt( strTmp );
//                if( iArrSize < iComputableLinkCount )
//                {
//                    System.err.println("ERROR LLLR 00200 : Total link in topology < Computable link count" );
//                    return 0;
//                }
//            }
//            else
//            {
//                System.err.println("ERROR LLLR 00400 : Can not read input file ["+strTrueFileName+"]");
//                System.exit(0);
//            }
            iComputableLinkCount = iArrSize;
            iUncomputableLinkCount = iArrSize - iComputableLinkCount;
            iInferredLinkCount = inferredLink.iCount - iUncomputableLinkCount;
            if( iInferredLinkCount != iComputableLinkCount )
            {
              System.err.println("ERROR LLLR 00901");
              System.exit( 0 );
            }

            for( int i = 0; i < inferredLink.iCount && ( ( strTmp = inferredLossRateReader.readLine() ) != null ); ++i )
            {
                inferredLink.dLossRate[i] = Double.parseDouble( strTmp );
            }





/*
            for( int i = 0; i < iComputableLinkCount; ++i )
            {

                if ( (strTmp = inferredLossRateReader.readLine()) == null)
                {
                  System.err.println("ERROR LLLR 00903");
                  System.exit(0);
                }
                int iDelimeter = strTmp.lastIndexOf( "\t" );
                int iIndex = Integer.parseInt( strTmp.substring( 0, iDelimeter ) );
                if( inferredLink.dLossRate[iIndex] != UNCOMPUTABLE )
                {
                  System.err.println("ERROR LLLR 00907");
                  System.exit( 0 );
                }
                inferredLink.dLossRate[iIndex] = ( 1 - Math.exp( Double.parseDouble( strTmp.substring( iDelimeter + 1 ) ) ) );

                LossyLinkDebug.println("X vector : [" + iIndex + "] [" + inferredLink.dLossRate[iIndex] + "]");//for debug
            }
*/
            inferredLossRateReader.close();
        }
        catch (IOException e)
        {
            System.err.println( "LossyLinkLossRate : File not opened properely\n" + e.toString());
            System.exit(1);
        }

        return trueLink.iCount;
    }

    public static int compareXtoT( double dToleranceRate )
    {
        int iCoverage = 0;
        LossyLinkDebug.println("\t\n\nCompare X matrix to true link loss rate -------------------------------------------------" );
        if( trueLink.iCount != inferredLink.iCount )
        {
            System.out.println("\tERROR LLLR 00058 : Dimension mismatch between vector T and vector X.");
            System.out.println("\tMaybe you forgot to process in MATLAB?" );
            return 0;
        }


        for( int i = 0; i < iTrueLinkCount; ++i )
        {
            if( inferredLink.dLossRate[i] == UNCOMPUTABLE )
            {
                LossyLinkDebug.println("\tUncomputable link : dTrueLossRate[" + i + "]:" + rounding( trueLink.dLossRate[i], 3 ) +
                                   " dInferredLossRate[" + i +"]: UNCOMPUTABLE" );
            }
            else if( ( inferredLink.dLossRate[i] < trueLink.dLossRate[i] + trueLink.dLossRate[i] * dToleranceRate )
              &&( inferredLink.dLossRate[i] > trueLink.dLossRate[i] - trueLink.dLossRate[i] * dToleranceRate ) )
            {
                iCoverage++;
            }
            else
            {
                LossyLinkDebug.println("\tLossRate not accurate : TrueLossRate[" + i + "]:" + rounding( trueLink.dLossRate[i], 3 )
                                   + " InferredLossRate[" + i +"]:" + rounding( inferredLink.dLossRate[i], 3 ) );
            }
        }

        LossyLinkDebug.println("\t\nResult of comparing X matrix to true link loss rate ------------------------------------------" );
        LossyLinkDebug.println("\tTotal true link count : " + iTrueLinkCount );
        System.out.println("***Total inferred link count : " + iInferredLinkCount );
        LossyLinkDebug.println("\tTotal uncomputable link count : " + iUncomputableLinkCount );
        LossyLinkDebug.println("\tCorrectly inferred link count : " + iCoverage + "/" + iTrueLinkCount + ", +-"+dToleranceRate+" tolerance"  );

        return 1;
    }

    public static int identifyLossyLink( boolean bPrintExtendedInfo )
    {
        int iCoverage = 0;
        int iTrueLossyLinkCount = 0;
        int iInferredLossyLinkCount = 0;
        int iUncomputableLossyLinkCount = 0;
        int iFalsePositiveCount = 0;
        int iFalseNegativeCount = 0;

        LossyLinkDebug.println("\t\n\nIdentify lossy link ---------------------------------------------------------");

        if( trueLink.iCount != inferredLink.iCount )
        {
            System.out.println("\tERROR LLLR 00057 : Dimension mismatch between vector T and vector X.");
            System.out.println("\tMaybe you forgot to process in MATLAB?" );
            return 0;
        }

        LossyLinkDebug.println("\tLossy link's loss rate assigned : " + rounding( lossyLinkModel.dBadLinkLossRateMin, 3 ) +
                           " ~ " + rounding( lossyLinkModel.dBadLinkLossRateMax, 3 ) );

        for( int i = 0; i < iTrueLinkCount; ++i )
        {
            if( trueLink.dLossRate[i] >= lossyLinkModel.dBadLinkLossRateMin )
//                                       - lossyLinkModel.dBadLinkLossRateMin * dToleranceRate )
            {
                trueLink.iLossyFlag[i] = 1;
                iTrueLossyLinkCount++;
            }

            if( inferredLink.dLossRate[i] == UNCOMPUTABLE )
            {
                LossyLinkDebug.println("\tUncomputable link[" + i + "] : inferred : UNCOMPUTABLE" + " true : " + rounding( trueLink.dLossRate[i], 3 ) );
            }
            else if( inferredLink.dLossRate[i]
                     >=
                     (lossyLinkModel.dBadLinkLossRateMin ) - 0.01 )
		//                    - lossyLinkModel.dBadLinkLossRateMin * dToleranceRate ) )
	    {
		inferredLink.iLossyFlag[i] = 1;
		iInferredLossyLinkCount++;
	    }

            if( trueLink.iLossyFlag[i] == 1
                && inferredLink.iLossyFlag[i] == 1 )
            {
                iCoverage++;
            }
            else if( trueLink.iLossyFlag[i] == 1
                     && inferredLink.iLossyFlag[i] != 1 )
            {
                if( inferredLink.dLossRate[i] == UNCOMPUTABLE )
                {
                    LossyLinkDebug.println("\tUncomputable lossy link[" + i + "] : inferred : UNCOMPUTABLE" + " true : " + rounding( trueLink.dLossRate[i], 3 ) );
                    iUncomputableLossyLinkCount++;
                }
                else
                {
                    LossyLinkDebug.println("\tFalse negative lossy link[" + i + "] : inferred : " + rounding( inferredLink.dLossRate[i], 3 ) + " true : " + rounding( trueLink.dLossRate[i], 3 ) );
                    LossyLinkDebug.println("\t\tdiff : " + rounding( ( inferredLink.dLossRate[i] - lossyLinkModel.dBadLinkLossRateMin ), 3 ) + " than BadLinkLossRateMin\n" );
                    iFalseNegativeCount++;
                }
            }
            else if( trueLink.iLossyFlag[i] != 1
             && inferredLink.iLossyFlag[i] == 1 )
            {
                LossyLinkDebug.println("\tFalse positive lossy link[" + i + "] : inferred : " + rounding( inferredLink.dLossRate[i], 3 ) + " true : " + rounding( trueLink.dLossRate[i], 3 ) + "\n" );
                LossyLinkDebug.println("\t\tdiff : " + rounding( ( inferredLink.dLossRate[i] - lossyLinkModel.dBadLinkLossRateMin ), 3 ) + " than BadLinkLossRateMin\n" );
                iFalsePositiveCount++;
            }
        }

        LossyLinkDebug.println("\t\nResult of lossy link identification  -------------------------------------------------");

        if( !bPrintExtendedInfo )
	{
	    System.out.println("\tAbout " +
			      ( 100 * iTrueLossyLinkCount / iTrueLinkCount )  +
			       "% of links are lossy." ) ;

	    System.out.println("\t# of true lossy links :  \t\t" + iTrueLossyLinkCount );
        }
	System.out.println("\t# of inferred lossy links :  \t\t" + iInferredLossyLinkCount );
        System.out.println("\t# of uncomputable lossy links :  \t" + iUncomputableLossyLinkCount );
        System.out.println("\t# of overlap :  \t\t\t" +
                           iCoverage +
                           "\n\t\t(Between true lossy link and inferred lossy links, " +
                           "\n\t\tnot including uncomputable lossy links)" );
        if( bPrintExtendedInfo &&
	    iUncomputableLossyLinkCount > 0 &&
            ( ( iTrueLossyLinkCount - iUncomputableLossyLinkCount ) > 0 ) )
        {
            System.out.println("\tAccuracy excluding uncomputable lossy links :  " + ( 100 * iCoverage / ( iTrueLossyLinkCount - iUncomputableLossyLinkCount ) ) + "%" );
        }
        if( iTrueLossyLinkCount > 0 && iInferredLossyLinkCount > 0 )
        {
            System.out.println("\tAccuracy :  \t\t\t\t" + ( 100 * iCoverage / iTrueLossyLinkCount ) + "%" );

            if( iFalsePositiveCount != ( iInferredLossyLinkCount - iCoverage ) )
            {
              System.err.println("ERROR LLLR 00906");
              System.exit( 0 );
            }

            int iFalsePositiveRatio = ( 100 * iFalsePositiveCount / iInferredLossyLinkCount );
            System.out.println("\tFalse positive (ratio) : \t\t" +
                               iFalsePositiveCount +
                               " (" +
                               iFalsePositiveRatio +
                               "%)" );
        }

        return 1;
    }

    private static void showUsage()
    {
	System.err.println("\nUsage: \njava LossyLinkLossRate " +
		"<T_vector_file_name> <X_vector_file_name> <New_X_vecotr_file_name>\n" +
                "or java LossyLinkLossRate <Input_script_file>\n" +
		"\ni.e. java LossyLinkLossRate ../BRITE/topology/ASBarabasi100U_T.out ../BRITE/topology/ASBarabasi100U_X.out ../BRITE/topology/ASBarabasi100U_newX.out" +
                "\ni.e. java LossyLinkLossRate ASBarabasi100U.input" );
    }

    public static String rounding( double dNumber, int point )
    {
        String strNumber = Double.toString( dNumber );
        if( ( strNumber.indexOf( "E" ) != -1 )
           || ( strNumber.indexOf( "NaN" ) != -1 )
           || ( strNumber.indexOf( "Inf" ) != -1 ) )
        {
            return strNumber;
        }
        int iDelimeter = strNumber.lastIndexOf(".");
        int iOffset = iDelimeter + point + 1;
        if( iOffset > strNumber.length() )
        {
            iOffset = strNumber.length();
        }
        return strNumber.substring( 0, iOffset );
    }

    private static void parseInput( String args[] )
    {
        if( args.length < 1 || args.length > 3 )
        {
            System.err.println("ERROR LLLR 00301 : Wrong arguments");
            showUsage();
            System.exit(0);
        }

	if (args[0].indexOf(".input") != -1)
        {
	    if (args.length != 1)
            {
		System.err.println("ERROR LLLR 00300 : Too many arguments");
		showUsage();
		System.exit(0);
	    }
	    parseInputScriptFile(args[0]);
	}
	//direct input of T vector file and X vector file
	else
        {
	    if (args.length != 3)
            {
                System.err.println("ERROR LLLR 00302 : Wrong arguments");
		showUsage();
		return;
	    }
	    strTvectorFile = args[0];
            strXvectorFile = args[1];
            strNewXvectorFile = args[2];
	}
    }

    private static void parseInputScriptFile( String strFileName )
    {
        String strLine;
        try
        {
            BufferedReader inputScriptReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strFileName))));

            for( int i = 0; ( strLine = inputScriptReader.readLine() ) != null; ++i )
            {
                StringTokenizer st = new StringTokenizer( strLine, "\t\n\r= " );

                //loop for every token
                String strDefinition = null;
                String strValue = null;
                String strToken = null;
                while( st.hasMoreTokens() )
                {
                    strToken = st.nextToken();

                    //Ignore comments
                    if( strToken.startsWith( "//" ) == true )
                    {
                        break;
                    }

                    //Store script file fields into strDefinition and strValue
                    if( strDefinition == null )
                    {
                        strDefinition = strToken;
                        continue;
                    }
                    strValue = strToken;
                    if( strValue.length() == 0 )
                    {
                        System.err.println("ERROR LLLR 00121 : No value specified for " + strDefinition + " field" );
                        showUsage();
                        System.exit( 0 );
                    }
                    LossyLinkDebug.println("strDefinition : " + strDefinition );
                    LossyLinkDebug.println("strValue : " + strValue );

                    if( strDefinition.compareToIgnoreCase( "OutputFileName" ) == 0 )
                    {
                        strTvectorFile = strValue + "_T.out";
                        strXvectorFile = strValue + "_X.out";
                        strNewXvectorFile = strValue + "_newX.out";
                        strVirTvectorFile = strValue + "_virT.out";
                        strVirXvectorFile = strValue + "_virX.out";
                        strVirNewXvectorFile = strValue + "_virNewX.out";
                        break;
                    }
                    else if( strDefinition.compareToIgnoreCase( "Debug" ) == 0 )
                    {
                        if( strValue.compareTo( "1" ) == 0 )
                        {
                            LossyLinkDebug.setDebug( true );
                        }
                        break;
                    }
                }//end while
            }//end for
            inputScriptReader.close();
        }
        catch (IOException e)
        {
            System.err.println( "LossyLinkLossRate : File not opened properely\n" + e.toString());
            showUsage();
            System.exit(1);
        }
    }
}


/*
class LossRateData
{
    public double dLossRate[];
    public int iLossyFlag[];
    public int iCount = 0;

    public LossRateData( int iLinkCount )
    {
        dLossRate = new double[ iLinkCount ];
        iLossyFlag = new int[ iLinkCount ];
        iCount = iLinkCount;
    }
}
*/


