//// package LossyLink;

/**
 * <p>Title: LossyLinkNode</p>
 * <p>Description: The <CODE>LossyLinkNode</CODE> class is used by LossyLink*.java
 * to store and load node data using a hash tbl.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


// package LossyLink;



import java.io.*;
import java.util.*;
import cluster.*;


public class LossyLinkNode// implements Serializable, WNMMSConstants
{
    Hashtable nodeTable;
    NodeIndex nodeIndex;

    public LossyLinkNode()
    {
	nodeTable = new Hashtable();
	nodeIndex = new NodeIndex();
    }

    public int count()
    {
	return nodeTable.size();
    }

    public Enumeration keys()
    {
	return nodeTable.keys();
    }

    /* Stores a node data to hash table.
     *
     * Return Values
     * 0 : reserved for errors
     * 1 : the node already exists in the hashtable
     * 2 : the node is not in the hashtable, and the node has been stored.
     */
    public  int set( String strNode, int iIndegreeInput, int iOutdegreeInput, int iFlag, int iEndHost, boolean bReduceIp )
    {
	int iNodeIndex = 0;
	iNodeIndex = nodeIndex.set( strNode, bReduceIp );
	return set( iNodeIndex, iIndegreeInput, iOutdegreeInput, iFlag, iEndHost );
    }

    public  int set( int iNodeIndex, int iIndegreeInput, int iOutdegreeInput, int iFlag, int iEndHost )
    {
	//Input consistency chk
	if( iNodeIndex < 0 )
	    {
		System.out.println("setNode() iNodeIndex < 0 " );
		return -1;
	    }

	//Debug code


	//Check if there exists the same node in the hash tbl.
      	if( nodeTable.put( new Integer( iNodeIndex ), new LossyLinkNodeData( iIndegreeInput, iOutdegreeInput, iFlag, iEndHost ) )  == null )
	    {
	      	//If this is the first time we pass by strNode,
		//store it to the hashtable.
		//		LossyLinkDebug.println( "StoreNodeData : Node[" + strNode + "] Indegree[" + iIndegreeInput + "] Outdegree[" + iOutdegreeInput + "]" );
	       	return iNodeIndex;
	    }
	else
	    {
		//If the node is in the hashtable already,
		//do not store it in the hashtable.
		return iNodeIndex;
	    }
    }



    /* Loads a node data to hash table.
     *
     * Input Values
     * iOutput[0] : Indegree of the node
     * iOutput[1] : Outdegree of the node
     * iOutput[2] : Flag of the node
     * iOutput[3] : isEndHostFlag
     * Return Values
     * 0 : errors
     * 1 : ok
     *
     */
    public int get( String strNode, int iOutput[], boolean bReduceIp )
    {
	int iNodeIndex = -1;
	if( ( iNodeIndex = nodeIndex.ip2index( strNode, bReduceIp ) ) == -1 )
	    {
		return 0;
	    }

	return get( iNodeIndex, iOutput );
    }

    public int get( int iNodeIndex, int iOutput[] )
    {
	Object objNodeData = new Object();
	if( ( objNodeData = nodeTable.get( new Integer( iNodeIndex ) ) ) == null )
	    {
	        System.out.println("ERROR getDegree() failed" + iNodeIndex );
		return 0;
	    }
	iOutput[0] = ((LossyLinkNodeData)objNodeData).getIndegree();
	iOutput[1] = ((LossyLinkNodeData)objNodeData).getOutdegree();
	iOutput[2] = ((LossyLinkNodeData)objNodeData).getFlag();
        iOutput[3] = ((LossyLinkNodeData)objNodeData).getEndHost();
	return 1;
    }

    public int setDegree( int iNodeIndex, int iIndegree, int iOutdegree )
    {
      Object objData = new Object();
      objData = nodeTable.get( new Integer(iNodeIndex) );
      if( objData == null )
      {
        return -1;
      }

      ((LossyLinkNodeData)objData).setIndegree( iIndegree );
      ((LossyLinkNodeData)objData).setOutdegree( iOutdegree );

        return 1;
    }

    public int incIndegree( int iNodeIndex )
    {
      Object objData = new Object();
      objData = nodeTable.get( new Integer(iNodeIndex) );

      if( objData == null )
      {
        return -1;
      }

      ((LossyLinkNodeData)objData).setIndegree( ((LossyLinkNodeData)objData).getIndegree() + 1 );
      return 1;
    }

    public int incOutdegree( int iNodeIndex )
    {
      Object objData = new Object();
      objData = nodeTable.get( new Integer(iNodeIndex) );

      if( objData == null )
      {
        return -1;
      }

      ((LossyLinkNodeData)objData).setOutdegree( ((LossyLinkNodeData)objData).getOutdegree() + 1 );
      return 1;
    }

    public int decIndegree( int iNodeIndex )
    {
      Object objData = new Object();
      objData = nodeTable.get( new Integer(iNodeIndex) );

      if( objData == null )
      {
        return -1;
      }

      ((LossyLinkNodeData)objData).setIndegree( ((LossyLinkNodeData)objData).getIndegree() - 1 );
      return 1;
    }

    public int decOutdegree( int iNodeIndex )
    {
      Object objData = new Object();
      objData = nodeTable.get( new Integer(iNodeIndex) );

      if( objData == null )
      {
        return -1;
      }

      ((LossyLinkNodeData)objData).setOutdegree( ((LossyLinkNodeData)objData).getOutdegree() - 1 );
      return 1;
    }

    public void clearFlag()
    {
	int iNodeIndex = 0;
	int iDegree = 0;
	int iFlag = 0;
	Object objNodeData = new Object();
	Object objNodeIndex = new Object();

	for( Enumeration enumNode = nodeTable.keys(); enumNode.hasMoreElements(); )
        {
            objNodeIndex = enumNode.nextElement();
            objNodeData = nodeTable.get( objNodeIndex );
            ((LossyLinkNodeData)objNodeData).setFlag( 0 );
            nodeTable.put( objNodeIndex, objNodeData );
        }
    }


    public int print( )
    {
	Object objIndex = new Object();
        int iNodeCount = nodeTable.size();
	if( LossyLinkDebug.isDebug() == true )
        {
            LossyLinkDebug.println("print Node Hashtable ---------------------------\n");
            for( Enumeration enum = nodeTable.keys(); enum.hasMoreElements(); )
            {
                objIndex = enum.nextElement();
                LossyLinkDebug.println( "Node Index : " + ((Integer)objIndex).intValue() );
                LossyLinkDebug.println( "Ip : " + nodeIndex.index2ip( ((Integer)objIndex).intValue() ) +
                                        " iIndegree : " + ((LossyLinkNodeData)nodeTable.get( objIndex ) ).getIndegree() +
                                        " iOutdegree : " + ((LossyLinkNodeData)nodeTable.get( objIndex ) ).getOutdegree() +
                                        " iEndHost : " + ((LossyLinkNodeData)nodeTable.get( objIndex ) ).getEndHost() );
            }
            LossyLinkDebug.println("------------------------------------------\n");
        }

        System.out.println( "\nTotal number of nodes (including the root and the leaf nodes) in topology : " + iNodeCount );

        return iNodeCount;
    }


    public int remove( int iReduceableNode )
    {
        int iOutput[] = new int[4];
        get( iReduceableNode, iOutput );
        if( iOutput[3] == 1 )
        {
            System.out.println( "ERROR 021 : Attempting to delete an endHost" );
            return 0;
        }

	LossyLinkDebug.println( "Class lossyLinkNode, Removing node : " + iReduceableNode );
	if( nodeTable.remove( new Integer(iReduceableNode) ) == null )
        {
            System.err.println( "reduceNode() Can't remove strReduceableNode[0][1] <- reduceableNode" );
            return 0;
        }

	nodeIndex.remove( iReduceableNode );

	return 1;
    }


    public void reindex( int iNodeReindexTable[], int iNodeReindexTableCount )
    {
	int iNodeTableSize = nodeTable.size();
	Hashtable newNodeTable = new Hashtable(iNodeTableSize);
	int i = 0;
	int iKey[] = new int[iNodeTableSize];
	Object objData = new Object();
        int iTmp = this.count();

        if( iNodeReindexTable == null || iNodeReindexTableCount == 0 )
        {
            System.out.println("ERROR 00090" );
            return;
        }

        for ( int a = 0; a < iNodeReindexTableCount; ++a)
        {
            iNodeReindexTable[a] = -1;
        }



	for (Enumeration enum = nodeTable.keys(); enum.hasMoreElements(); ++i)
        {
	    iKey[i] = ( (Integer) enum.nextElement()).intValue();
	    newNodeTable.put(new Integer(i), objData);

	}
	LossyLinkSort.sortArray(iKey, 0, iNodeTableSize - 1);
	for (int j = 0; j < iNodeTableSize; ++j)
        {
	    objData = nodeTable.get(new Integer(iKey[j]));
	    newNodeTable.put(new Integer(j), objData);

            iNodeReindexTable[iKey[j]] = j;
	}
	nodeTable = newNodeTable;
        nodeIndex.reindex();
    }





// NODE DEGREE RELATED METHODS //////////////////////////////////////////////
    public int endHostDenotionByCount( int iEndHostCountMax )
    {
        Object objKey;
        Object objData;
        int iEndHostCount = 0;
        int iNodeIndex = -1;
        int iNodeCount = this.count();
        System.out.println("WARNING : endHostDenotionByCount() : Assuming minimum total degrees as 2." );

        for( Enumeration enum = nodeTable.keys(); enum.hasMoreElements(); )
        {
            objKey = enum.nextElement();
            objData = nodeTable.get( objKey );
            ( (LossyLinkNodeData) objData).setEndHost(0);
            nodeTable.put( objKey, objData );
        }

        //for 5 end hosts
        //153529     //default for 500, 1000 nodes
        //4444
        //345325
        //0
        //2 //this number with ASWaxman100_10 causes deficient case in virtualization
        //for 10 end hosts
        //54337
        //12345

        //28828  Waxman virtualization  ߻
	//345325
	//29931
        Random random = new Random( 29931 );
        while( iEndHostCount < iEndHostCountMax )
        {
            iNodeIndex = (int)( random.nextDouble() * ( iNodeCount-1 ) );
            objKey = new Integer( iNodeIndex );
            objData = nodeTable.get( objKey );
            if ( ( (LossyLinkNodeData) objData).getEndHost() == 0 &&
                 ( (LossyLinkNodeData) objData).getIndegree() +
                 ( (LossyLinkNodeData) objData).getOutdegree()
                <= 2 )
            {
                ( (LossyLinkNodeData) objData).setEndHost(1);
                nodeTable.put( objKey, objData );
                iEndHostCount++;
            }
        }

        //for debug
        if( LossyLinkDebug.isDebug() == true )
        {
            for( Enumeration enum = nodeTable.keys(); enum.hasMoreElements(); )
            {
                objKey = enum.nextElement();
                objData = nodeTable.get( objKey );
                System.out.println( "endHostDenotionByCount iNodeId : " + ((Integer)objKey).intValue() + " iEndHost : " + ((LossyLinkNodeData)objData).getEndHost() );
            }
        }

        return iEndHostCount;
    }



    public int endHostDenotionByFile( String strFileName )
    {
          int iEndHostCount = 0;

          String strFileLine;
          String strTmp;

          Object objKey;
          Object objData;

          int iRowCount = 0;
          int iColCount = 0;

          try
              {
                  BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(new FileInputStream(new File(strFileName))));
                  strFileLine = bufferedReader.readLine();
                  iRowCount = Integer.parseInt( strFileLine );
                  strFileLine = bufferedReader.readLine();
                  iColCount = Integer.parseInt( strFileLine );
                  while( (strFileLine = bufferedReader.readLine()) != null )
                      {
                        StringTokenizer st = new StringTokenizer( strFileLine, " " );
                        for( int i = 0; st.hasMoreTokens(); i++ )
                            {
                              Integer intNodeIndex = Integer.valueOf(st.nextToken());
                              objData = nodeTable.get( intNodeIndex );
                              ((LossyLinkNodeData)objData).setEndHost( 1 );
                              System.out.println("intNodeIndex : " + intNodeIndex );
                              nodeTable.put( intNodeIndex, objData );
                              iEndHostCount++;
                            }
                      }
                      bufferedReader.close();
              }
          catch( IOException e )
              {
                  System.err.println("LossyLinkVinfo : File format error\n" + e.toString() );
                  System.exit(1);
              }
          return iEndHostCount;
    }

    public int endHostDenotionByDegree( int iEndHostMaxDegreeInput )
    {
        int iEndHostCount = 0;
        Object objKey;
        Object objData;

//        for( Enumeration enum = nodeTable.keys(); enum.hasMoreElements(); )
        for( int i = 0; i < count(); i++ )
        {
/*
          if( iEndHostCount >= 100 )//iEndHostMaxDegreeInput )
          {
            break;
          }
*/

//            objKey = enum.nextElement();
//            objData = nodeTable.get( objKey );
            objData = nodeTable.get( new Integer(i) );
            if( iEndHostMaxDegreeInput == -1 )//every nodes are end host.
            {
                ((LossyLinkNodeData)objData).setEndHost( 1 );
                iEndHostCount ++;
            }
            else
            {
		if ( ( (LossyLinkNodeData) objData).getIndegree() +
		     ( (LossyLinkNodeData) objData).getOutdegree()
                    <= 2 )
                {
		    ( (LossyLinkNodeData) objData).setEndHost(1);
                    iEndHostCount ++;
                    System.out.println("EndHost Key" + i );
		}
		else
                {
		    ( (LossyLinkNodeData) objData).setEndHost(0);
		}
            }
//            nodeTable.put( objKey, objData );
            nodeTable.put( new Integer(i), objData );
        }

        //for debug
        if( LossyLinkDebug.isDebug() == true )
        {
            for( Enumeration enum = nodeTable.keys(); enum.hasMoreElements(); )
            {
                objKey = enum.nextElement();
                objData = nodeTable.get( objKey );
                System.out.println( "endHostDenotionByDegree iNodeId : " + ((Integer)objKey).intValue() + " iEndHost : " + ((LossyLinkNodeData)objData).getEndHost() );
            }
        }

        return iEndHostCount;
    }

    public int isEndHost( int iNodeIndex )
    {
        Object objData = new Object();
        if( ( objData = nodeTable.get( new Integer( iNodeIndex ) ) ) != null )
        {
            return ((LossyLinkNodeData)objData).getEndHost();
        }
	return -1;
    }


    public void saveDegree( String strFileName )
    {
	int iOutput[] = new int[4];
	String strLine = new String();
	int iDegreeArrayLength = this.count();
	int iDegreeArray[] = new int[iDegreeArrayLength]; //Array of node index sorted by its degrees

	if( strFileName == null )
	{
	    strFileName = "LossyLinkDegree.out";
	}

	for( int i = 0; i < iDegreeArrayLength; ++i )
	    {
		this.get( i, iOutput );
		iDegreeArray[i] = ( iOutput[0] + iOutput[1] );
	    }
	LossyLinkSort.sortArray( iDegreeArray, 0, iDegreeArrayLength - 1 );

	try
	   {
		FileWriter output = new FileWriter( strFileName );

	        int iCurrentDegree = 0;
		int iCurrentDegreeCount = 0;
		for( int i = 0; i < iDegreeArrayLength; ++i )
		    {
			if( iDegreeArray[i] == iCurrentDegree )
			{
			    iCurrentDegreeCount++;
			}
			else
			{
			    strLine = String.valueOf( iCurrentDegree ) + '\t' + String.valueOf( iCurrentDegreeCount ) + '\n';
//			    LossyLinkDebug.println( "Degree output : " + strLine );
			    output.write( strLine );
			    iCurrentDegree = iDegreeArray[i];
			    iCurrentDegreeCount++;
			}
		    }

		output.close();
	    }
	catch( IOException e )
	    {
		System.err.println("LossyLinkNode : SaveDegree() File not written properely\n" + e.toString() );
		System.exit(1);
	    }
    }
}






/*
 *
 *
 *
 */
class LossyLinkNodeData// implements Serializable, WNMMSConstants
{
    int iIndegree;
    int iOutdegree;
    int iFlag;
    int iEndHost;//A flag for indicating whether this node is an end host or not.
    public int iOriginalIndex;

    public LossyLinkNodeData()
    {
	iIndegree = 0;
	iOutdegree = 0;
	iFlag = 0;
        iEndHost = 0;
        iOriginalIndex = 0;
    }

    public LossyLinkNodeData( int iIndegree, int iOutdegree, int iFlag, int iEndHost )
    {
	this.iIndegree = iIndegree;
	this.iOutdegree = iOutdegree;
	this.iFlag = iFlag;
        this.iEndHost = iEndHost;
    }

    public void set( LossyLinkNodeData nodeData )
    {
	iIndegree = nodeData.iIndegree;
	iOutdegree = nodeData.iOutdegree;
	iFlag = nodeData.iFlag;
        iEndHost = nodeData.iEndHost;
    }

    public void setIndegree( int iIndegree )
    {
	this.iIndegree = iIndegree;
    }
    public void setOutdegree( int iOutdegree )
    {
	this.iOutdegree = iOutdegree;
    }

    public int getIndegree()
    {
	return iIndegree;
    }
    public int getOutdegree()
    {
	return iOutdegree;
    }

    public void setFlag( int iFlag )
    {
	this.iFlag = iFlag;
    }

    public int getFlag()
    {
	return iFlag;
    }

    public void setEndHost( int iEndHost )
    {
        this.iEndHost = iEndHost;
    }

    public int getEndHost()
    {
        return iEndHost;
    }
}













class NodeIndex
{
    Hashtable nodeIndexTable;
    int iNodeIndex;

    public NodeIndex()
    {
	nodeIndexTable = new Hashtable();
	iNodeIndex = -1;
    }

    public int set( String strIp, boolean bReduceIp )
    {
	int iRet = -1;
	if( ( iRet = ip2index( strIp, bReduceIp )) != -1 )
	    {
		return iRet;
	    }

	if( bReduceIp )
	    {
		strIp = reduceIp( strIp );
	    }

	iNodeIndex++;
	LossyLinkDebug.println( "iNodeINdex : " + iNodeIndex );
	nodeIndexTable.put( strIp, new Integer( iNodeIndex ) );
	return iNodeIndex;
    }

    public int curIndex()
    {
     	return iNodeIndex;
    }

    public int maxIndex()
    {
       	return iNodeIndex;
    }

    public int ip2index( String strIp, boolean bReduceIp )
    {
	if( bReduceIp )
	    {
		strIp = reduceIp( strIp );
	    }

	Object objIndex = new Object();

	if( ( objIndex = nodeIndexTable.get( strIp ) ) == null )
	    {
		return -1;
	    }
	return ((Integer)objIndex).intValue();
    }

    public String index2ip( int iIndexInput )
    {
	Object objIP = new Object();
	int iIndex = -1;

	for( Enumeration enum = nodeIndexTable.keys(); enum.hasMoreElements(); )
	    {
		objIP = enum.nextElement();
		iIndex = ((Integer)nodeIndexTable.get( objIP )).intValue();
		if( iIndex == iIndexInput )
		    {
			return objIP.toString();
		    }
	    }
	return null;
    }

    static String reduceIp( String strIP )
    {
	int iLastDotIndex = 0;

	if( ( iLastDotIndex = strIP.lastIndexOf( "." ) ) == -1 )
	    {
		System.out.println( "ReduceIP(), No dot found in a IP addr" );
		return null;
	    }
	int iLastByte = Integer.parseInt( strIP.substring(iLastDotIndex+1) );
	iLastByte = iLastByte >> 3;

        return new String( strIP.substring(0, iLastDotIndex+1) + Integer.toString( iLastByte ));
    }

    public int remove( int iReduceableNode )
    {
        Object objIP = new Object();
        int iIndex = -1;

        for( Enumeration enum = nodeIndexTable.keys(); enum.hasMoreElements(); )
            {
                objIP = enum.nextElement();
                iIndex = ((Integer)nodeIndexTable.get( objIP )).intValue();
                if( iIndex == iReduceableNode )
                    {
                        System.out.println("LossyLinkNodeIndex, removing node : "+iReduceableNode);
                        nodeIndexTable.remove( objIP );
                        return 1;
                    }
            }
        return 0;
    }

    public void reindex()
    {
      int iNodeIndexTableSize = nodeIndexTable.size();
      Hashtable newNodeIndexTable = new Hashtable( iNodeIndexTableSize );
      int i = 0;
      int iKey[] = new int[iNodeIndexTableSize];
      Object objData = new Object();
      for( Enumeration enum = nodeIndexTable.keys(); enum.hasMoreElements(); ++i )
          {
	      iKey[i] = ((Integer)enum.nextElement()).intValue();
              newNodeIndexTable.put( new Integer(i), objData );
          }
      LossyLinkSort.sortArray( iKey, 0 , iNodeIndexTableSize-1 );
      for( int j = 0;j < iNodeIndexTableSize; ++j )
          {
	      objData = nodeIndexTable.get(new Integer(iKey[j]));
	      newNodeIndexTable.put(new Integer(j), objData);
	  }
      nodeIndexTable = newNodeIndexTable;
    }

}


