/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

import java.io.*;
import java.util.*;

public class LossyLinkVirtualize
{
  private LossyLinkPathInfo path;
  private LossyLinkData data;
  private boolean bVirtualizeDataInit;
  private int iVirtualizeData[][];        //Virtualize Data
                                  //Assume there's two nodes L0, L1 surrounding a node n0.
                                  //iVirtualizeData[0] = [ L0, L1 ]
                                  //iVirtualizeData[1] = [ L1, L2 ]
                                  //iVirtualizeData[2] = [ L2, L0 ]

  public LossyLinkVirtualize( LossyLinkPathInfo path, LossyLinkData data )
  {
    bVirtualizeDataInit = false;
    this.path = path;
    this.data = data;
  }


  //Note : This function contains some portions that is dependent to directed/undirected links
  public void virtualize()
  {
    int iCenterNodeDegree = -1;     //Degree of the node in the center( center node's degree )
    int iCenterNodeIndex = -1;
    int iNodeData[] = new int[4];   //temporal space to store the result of data.node.get()

    int iPathCountToBeSaturated = 0;//Indicates how many paths are needed to be a fully linked set.
    int iPathCountInSet = 0;

    int iLinkIndex = -1;
    //Pick each node
    for( Enumeration enumNode = data.node.nodeTable.keys(); enumNode.hasMoreElements(); )
    {
      iCenterNodeIndex = ((Integer)enumNode.nextElement()).intValue();
      data.node.get( iCenterNodeIndex, iNodeData );

      iCenterNodeDegree = iNodeData[0] + iNodeData[1];
      if( iNodeData[3] == 0 &&       //If node is internal node.
          iCenterNodeDegree >= 3 )   //If node's degree is greater than 3.
      {
        iPathCountToBeSaturated = iCenterNodeDegree * ( iCenterNodeDegree - 1 ) / 2; //Assume undirected graph.
        iPathCountInSet = findLinkSetsInPaths( iCenterNodeIndex, iPathCountToBeSaturated );

        //If ( iPathCountInSet == iCenterNodeDegree ), than it is definitely
        //not a virtualizable case. Every node/link are used in some paths.
        if( iPathCountInSet == iCenterNodeDegree - 1 )
        {
          //Maybe a virtualizable case.

          //Initialize
          int iTargetLink = -1;
          int iMatchCount = -1;
          int iRemovableLink = -1;
          int iUpdatableLink[] = new int[iPathCountInSet];
          boolean bBreak = false;

          for( int i = 0; bBreak == false && i < iVirtualizeData.length; ++i )
          {
            for( int j = 0; bBreak == false && j < iVirtualizeData[0].length; ++j )
            {
              iTargetLink = iVirtualizeData[i][j];
              iMatchCount = searchIdenticalLinksInSet( iTargetLink );
              if( iMatchCount == iPathCountInSet )
              {
                iRemovableLink = iTargetLink;
                if( searchUnidenticalLinksInSet( iTargetLink, iUpdatableLink )
                    != iPathCountInSet )
                {
                  System.err.println("ERROR LLV 00300" );
                  System.exit( 0 );
                }
                mergeLinks( iCenterNodeIndex, iRemovableLink, iUpdatableLink );
                System.out.println("LLV : Virtualized. Node["+iCenterNodeIndex+"] iPathCountInSet : " + iPathCountInSet + "\tiCenterNodeDegree : " + iCenterNodeDegree + "\tiRemovableLink : " + iRemovableLink );
                bBreak = true;
              }
            }
          }
        }
        else if( iPathCountInSet < iCenterNodeDegree - 1  )
        {
          System.out.println("LLV : Possibly deficient case. Skipped... Node["+iCenterNodeIndex+"] iPathCountInSet : " + iPathCountInSet + "\tiCenterNodeDegree : " + iCenterNodeDegree );
        }
        else//for debug
        {
          System.out.println("LLV : Acceptable case... Node["+iCenterNodeIndex+"] iPathCountInSet : " + iPathCountInSet + "\tiCenterNodeDegree : " + iCenterNodeDegree );
        }
      }
    }
  }


  //output : Array of unidentical links to iLinkToFind
  private int searchUnidenticalLinksInSet( int iTargetLink, int iUnidenticalLinks[] )
  {
    int iUnidenticalLinksCount = 0;
    boolean bBingo = true;

    if( bVirtualizeDataInit == false )
    {
      System.err.println("ERROR LLV 00200 : Attempted to use iVirtualizeData before initialization." );
      System.exit( 0 );
    }

    for( int a = 0; a < iVirtualizeData.length; ++a )
    {
      for( int b = 0; b < iVirtualizeData[0].length; ++b )
      {
        if( iTargetLink == iVirtualizeData[a][b] )
        {
          bBingo = false;
        }
        for( int c = 0; c < iUnidenticalLinksCount; ++c )
        {
          if( iUnidenticalLinks[c] == iVirtualizeData[a][b] )
          {
            bBingo = false;
          }
        }
        if( bBingo )
        {
          iUnidenticalLinks[iUnidenticalLinksCount] = iVirtualizeData[a][b];
          iUnidenticalLinksCount++;
        }
        bBingo = true;
      }
    }

    return iUnidenticalLinksCount;
  }

  //output : # of identical link to iLinkToFind
  private int searchIdenticalLinksInSet( int iLinkToFind )
  {
    int iMatchCount = 0;

    if( bVirtualizeDataInit == false )
    {
      System.err.println("ERROR LLV 00201 : Attempted to use iVirtualizeData before initialization." );
      System.exit( 0 );
    }

    for( int a = 0; a < iVirtualizeData.length; ++a )
    {
      for( int b = 0; b < iVirtualizeData[0].length; ++b )
      {
        if( iLinkToFind == iVirtualizeData[a][b] )
        {
          iMatchCount ++;
        }
      }
    }

    return iMatchCount;
  }

  private int findLinkSetsInPaths( int iCenterNodeIndex, int iPathCountToBeSaturated )
  {
    LossyLinkPathData pathData;
    int iPathListLength = path.pathList.size();
    int j = 0;//j indicates the number of valid iLinkSetData rows.
              //That is the number if link sets found so far.
    //Set row count of iLinkSetData to be the maximum size possible.
    //Later, we'll only copy valid data to iVirtualizeData.
    int iLinkSetData[][] = new int[iPathCountToBeSaturated][2];


    for( int i = 0; i < iPathListLength; i++ )
    {
      boolean bIgnoreThisLinkSet = false;

      if( j == iLinkSetData.length )
      {
        return j;
      }
      pathData = (LossyLinkPathData)path.pathList.get( i );
      if( findALinkSetInAPath( pathData, iCenterNodeIndex, iLinkSetData[j], LossyLinkLink.UNDIRECTED ) == 1 )
      {
        //Check if we found a path same as one we retrieved before.
        for( int a = 0; a < j; a++ )//cycle for number of retrived node sets
        {
          int iSame = 0;
          for( int b = 0; b < iLinkSetData[0].length; b++ )
          {
            if( iLinkSetData[j][b] == iLinkSetData[a][0] )
              iSame++;
            else if( iLinkSetData[j][b] == iLinkSetData[a][1] )
              iSame++;
          }

          //if iSame == 2, We found a path that is already in iVirtualizeData array, discard it.
          if( iSame == iLinkSetData[0].length )
          {
            bIgnoreThisLinkSet = true;
            break;
          }
        }
        //end check

        if( bIgnoreThisLinkSet == false )
        {
          j++;
        }
      }
    }

    //Initialize and fill iVirtualizeData by valid iLinkSetData fields.
    iVirtualizeData = new int[j][2];
    bVirtualizeDataInit = true;
    for( int x = 0; x < j; ++x )
    {
      for( int y = 0; y < iLinkSetData[0].length; ++y )
      {
        iVirtualizeData[x][y] = iLinkSetData[x][y];
      }
    }

    return j;
  }

  //findALinkSetInAPath : finds a node and it's neighbouring two other nodes from a path.
  //
  // Notice :
  //          This function's behavior should be different depending whether the graph
  //          is directed or undirected one, but for now I only take care of undirected graph.
  //
  // Input :
  //          bUndirectedGraph : a flag telling whether the link is directed or undirected
  //
  // Output :
  //          iOutput[0] : a node that is located right before the specified node in the path.
  //          iOutput[1] : the specified node itself.
  //          iOutput[2] : a node that is located right after the specified node in the path.
  //
  private int findALinkSetInAPath( LossyLinkPathData pathData,
                                   int iCenterNodeIndex,
                                   int iOutput[],
                                   boolean bUndirectedGraph )
  {
      int iLinkIndex[] = new int[2];//pick 2 links.
      int iNodeIndex[] = new int[2];//in each link, there are 2 nodes.

      if( bUndirectedGraph == false )
      {
        System.err.println("ERROR LLV 00004 : Not implemented for directed graphs yet" );
        System.exit( 0 );
      }


      iLinkIndex[0] = pathData.getFirstLinkIndex();
      for( int i = 0; i < pathData.getLinkCount(); i++ )
      {
        if( iLinkIndex[0] == -1 )
        {
          System.err.println("ERROR LLV 00001");
          System.exit( 0 );
        }
        data.link.linkIndex.linkIndex2nodeIndex( iLinkIndex[0], iNodeIndex );
        if( iCenterNodeIndex == iNodeIndex[0] || iCenterNodeIndex == iNodeIndex[1] )
        {
          //found
          iOutput[0] = iLinkIndex[0];

          iLinkIndex[1] = pathData.getNextLinkIndex();
          data.link.linkIndex.linkIndex2nodeIndex( iLinkIndex[1], iNodeIndex );
          iOutput[1] = iLinkIndex[1];

          //for debug
          if( iOutput[0] == iOutput[1] )
          {
            System.err.println( "ERROR LLV 00002" );
            System.exit( 0 );
          }

          return 1;
        }
        iLinkIndex[0] = pathData.getNextLinkIndex();
      }
      return 0;
  }



  private int mergeLinks( int iCenterNodeIndex, int iRemovableLink, int iUpdatableLink[] )
  {
    if( bVirtualizeDataInit == false )
    {
      System.err.println("ERROR LLV 00206 : Attempted to use iVirtualizeData before initialization." );
      System.exit( 0 );
    }

    int iNewLinkIndex[] = new int[iUpdatableLink.length];
    double dNewLinkLossRate[] = new double[iUpdatableLink.length];
    double dNewLinkWeight[] = new double[iUpdatableLink.length];
    int iNewCenterNodeIndex = -1;
    boolean bNewLinkCoverage[] = new boolean[iUpdatableLink.length];
    int iNodeIndex[] = new int[2];//for temporal use.

    //Reindex table init.
    data.iLinkReindexTableCount = data.link.linkIndex.maxIndex() + 1;
    data.iLinkReindexTable = new int[data.iLinkReindexTableCount];
    data.iNodeReindexTableCount = data.node.count() + 1;
    data.iNodeReindexTable = new int[data.iNodeReindexTableCount];


    //determine new center node
    data.link.linkIndex.linkIndex2nodeIndex( iRemovableLink, iNodeIndex );
    if( iNodeIndex[0] == iCenterNodeIndex )
    {
      iNewCenterNodeIndex = iNodeIndex[1];
    }
    else if( iNodeIndex[1] == iCenterNodeIndex )
    {
      iNewCenterNodeIndex = iNodeIndex[0];
    }

    //set new parameters for newely replaced center node.
    int iCenterNodeData[] = new int[4];
    int iNewCenterNodeData[] = new int[4];
    data.node.get( iCenterNodeIndex, iCenterNodeData );
    data.node.get( iNewCenterNodeIndex, iNewCenterNodeData );
    data.node.setDegree( iNewCenterNodeIndex,
                         iCenterNodeData[0] + iNewCenterNodeData[0] - 1,
                         iCenterNodeData[1] + iNewCenterNodeData[1] - 1 );
    //

    //Pick sets of mergeable links.
    for( int i = 0; i < iUpdatableLink.length; i++)
    {
      //set new parameters for newely created link.
      dNewLinkWeight[i] = data.link.getWeight( iUpdatableLink[i] )
                        + data.link.getWeight( iRemovableLink );
      if( data.link.getCoverage( iUpdatableLink[i] ) == true &&
          data.link.getCoverage( iRemovableLink ) == true )
      {
          bNewLinkCoverage[i] = true;
      }
      dNewLinkLossRate[i] = 1 - ( 1 - data.link.getLossRate( iUpdatableLink[i] )
                              * ( 1 - data.link.getLossRate( iRemovableLink ) ) );
      data.link.set( iUpdatableLink[i], dNewLinkWeight[i], dNewLinkLossRate[i], bNewLinkCoverage[i] );
      //end parameter settings
    }
    data.link.updateLinkVertex( iCenterNodeIndex, iNewCenterNodeIndex );
    data.link.remove( iRemovableLink );
    data.node.remove( iCenterNodeIndex );
    path.removeLink( iRemovableLink );

    //reindex
    data.node.reindex( data.iNodeReindexTable, data.iNodeReindexTableCount );
    data.link.reindex( data.iLinkReindexTable, data.iLinkReindexTableCount,
                       data.iNodeReindexTable, data.iNodeReindexTableCount );
    path.reindex( data );

    return 0;
  }
}
