function  RunDeleteNodes( inputScriptFile )

%retValue returns if the RunDeleteAddNodes completed successfully
%or not.

fis = fopen( inputScriptFile, 'r' );
if fis == -1
    fprintf('ERROR : Cannot read file %s', inputScriptFile );
    return;
end



%-----------------------------------
% Parse Input file
%-----------------------------------
while 1
    [strRead, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
    
    if readCount == 0
       break;
    end
    
    if strcmpi( strRead, 'num_end_hosts' ) == 1
       [strEqual, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
       if readCount == 0
	  fprintf('ERROR NUM_DELETENODES1 : Unexpected end of line');
	  return;
       end
       [strValue, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
       if readCount == 0
	  fprintf('ERROR NUM_DELETENODES2 : Can not read [%s] name from input file script [%s]', strValue, inputScriptFile );
	  return;
       end
       num_end_hosts = str2num( strValue );
    end

    if strcmpi( strRead, 'outputFileName' ) == 1
        [strEqual, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
	if readCount == 0
	    fprintf('ERROR NUM_DELETENODES3 : Unexptected end of line');
	    return;
	end
	
	[strValue, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
	if readCount == 0
	    fprintf('ERROR NUM_DELETENODES4 : Can not read [%s] name from input script file [%s]', strValue, inputScriptFile );
            return;
         end
         
         strGfile = strcat( strValue, '_G.out' );
         strIfile = strcat( strValue, '_I.out' );
         strBfile = strcat( strValue, '_B.out' );
         strBbarFile = strcat( strValue, '_Bbar.out' );
         strMatFile = strValue;
         
     end
     if strcmpi( strRead, 'delete_num_nodes' )
	[strEqual, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
	if readCount == 0
	   fprintf('ERROR NUM_DELETENODES5 : Unexpected end of line');
	   return;
	end
	
	[strValue, readCount] = fscanf( fis, '%s[\t\n\r\f= ]' );
	if readCount == 0
            fprintf('ERROR NUM_DELETENODES6 : Can not read [%s] name from input script file [%s]', strValue, inputScriptFile );
	    return;
	end

	del_num_nodes = str2num( strValue );
        del_num_nodes_str = strValue;
	
     end
     
end
fclose( fis );


%---------------
%After parsing we should have the following in matlab.
%     num_end_hosts : Number of end hosts for matlab.  Num of
%     endhosts on the overlay network.
%     
%     rem_num_nodes : Number of nodes to add to the overlay
%     topology.
%     
%     strGfile :
%     strIfile :
%     strBfile :
%     strBbarFile :
%     strMatFile :
%---------------


%Program Pesudo
%---------------
%1)Recreate the simiulation as if we were runnig a simulation from
%scratch.
%2)Save those results
%3)loop Then incremently remove n nodes
%4) save those files
%5)end loop
%---------------


load ../../scripts/lucent.mat
%load test.mat

%----------------
%After load we should have the following in matlab.
%     end_hosts - is a row vector of consisting of end hosts
%     adj - is a sparse matrix which defines if adj(i, j) 1 if
%           a link exists between both i and j.
%     A - where A(i, j) is the label of that edge.


%----------------
%Generate the random permutations
%

perm = randperm(length(end_hosts));%indicies for end_hosts in
                                   %random order.
end_hosts_on_overlay = end_hosts(perm(1:num_end_hosts));

topologyTypeFlag = 0;
%-----------------


%Compute G in full matrix form
param = [];
param.precision = 'double';
%param.layout    = 'packed';
param.layout    = 'full';
param.tol       = 1e-5;

G = pathmatrix(end_hosts_on_overlay, adj, A);
%-----------------------------
size_G = size( G )

[I,R] = rowselect(G, param);

Gbar = G(I,:);


%--------------------------------------------------
%CHECKPOINT 1 REACHED -- SAVE INITALIZED TOPOLOGY
%__________________________________________________
%DESC: Now we are done with the first lucent batch
%      At this point we have not added nodes to the
%      topology.  We merely did a RunLucent()
%      call to generate the standard files we need
%      to process the results.   Now we need to add
%      the nodes to the topology



fprintf('\nNow deleting nodes to overlay ------------\n');

%Deleting a node means removing every path related to the node.
index_of_end_hosts_to_delete = randperm( length(end_hosts_on_overlay ));
     fprintf('Index_of_end_hosts_to_delete: ');
     disp( index_of_end_hosts_to_delete );
     fprintf('\n');
end_hosts_to_delete = end_hosts_on_overlay( index_of_end_hosts_to_delete );
     fprintf('End_hosts_to_delete: ');
     disp( end_hosts_to_delete );
     fprintf('\n');

for iNodeIndex = end_hosts_to_delete
  fprintf('Node index to be deleted [%d]\n', iNodeIndex );
  iPathsFromTheNode = A( iNodeIndex, : );
  iPathsToTheNode   = A( :, iNodeIndex );
  for iPathIndex = iPathsFromTheNode
    fprintf('Path index to be deleted [%d]\n', iPathIndex );
    [G, Gbar, R, kk] = deletepath(G, Gbar, R, iPathIndex, param);
    fprintf('kk = [%d]\n', kk );
  end
end




stat_endHostCount = length(end_hosts);
stat_pathCount = size(G, 1)
stat_overlayEndHostCount = length(end_hosts_on_overlay);

fprintf('size of I with new nodes.\n');

size_I = size(I)

fprintf('size of Gbar with new nodes.\n');
size_Gbar = size(Gbar)


saveG( G,  strGfile );

saveI( I,  strIfile );

save( strMatFile  , 'G', 'Gbar', 'R', 'I', 'param', ...
                  'stat_endHostCount', ...
                  'stat_overlayEndHostCount', ...
                  'stat_pathCount' );















