function [V, R] = RunPath( inputScriptFile, isWindows )

% Open input script file --------------------------------------------------
fid = fopen( inputScriptFile, 'r' );
if fid == -1
    fprintf('ERROR : Can not read file %s', inputScriptFile );
end

readEndFlag = 0;


while 1
    [strRead, readCount] = fscanf( fid, '%s[\t\n\r\f= ]' );
    
    if readCount == 0
        break;
    end

    strOutputFileSuffix = 'Bernoulli';    %default setting
    if strcmpi( strRead, 'LossProcess' ) == 1
        [strEqual, readCount] = fscanf( fid, '%s[\t\n\r\f= ]' );
        if readCount == 0
            fprintf('ERROR 000111 : Unexpected end of line');
            return;
        end
        
        [strValue, readCount] = fscanf( fid, '%s[\t\n\r\f= ]' );
        if readCount == 0
            fprintf('ERROR 000112 : Can not read [%s] name from input script file [%s]', strValue, inputScriptFile );
            return;
        end
        
        strOutputFileSuffix = strValue;
    end
    
    if strcmpi( strRead, 'outputFileName' ) == 1
        [strEqual, readCount] = fscanf( fid, '%s[\t\n\r\f= ]' );
        if readCount == 0
            fprintf('ERROR 000011 : Unexpected end of line');
            return;
        end
        
        [strValue, readCount] = fscanf( fid, '%s[\t\n\r\f= ]' );
        if readCount == 0
            fprintf('ERROR 000012 : Can not read [%s] name from input script file [%s]', strValue, inputScriptFile );
            return;
        end

        strBbarFile = strcat( strValue, '_Bbar.out');
        strBhatFile = strcat( strValue, '_Bhat.out');
        strIfile   = strcat( strValue, '_I.out' );
        strMatFile = strValue;
        strOutputFile = strrep( inputScriptFile, '.input', '.out');
        strOutputFileDir = strcat( '../../BRITE/topology/out', strOutputFileSuffix );
        strOutputFileDir = strcat( strOutputFileDir, '/' );
        strOutputFile = strcat( strOutputFileDir, strOutputFile ) %%for debug
        
        readEndFlag = readEndFlag + 1;
    end
end

fclose( fid );



if readEndFlag ~= 1
    fprintf( 'ERROR 00002 : Input script file [%s] does not contain every information needed [%d]', inputScriptFile, readEndFlag );
    return;
end



% for MS Windows - Unix compatibility -------------------
if isWindows == 1
    strrep( strBbarFile, '/', '\\' );
    strrep( strBhatFile, '/', '\\' );
    strrep( strMatFile, '/', '\\' );
    strrep( strOutputFile, '/', '\\' );
    strrep( strIfile, '/', '\\' );
end

% for debug -------------------------------------------
fprintf( '%s\n%s\n%s\n%s\n%s\n', strBbarFile, strBhatFile, strMatFile, strOutputFile, strIfile );


% call appropriate functions -----------------------------

fprintf('calculateBhat -------------------------------------------------\n');
t0 = clock;
     calculateBhat;
fprintf('   Path loss rate calculation completed in %f seconds\n', etime(clock, t0));
fprintf('Runpath terminated normally -------------------------------\n');

