/**
 *OBJECT:	This class is an abstract class
 *				which all shortest path algorithms
 *				inherit from.
 *
 * @version    $Id: ShortestPath.java,v 1.1 12/26/2001
 *
 * @author    Mark Rabin
 *
 * Revisions:
 *     $Log: ShortestPath.java,v 1.1$
 *     Revision 1.1  12/26/2001
 *     Initial revision
 *
 */


// package LossyLink;


public abstract class ShortestPath {

    double [] S;	//set of final shortest path weights
    int [] L;
    int [] pi;  //predecessor function
    //weight of the shortest path to the vertex last
    //extracted from the priority queue
    double minWeight;

    /**
	* This constructor performs the basic initialization of the
	* needed arrays.
	*
	* @param graph the graph in adjacency list form
   */
    public ShortestPath( Node [] graph )
    {
        S = new double[graph.length];
        for( int i = 0; i < graph.length; i++ )
        	S[i] = Double.MAX_VALUE;
        pi = new int[graph.length];
	L = new int[graph.length];
    }

	/**
	* This constructor runs the shortest path algorithm until
	* all nodes connected to the source have been searched.
	*
	* @param graph the graph in adjacency list form
   * @param source the source vertex in the graph
   */
//  public LinearDijkstra( Node [] graph, int source, LossyLinkPathInfo pathInfo, LossyLinkData data )
/*  public ShortestPath( Node [] graph, int source, LossyLinkPathInfo pathInfo, LossyLinkData data )
  {
      this( graph );
      initializeSingleSource( graph, source );

      int min = extractMin( );
      while( min != -1 ) {
	  S[min] = minWeight;
	  for( Node cursor = graph[min];
		  cursor.next != null; cursor = cursor.next )
	      relax( graph[min].id, cursor.next.id, cursor.edgeWeight );
	  min = extractMin( );
      }
  }
*/

    public ShortestPath( Node [] graph, int source )
    {
        this( graph );
        initializeSingleSource( graph, source );

        int min = extractMin( );
        while( min != -1 ) {
            S[min] = minWeight;
            for( Node cursor = graph[min];
            	cursor.next != null; cursor = cursor.next )
                relax( graph[min].id, cursor.next.id, cursor.edgeWeight );
            min = extractMin( );
        }
    }



    /**
	 * This method removes the minimum valued node form the shortest path
	 * estimate priority queue.  If the priority queue is empty, -1 is returned.
	 *
	 * @return the node number with the minimum shortest path estimate
	 */
    public abstract int extractMin( );


    /**
     * Initialize the shortest-path estimates and predecessor function.
     * The predecessor function is left at the default of all zeros.
     * Because there is no zero node in the graph, zero denotes a null
     * value for a predecessor.
     */
    public abstract void initializeSingleSource( Node [] graph, int source );

    /**
     * Test if shortest path to v can be improved by going through u, and
     * if so, updating d[v] and pi[v]. w is the weight from u to v.
     */
    //public abstract void relax( int u, int v, double w, int iLinkIndex );
    public abstract void relax( int u, int v, double w );


    /**
     * Print out the final shortest path weights and the
     * predecessor function.
     *
     * @return string representation of the final shortest paths
     */
    public String allVals() {
        String val = "";
        for( int i = 1; i < S.length; i++ )
            val +=  "S" + i + ":" + S[i] + "\n";
        for( int i = 1; i < pi.length; i++ )
           val += "pi" + i + ":" + pi[i] + "\n";
        return val;

	}

	/**
     * Print out the final shortest path weights
     *
     * @return string representation of the final shortest paths
     */
	public String toString() {
		  String val = "";
        for( int i = 1; i < S.length; i++ )
            val +=  "S" + i + ":" + S[i] + "\n";
        return val;
	}

   /**
   * Overridden equals method used for debugging.
   *
   * @param other the shortest path result to compare to
   *
   * @return true if the shortest path estimates are the same
   */
   public boolean equals( Object other ) {
   	return toString().equals( other.toString() );
   }



    public double getS( int iIndex )
    {
	if( S[iIndex] == 0.0f )
	    {
		return -1;
	    }
	if( S[iIndex] >= 1.7976931348623157E308 )
	    {
		return 0;
	    }
	return S[iIndex];
    }

    public int getL( int iIndex )
    {
	return L[iIndex];
    }

}
