% [V, R, e] = addpaths(V, R, Vi, tol);
%
% Extend the path set V with independent directions Vi(:,e).
% R = chol(V'*V) is extended during the computation.

function [V, R, e] = addpaths(V, R, Vi, tol);

k = size(V,2);

if isempty(V)

  Vp    = overlapm(Vi, Vi);
  e     = qrchoose(Vp, tol);

  k          = length(e);
  R(1:k,1:k) = chol(Vp(e,e));

else

  VTVi  = overlapm(V,  Vi);
  ViTVi = overlapm(Vi, Vi);
  C     = dtrsolve(R, k, 'U', 'T', 'N', VTVi);
  Vp    = ViTVi - C'*C; 
  e     = qrchoose(Vp, tol);

  knew                  = k + length(e);
  R(1:k,      k+1:knew) = C(:,e);
  R(k+1:knew, k+1:knew) = chol(Vp(e,e));
  k                     = knew;

end

V = [ V, Vi(:,e) ];

