package cluster;

import java.io.*;
import java.util.*;

public class ChosenMonitorClusteringMultipleMonitors extends Clustering implements Serializable
{  
    private RTTMatrix entire_matrix;

    private int clusterLimit;
    private int hosts;
    
    private Host[] monitors;
    private MinimalCover minimalCover;
    private RelativeErrorMatrix relativeErrorMatrix;
    private HostMap hostMap;

    private HashMap clusters;

    public ChosenMonitorClusteringMultipleMonitors(RTTMatrix entire_matrix, Host[] monitors,
						   String internalClusteringMethod,
						   double internalClusteringArg,
						   int internalClusteringLimit)
    {
	this.clusterLimit = clusterLimit;
	this.hosts = entire_matrix.getNumberOfHosts();
	this.entire_matrix = entire_matrix;
	this.hostMap = entire_matrix.getHostMap();
	this.monitors = monitors;

	clusters = new HashMap();
	findClusters();

	Cluster[] clusters_v = new Cluster[monitors.length];

	int k=0;
	Iterator iterator = clusters.keySet().iterator();
	while(iterator.hasNext())
	    clusters_v[k++] = (Cluster) clusters.get(iterator.next());

	minimalCover = new MinimalCover(clusters_v,entire_matrix,internalClusteringMethod,
					     internalClusteringArg,internalClusteringLimit);
	//System.out.println(minimalCover);
    }

    public MinimalCover getMinimalCover()
    {
	return minimalCover;
    }

    public RelativeErrorMatrix getRelativeErrorMatrix(String date, String intraEstimationMethod, boolean naive)
    {
	return new RelativeErrorMatrix(minimalCover,date,intraEstimationMethod,naive);
    }
 
    public String toString()
    {
	StringBuffer result = new StringBuffer();
	Iterator iterator = clusters.keySet().iterator();
	while(iterator.hasNext())
	    result.append(clusters.get(iterator.next()).toString()+"\n");

	return result.toString();
    }
		
    private void findClusters()
    {
	for(int i=0;i<monitors.length;i++)
	    clusters.put(monitors[i],new Cluster(monitors[i]));

	for(int i=0;i<hosts;i++)
	{
	    Host h = hostMap.getHost(i);
	    if(isMonitor(h))
		continue;
	    else
		determineCluster(h);
	}
    }

    private void determineCluster(Host h)
    {
	Distance[] d = entire_matrix.squareEuclidianDistance(h,monitors);

	double min = d[0].getDistance();
	Host candidate = monitors[0];

	for(int i=1;i<monitors.length;i++)
	{
	    double current = d[i].getDistance();
	    if(current < min)
	    {
		min = current;
		candidate = monitors[i];
	    }
	}

	Cluster destination = (Cluster) clusters.get(candidate);
	destination.add(h);
	clusters.put(candidate,destination);
    }

    private boolean isMonitor(Host h)
    {
	for(int i=0;i<monitors.length;i++)
	    if(monitors[i].equals(h))
		return true;
       
	return false;
    }

    private void writeObject(ObjectOutputStream out) throws IOException
    {
	out.writeObject(hostMap);
	out.writeObject(clusters);
	out.writeObject(minimalCover);
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
    {
	hostMap = (HostMap) in.readObject();
	clusters = (HashMap) in.readObject();
	minimalCover = (MinimalCover) in.readObject();
    }
}













