package cluster;

/**************************************************************
* Contains static methods for formatting double precison numbers.
***************************************************************/

public class Format
{
    /*****************************************************
     *Formats number with digitsAfterPoint digits after
     *the decimal point. Truncates any extra digits.
     *Does not advance to the next line after output.
     *****************************************************/
    public static String format(double number,
                                 int digitsAfterPoint)
    {
        StringBuffer answer = new StringBuffer();

        if(digitsAfterPoint <= 0)
            throw new IllegalArgumentException("Error. Requested accuracy <= 0.");

        if (number >= 0)
            answer.append(formatPositive(number, digitsAfterPoint));
        else
        {
            double positiveNumber = -number;
            answer.append('-');
            answer.append(formatPositive(positiveNumber, digitsAfterPoint));
        }

        return answer.toString();
    }

    //Precondition: number >= 0
    //Formats number with digitsAfterPoint digits after the
    //decimal point. Truncates any extra digits.
    private static String formatPositive(double number,
                                          int digitsAfterPoint)
    {
        StringBuffer answer = new StringBuffer();

        long mover = (long)(Math.pow(10, digitsAfterPoint));
                  //1 followed by digitsAfterPoint zeros
        long allWhole;//number with the decimal point
                     //moved digitsAfterPoint places
        allWhole = (long)(Math.floor(number*mover));
        long beforePoint = allWhole/mover;
        long afterPoint = allWhole%mover;

        answer.append(beforePoint);
        answer.append('.');
        answer.append(formatFraction(afterPoint, digitsAfterPoint));

        return answer.toString();
    }
    
    //Formats afterPoint with enough 0s
    //in front to make it digitsAfterPoint digits long.
    private static String formatFraction(long afterPoint,
                                              int digitsAfterPoint)
    {
        StringBuffer answer = new StringBuffer();

        int n = 1;
        while (n < digitsAfterPoint)
        {
            if (afterPoint < Math.pow(10, n))
                answer.append('0');
            n = n + 1;
        }
        answer.append(afterPoint);

        return answer.toString();
    }
}

