package cluster;

import java.io.*;
import java.util.*;

/* update the code to compute c/f
 */
public class LimitNumberProximityClustering extends Clustering implements Serializable
{

    private Vector[] vClusters;
    private Cluster[] clusters;
  
    private RTTMatrix performance_matrix;

    private double[][] euclidian_matrix;
 
    private int clusterLimit;
    private int hosts;
    
    private MinimalCover minimalCover;
    private RelativeErrorMatrix relativeErrorMatrix;
    private HostMap hostMap;

    public LimitNumberProximityClustering(RTTMatrix performance_matrix, int clusterLimit, 
				          String internalClusteringMethod,
					  double internalClusteringArg, int internalClusteringLimit)
    {
	this.clusterLimit = clusterLimit;
	this.hosts = performance_matrix.getNumberOfHosts();
	this.performance_matrix = performance_matrix;
	this.hostMap = performance_matrix.getHostMap();

	euclidian_matrix = new double[hosts][hosts];
	for(int i=0;i<hosts;i++)
	    for(int j=0;j<hosts;j++)
	    {
		if(i==j)
		    continue;
		double value = performance_matrix.get(hostMap.getHost(i),hostMap.getHost(j));
		if(value < 0.01)
		    euclidian_matrix[i][j] = 10000000;
	        else
		    euclidian_matrix[i][j] = value;
            }
  
	kCluster_limitNum(hosts);

	clusters = new Cluster[clusterLimit];

	for(int i=0;i<clusterLimit;i++)
	{
	    clusters[i] = new Cluster();
	    Vector c = vClusters[i];
	    Iterator iterator = c.iterator();
	    while(iterator.hasNext())
	    {
		Host h = hostMap.getHost(((Integer) iterator.next()).intValue());
		clusters[i].add(h);
	    }
	}
	findCenters();	
	minimalCover = new MinimalCover(clusters,performance_matrix,internalClusteringMethod,
					     internalClusteringArg,internalClusteringLimit);
    }

    public MinimalCover getMinimalCover()
    {
	return minimalCover;
    }

    public RelativeErrorMatrix getRelativeErrorMatrix(String date, String intraEstimationMethod, boolean naive)
    {
	return new RelativeErrorMatrix(minimalCover,date,intraEstimationMethod,naive);
    }
 
 	public RelativeErrorMatrixSW getRelativeErrorMatrixSW(String srcDate, String compareDate, String intraEstimationMethod, boolean naive)
   {
	   return new RelativeErrorMatrixSW(minimalCover,srcDate, compareDate,intraEstimationMethod,naive);
    }

    public String toString()
    {
	StringBuffer result = new StringBuffer();
	for(int i=0;i<clusters.length;i++)
	    result.append(clusters[i].toString()+"\n");

	return result.toString();
    }
    
    private void findCenters()
    {
	for(int i=0;i<clusters.length;i++)
	    clusters[i].minimizeAvgRTTToFindCenter(performance_matrix);
       
    }
		
    private void kCluster_limitNum(int hosts) {
	// init the B sets
	Vector[] B = new Vector[clusterLimit];
	for (int i = 0; i < clusterLimit; i++) {
	    B[i] = new Vector();
	}
	vClusters = B;
	
	// init B1 with all the hosts
	for (int i = 0; i < hosts; i++) {
	    B[0].insertElementAt(new Integer(i), i);
	}

	// assume Bi[0] will always be the head of the set
	// begin algorithm ; for l->1 to k-1 do
	for (int i = 0; i < (clusterLimit - 1); i++) {
	    Vector Bi = B[i];
	    double h = -1; //highest distance
	    int viCluster_limitNum = 0;
	    int vi = 1;           // init to something actual host number
	    int viPosition = 1;   // init to something position in the cluster
	    // h <- max(W(headj, vi))
	    
	    //int headi = ((Integer)Bi.elementAt(0)).intValue();//old code
	    //for (int j = 1; j < Bi.size(); j++) {//old code
	    
	    for (int j = 0; j <= i; j++) {
		Vector Bj = B[j];
		int headj = ((Integer)Bj.elementAt(0)).intValue();
		
		for(int k=1; k<Bj.size(); k++){
		    int viTemp = ((Integer)Bj.elementAt(k)).intValue();
		    if (euclidian_matrix[headj][viTemp] > h) {
			h = euclidian_matrix[headj][viTemp];
			vi = viTemp;
			viPosition = k;
			viCluster_limitNum=j;
		    }
		}
	    }
	    
	    if (h == -1) {
		break;
	    }
	    // move vi to B(l+1) ; make it the head
	    B[viCluster_limitNum].removeElementAt(viPosition);
	    B[(i + 1)].addElement(new Integer(vi));

	    
	    // for each vt...
	    for (int j = 0; j <= i; j++) {
		    Vector Bj = B[j];
		    int headj = ((Integer)Bj.elementAt(0)).intValue();

            Vector Bj_clone = (Vector)Bj.clone();
		    
            for (Enumeration e = Bj_clone.elements(); e.hasMoreElements();) {
                int vt = ((Integer)e.nextElement() ).intValue();
	    
		        if (euclidian_matrix[vt][headj] >= euclidian_matrix[vt][vi]) {
			        // move vt from Bj to B(l + 1)
			        Bj.remove(new Integer(vt));//newer version
			        B[(i + 1)].addElement(new Integer(vt));
    		    }
	    	}
	    }
	}
    }

    private void writeObject(ObjectOutputStream out) throws IOException
    {
	out.writeObject(hostMap);
	out.writeObject(clusters);
	out.writeObject(minimalCover);
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
    {
	hostMap = (HostMap) in.readObject();
	clusters = (Cluster[]) in.readObject();
	minimalCover = (MinimalCover) in.readObject();
    }
}



/*	for(int i=0;i<hosts;i++) {
	    for(int j=i+1;j<hosts;j++) {
		double sum = 0;
		//int comparable_dimension = 0;//what is this??xxx
		for(int k=0;k<hosts;k++) {
			  This part is only for WNMMS clustering, which u do not count 
				the distance in the two dimensions determined by two comparator themselves.
		    if(k==i || 
		       k==j || 
		       performance_matrix[i][k] == performance_matrix[j][k]) {
			continue;
		    }
			
		    sum = sum + java.lang.Math.pow((performance_matrix.get(i,k) - 
						    performance_matrix.get(j,k)),2);
		    //comparable_dimension++;//what is this??xxx
		}
		
		euclidean_matrix[i][j]=euclidean_matrix[j][i]
		    = java.lang.Math.pow(sum, .5);
		what is this??xxx
		if(sum < (comparable_dimension * tolerance * tolerance)) {
		    clusters[i].addElement(new Integer(j));
		}
		
	    }
	}
    }
*/










