package cluster;

import java.util.*;
import java.io.*;

public class MonitorRank implements Serializable
{
    private MonitorDistancePair[] monitorRanking;
    private Host host;

    public MonitorRank(double[][] euclidian_matrix, MinimalCover minimalCover,
		       HostMap hostMap, Host h)
    {
	HashSet monitors = minimalCover.getMonitors();
	monitorRanking = new MonitorDistancePair[monitors.size()];
	host = h;

	Iterator iterator = monitors.iterator();
	int count=0;

	while(iterator.hasNext())
	{
	    Host m = (Host) iterator.next();
	    double d = euclidian_matrix[hostMap.hash(h)][hostMap.hash(m)];
	    monitorRanking[count] = new MonitorDistancePair(m,d);
	    count++;
	}
	sortMonitors();
    }

    public Host getRank(int i)
    {
	if(i > monitorRanking.length)
	    throw new IllegalArgumentException("Out of bounds in monitor ranking.");

	return monitorRanking[i-1].monitor;
    }


    public double getNDistanceToRank(int i)
    {
	if(i > monitorRanking.length)
	    throw new IllegalArgumentException("Out of bounds in monitor ranking.");

	return monitorRanking[i-1].distance;
    }

    public String toString()
    {
	StringBuffer result = new StringBuffer("Ranking for "+host+":\n");
	for(int i=0;i<monitorRanking.length;i++)
	    result.append(monitorRanking[i].toString()+"\n");

	return result.toString();
    }

    private void sortMonitors()
    {
	for(int out=monitorRanking.length-1;out>0;out--)
	    for(int in=0;in<out;in++)
		if(monitorRanking[in].distance > monitorRanking[in+1].distance)
		    swap(in,in+1);
    }

    private void swap(int one, int two)
    {
	MonitorDistancePair temp = monitorRanking[one];
	monitorRanking[one] = monitorRanking[two];
	monitorRanking[two] = temp;
    }

    private class MonitorDistancePair implements Serializable
    {
	Host monitor;
	double distance;

	public MonitorDistancePair(Host monitor, double distance)
	{
	    this.monitor = monitor;
	    this.distance = distance;
	}

	public String toString()
	{
	    return monitor+"\t"+Format.format(distance,2);
	}
    }

}











