package cluster.stability;

import cluster.*;
import java.util.*;

public class FrequencyVector
{
    private HashMap frequency;
    private Host center;

    public FrequencyVector(ClusterVector[] days, Host center)
    {
	this.center = center; 
	frequency = new HashMap();
	for(int i=0;i<days.length;i++)
	{
	    Cluster current = days[i].getCluster(center);
	    Iterator iterator = current.iterator();
	    while(iterator.hasNext())
	    {
		Host site = (Host) iterator.next();
		Integer count = (Integer) frequency.get(site);
		if(count == null)
		    count = new Integer(1);
		else
		    count = new Integer(count.intValue()+1);

		frequency.put(site,count);
	    }
	}

	Iterator avgIterator = frequency.keySet().iterator();
	while(avgIterator.hasNext())
	{
	    Host site = (Host) avgIterator.next();
	    double f = ((Integer) frequency.get(site)).doubleValue();
	    frequency.put(site,new Double(f/days.length));
	}	
    }

    public String toString()
    {
	StringBuffer result = new StringBuffer();
	result.append("Mean vector for cluster centered at " + center + "\n");
	Iterator iterator = frequency.keySet().iterator();
	while(iterator.hasNext())
	{
	    Host site = (Host) iterator.next();
	    result.append(site + ": " +
                 Format.format(((Double) frequency.get(site)).doubleValue(),3) + "\n");
	}

	return result.toString();
    }

    public Set getHosts()
    {
	return frequency.keySet();
    }

    public double getStability(Cluster c)
    {
	double total = 0.0;

	Iterator iterator = c.iterator();
	while(iterator.hasNext())
	{
	    total += ((Double) frequency.get((Host) iterator.next())).doubleValue();
	}
	
	return total/c.size();
    }

    public double getFrequency(Host h)
    {
	if(frequency.containsKey(h))
	    return ((Double) frequency.get(h)).doubleValue();
	else
	    return 0.0;
    }

}











