% [I] = colselect(G)
%
% Remove clearly redundant columns from G (zero columns or columns which
% are identical to others).  The remaining columns are in G(:,I).

function [I] = colselect(G)

l      = full(sum(G));
lmax   = max(l);
unused = find(l == 0);

for i = 1:lmax

  % -- Find cases when two columns are identical.

  J       = find(l == i);   
  overlap = G(:,J)' * G(:,J);
  [j,k]   = find(overlap == i);

  % -- If distinct columns have identical patterns, 
  %    eliminate the later ones.

  p       = find(k > j);
  unused  = [unused, J(k(p))];

end

I = setdiff(1:length(l), unused);
