% [G, Gbar, R, kk] = deletepath(G, Gbar, R, k, param)
%
% Remove row k of Gbar.  Adds a replacement row kk of G 
% if necessary.  Updates G, Gbar, and R as needed.

function [G, Gbar, R, kk] = deletepath(G, Gbar, R, k, param)

n = size(Gbar,1);


% -- Find and remove the row in G
%    (relies on the fact that no path is a subpath of another)

l  = G*Gbar(k,:)';
kG = find(l == Gbar(k,:)*Gbar(k,:)');
G  = G([1:kG-1, kG+1:end], :);


% -- Search for a replacement row

ek    = zeros(n, 1);
ek(k) = 1;
y     = compute_xG(Gbar, R, ek, param);
r     = abs(G*y);

kk = find(r == max(r));
kk = kk(1);
if r(kk) < param.tol
  kk = [];
end


% -- Actually delete the path

% Remove a column from R

Gbar          = Gbar([1:k-1,k+1:n], :);
R(1:n, k:n-1) = R(1:n, k+1:n);
R(1:n, n)     = 0;

% Zero out the subdiagonal R entries
for j = k:n-1
  GG              = givens(R(j,j), R(j+1,j));
  R([j,j+1], j:n) = GG*R([j,j+1], j:n);
  R(j+1,j)        = 0;
end


% -- And add the replacement as needed

if ~isempty(kk)
  [Gbar, R] = addpath(Gbar, R, G(kk,:), param);
end












