#include <mex.h>

/* 
 * function X = dtrsolve(A, n, uplo, trans, diag, B)
 *
 * Solve triangular systems involving A(1:n,1:n)
 */

/*
 * SUBROUTINE DTRTRS( UPLO, TRANS, DIAG, N, NRHS, A, LDA,  B,
 *                    LDB, INFO )
 */
void dtrtrs_(char* uplo, char* trans, char* diag,
             int* n, int* nrhs,
             double* a, int* lda,
             double* b, int* ldb,
             int* info);

void mexFunction(int nlhs, mxArray** plhs,
                 int nrhs, const mxArray** prhs)
{
    int n, brhs, ldA, info;
    double* A;
    double* B;
    char uplo [16];
    char trans[16];
    char diag [16];

    if (nrhs != 6)
        mxErrMsgTxt("Too few arguments");

    if (!mxIsNumeric(prhs[0]) || mxGetM(prhs[0]) != mxGetN(prhs[0]))
        mxErrMsgTxt("A must be a square matrix");
    A   = mxGetPr(prhs[0]);
    ldA = mxGetM(prhs[0]);

    if (!mxIsNumeric(prhs[1]) || mxGetM(prhs[1]) != mxGetN(prhs[1]))
        mxErrMsgTxt("k must be a scalar");
    n = (int) *mxGetPr(prhs[1]);
    if (n < 0 || n > ldA)
        mxErrMsgTxt("n is out of bounds");

    if (!mxIsChar(prhs[2]))
        mxErrMsgTxt("uplo must be a string");
    mxGetString(prhs[2], uplo, 16);

    if (!mxIsChar(prhs[3]))
        mxErrMsgTxt("trans must be a string");
    mxGetString(prhs[3], trans, 16);

    if (!mxIsChar(prhs[4]))
        mxErrMsgTxt("diag must be a string");
    mxGetString(prhs[4], diag, 16);

    if (!mxIsNumeric(prhs[5]) || mxGetM(prhs[5]) != n)
        mxErrMsgTxt("B must be a matrix matching A");
    brhs = mxGetN(prhs[5]);

    plhs[0] = mxCreateDoubleMatrix(n, brhs, mxREAL);
    B       = mxGetPr(plhs[0]);
    memcpy(B, mxGetPr(prhs[5]), n*brhs*sizeof(double));
    dtrtrs_(uplo, trans, diag, &n, &brhs, A, &ldA, B, &n, &info);

    plhs[1] = mxCreateDoubleMatrix(1,1, mxREAL);
    *mxGetPr(plhs[1]) = info;
}
