#include <mex.h>

/*
 * function [VTU] = overlap(V,U)
 * 
 * Compute VTU = V'*U for 0-1 matrices
 *
 */


/* Compute overlap for two path vectors
 *  p1 -- l1 edge numbers in sorted order
 *  p2 -- l2 edge numbers in sorted order
 */
int path_overlap(int* p1, int l1, 
                 int* p2, int l2)
{
    int i1      = 0;
    int i2      = 0;
    int overlap = 0;

    while (i1 < l1 && i2 < l2) {
        if (p1[i1] < p2[i2]) {
            ++i1;
        } else if (p1[i1] > p2[i2]) {
            ++i2;
        } else {
            ++overlap;
            ++i1;
            ++i2;
        }
    }

    return overlap;
}


void mexFunction(int nlhs, mxArray** plhs,
                 int nrhs, const mxArray** prhs)
{
    int i, j, m, n;
    double* overlap;
    int* jcU;
    int* jcV;
    int* irU;
    int* irV;

    /* Process arguments */

    if (nrhs != 2)
        mexErrMsgTxt("Wrong number of arguments");

    if (!mxIsSparse(prhs[0]))
        mexErrMsgTxt("U must be sparse");
    if (!mxIsSparse(prhs[1]))
        mexErrMsgTxt("V must be sparse");

    m = mxGetN(prhs[0]);
    n = mxGetN(prhs[1]);

    irU = mxGetIr(prhs[0]);
    jcU = mxGetJc(prhs[0]);
    irV = mxGetIr(prhs[1]);
    jcV = mxGetJc(prhs[1]);

    plhs[0] = mxCreateDoubleMatrix(m,n, mxREAL);
    overlap = mxGetPr(plhs[0]);
    for (j = 0; j < n; ++j) {
        for (i = 0; i < m; ++i) {
            overlap[j*m+i] = path_overlap(irU + jcU[i], jcU[i+1] - jcU[i],
                                          irV + jcV[j], jcV[j+1] - jcV[j]);
        }
    }
}
