% [G] = parse_graph(fname)
%
% Build a structure G to represent the BRITE topology file.

function [G] = parse_graph(fname)

fid = fopen(fname, 'r');

% Scan to node line
l = fgetl(fid);
while ~strncmp(l, 'Node', 4)
  l = fgetl(fid);
end

% Get node count
num_nodes = strread(l, 'Nodes: ( %d )');

% Read node lines
nodeOffset = 0;
linkOffset = 0;
for k = 1:num_nodes
  l = fgetl(fid);

  if k == 1
     [v(k).id, ...
      v(k).x(1), ...
      v(k).x(2), ...
      v(k).ind,  ...
      v(k).outd, ...
      v(k).asid, ...
      v(k).type] = ...
     strread(l, '%d %f %f %d %d %d %s');
     nodeOffset = v(k).id;
     v(k).id = 0;
  else
     [v(k).id, ...
      v(k).x(1), ...
      v(k).x(2), ...
      v(k).ind,  ...
      v(k).outd, ...
      v(k).asid, ...
      v(k).type] = ...
     strread(l, '%d %f %f %d %d %d %s');
  end
  
  v(k).x(1) = v(k).x(1) - nodeOffset;
  v(k).x(2) = v(k).x(2) - nodeOffset;
end

% Scan to edge line
l = fgetl(fid);
while ~strncmp(l, 'Edge', 4)
  l = fgetl(fid);
end

% Edges: ( numedges )
% id from to length delay bw asfrom asto type
% Get node count
num_edges = strread(l, 'Edges: ( %d )');

% Read edge lines
for k = 1:num_edges
  l = fgetl(fid);

  if k == 1
     [e(k).id,     ...
      e(k).from,   ...
      e(k).to,     ...
      e(k).length, ... 
      e(k).delay,  ...
      e(k).bw,     ...
      e(k).asfrom, ...
      e(k).asto,   ...
      e(k).type] = ...
     strread(l, '%d %d %d %f %f %f %d %d %s U');
     linkOffset = e(k).id;
     e(k).id = 0;
  else
     [e(k).id,     ...
      e(k).from,   ...
      e(k).to,     ...
      e(k).length, ... 
      e(k).delay,  ...
      e(k).bw,     ...
      e(k).asfrom, ...
      e(k).asto,   ...
      e(k).type] = ...
     strread(l, '%d %d %d %f %f %f %d %d %s U');
  end
  
  e(k).from = e(k).from - nodeOffset;
  e(k).to = e(k).to - nodeOffset;
end

G.v = v;
G.e = e;
G.nv = num_nodes;
G.ne = num_edges;








