% [G] = pathmatrix(nodes, cost, labels, verbose)
%
% Generate the path matrix.  Each row represents a path through
% between two end nodes; the jth entry is zero if the edge with
% label j is unused, and one if it is used.
%
% Inputs:
%   nodes   -- list of connected end nodes
%   cost    -- cost(i,j)   = length of edge from i to j, zero if none
%   labels  -- labels(i,j) = ID of edge from i to j, zero if none
%   verbose -- output status during computation (default false)

function [G] = pathmatrix(nodes, cost, labels, verbose)

if nargin < 4, verbose = 1; end  % Default is non-verbose

ne = full(max(max(labels)));  % Num edges
nv = length(nodes);           % Num vertices
V  = [];

for i = 1:nv-1
  if verbose, fprintf('\rProcessing end node %d ', i); end
  [d, p] = dijkstra(cost, nodes(i));
  V      = [V, pathv(nodes(i+1:nv), p, labels, ne)];
end

if verbose, fprintf('\n'); end
G = V';
